/* Generated by Cython 0.29.26 */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_26"
#define CYTHON_HEX_VERSION 0x001D1AF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030B00A1)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ssh__libssh_enums
#define __PYX_HAVE_API__ssh__libssh_enums
/* Early includes */
#include <stddef.h>
#include <time.h>
#include <sys/types.h>
#include "libssh/libssh.h"
#include "fcntl.h"
#include "libssh/sftp.h"
#include "libssh/ssh2.h"
#include "libssh/callbacks.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "ssh/libssh_enums.pyx",
};

/*--- Type declarations ---*/

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_kex_types_e(enum ssh_kex_types_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_auth_e(enum ssh_auth_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_requests_e(enum ssh_requests_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_channel_type_e(enum ssh_channel_type_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_channel_requests_e(enum ssh_channel_requests_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_global_requests_e(enum ssh_global_requests_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_publickey_state_e(enum ssh_publickey_state_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_server_known_e(enum ssh_server_known_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_error_types_e(enum ssh_error_types_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_keytypes_e(enum ssh_keytypes_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_keycmp_e(enum ssh_keycmp_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_options_e(enum ssh_options_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_scp_request_types(enum ssh_scp_request_types value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_connector_flags_e(enum ssh_connector_flags_e value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'ssh' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'libc.time' */

/* Module declarations from 'posix.types' */

/* Module declarations from 'ssh.c_ssh' */

/* Module declarations from 'ssh.c_sftp' */

/* Module declarations from 'ssh.c_ssh2' */

/* Module declarations from 'ssh.c_callbacks' */

/* Module declarations from 'ssh.libssh_enums' */
#define __Pyx_MODULE_NAME "ssh.libssh_enums"
extern int __pyx_module_is_main_ssh__libssh_enums;
int __pyx_module_is_main_ssh__libssh_enums = 0;

/* Implementation of 'ssh.libssh_enums' */
static const char __pyx_k_FD[] = "FD";
static const char __pyx_k_OK[] = "OK";
static const char __pyx_k_ACL[] = "ACL";
static const char __pyx_k_DSS[] = "DSS";
static const char __pyx_k_ENV[] = "ENV";
static const char __pyx_k_EOF[] = "EOF";
static const char __pyx_k_KEX[] = "KEX";
static const char __pyx_k_Kex[] = "Kex";
static const char __pyx_k_Log[] = "Log";
static const char __pyx_k_MAC[] = "MAC";
static const char __pyx_k_PTY[] = "PTY";
static const char __pyx_k_RSA[] = "RSA";
static const char __pyx_k_SCP[] = "SCP";
static const char __pyx_k_SSH[] = "SSH";
static const char __pyx_k_X11[] = "X11";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_AUTH[] = "AUTH";
static const char __pyx_k_Auth[] = "Auth";
static const char __pyx_k_BOTH[] = "BOTH";
static const char __pyx_k_COMP[] = "COMP";
static const char __pyx_k_DATA[] = "DATA";
static const char __pyx_k_EXCL[] = "EXCL";
static const char __pyx_k_EXEC[] = "EXEC";
static const char __pyx_k_Enum[] = "Enum";
static const char __pyx_k_HOST[] = "HOST";
static const char __pyx_k_IFMT[] = "IFMT";
static const char __pyx_k_INFO[] = "INFO";
static const char __pyx_k_INIT[] = "INIT";
static const char __pyx_k_LANG[] = "LANG";
static const char __pyx_k_NAME[] = "NAME";
static const char __pyx_k_NONE[] = "NONE";
static const char __pyx_k_OPEN[] = "OPEN";
static const char __pyx_k_PORT[] = "PORT";
static const char __pyx_k_RARE[] = "RARE";
static const char __pyx_k_READ[] = "READ";
static const char __pyx_k_RSA1[] = "RSA1";
static const char __pyx_k_SFTP[] = "SFTP";
static const char __pyx_k_SIZE[] = "SIZE";
static const char __pyx_k_SSH1[] = "SSH1";
static const char __pyx_k_SSH2[] = "SSH2";
static const char __pyx_k_STAT[] = "STAT";
static const char __pyx_k_TEXT[] = "TEXT";
static const char __pyx_k_USER[] = "USER";
static const char __pyx_k_WARN[] = "WARN";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_AGAIN[] = "AGAIN";
static const char __pyx_k_ATTRS[] = "ATTRS";
static const char __pyx_k_CLOSE[] = "CLOSE";
static const char __pyx_k_CREAT[] = "CREAT";
static const char __pyx_k_CRYPT[] = "CRYPT";
static const char __pyx_k_DEBUG[] = "DEBUG";
static const char __pyx_k_ECDSA[] = "ECDSA";
static const char __pyx_k_EINTR[] = "EINTR";
static const char __pyx_k_ERROR[] = "ERROR";
static const char __pyx_k_FATAL[] = "FATAL";
static const char __pyx_k_FSTAT[] = "FSTAT";
static const char __pyx_k_IFBLK[] = "IFBLK";
static const char __pyx_k_IFCHR[] = "IFCHR";
static const char __pyx_k_IFDIR[] = "IFDIR";
static const char __pyx_k_IFIFO[] = "IFIFO";
static const char __pyx_k_IFLNK[] = "IFLNK";
static const char __pyx_k_IFREG[] = "IFREG";
static const char __pyx_k_LSTAT[] = "LSTAT";
static const char __pyx_k_MKDIR[] = "MKDIR";
static const char __pyx_k_NOLOG[] = "NOLOG";
static const char __pyx_k_PK_OK[] = "PK_OK";
static const char __pyx_k_RMDIR[] = "RMDIR";
static const char __pyx_k_SHELL[] = "SHELL";
static const char __pyx_k_TRACE[] = "TRACE";
static const char __pyx_k_TRUNC[] = "TRUNC";
static const char __pyx_k_VALID[] = "VALID";
static const char __pyx_k_WRITE[] = "WRITE";
static const char __pyx_k_WRONG[] = "WRONG";
static const char __pyx_k_APPEND[] = "APPEND";
static const char __pyx_k_ATOMIC[] = "ATOMIC";
static const char __pyx_k_BANNER[] = "BANNER";
static const char __pyx_k_CLOSED[] = "CLOSED";
static const char __pyx_k_DENIED[] = "DENIED";
static const char __pyx_k_ENDDIR[] = "ENDDIR";
static const char __pyx_k_GLOBAL[] = "GLOBAL";
static const char __pyx_k_HANDLE[] = "HANDLE";
static const char __pyx_k_IFSOCK[] = "IFSOCK";
static const char __pyx_k_IGNORE[] = "IGNORE";
static const char __pyx_k_KeyCMP[] = "KeyCMP";
static const char __pyx_k_NATIVE[] = "NATIVE";
static const char __pyx_k_NEWDIR[] = "NEWDIR";
static const char __pyx_k_O_EXCL[] = "O_EXCL";
static const char __pyx_k_O_RDWR[] = "O_RDWR";
static const char __pyx_k_PACKET[] = "PACKET";
static const char __pyx_k_PUBLIC[] = "PUBLIC";
static const char __pyx_k_REMOVE[] = "REMOVE";
static const char __pyx_k_RENAME[] = "RENAME";
static const char __pyx_k_SFTP_S[] = "SFTP_S";
static const char __pyx_k_STATUS[] = "STATUS";
static const char __pyx_k_STDERR[] = "STDERR";
static const char __pyx_k_STDOUT[] = "STDOUT";
static const char __pyx_k_Server[] = "Server";
static const char __pyx_k_UIDGID[] = "UIDGID";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_CHANNEL[] = "CHANNEL";
static const char __pyx_k_Channel[] = "Channel";
static const char __pyx_k_ED25519[] = "ED25519";
static const char __pyx_k_FAILURE[] = "FAILURE";
static const char __pyx_k_KEXINIT[] = "KEXINIT";
static const char __pyx_k_KeyType[] = "KeyType";
static const char __pyx_k_MAC_C_S[] = "MAC_C_S";
static const char __pyx_k_MAC_S_C[] = "MAC_S_C";
static const char __pyx_k_NEWFILE[] = "NEWFILE";
static const char __pyx_k_NEWKEYS[] = "NEWKEYS";
static const char __pyx_k_NODELAY[] = "NODELAY";
static const char __pyx_k_OPENDIR[] = "OPENDIR";
static const char __pyx_k_O_CREAT[] = "O_CREAT";
static const char __pyx_k_O_TRUNC[] = "O_TRUNC";
static const char __pyx_k_Options[] = "Options";
static const char __pyx_k_PARTIAL[] = "PARTIAL";
static const char __pyx_k_PRIVATE[] = "PRIVATE";
static const char __pyx_k_READDIR[] = "READDIR";
static const char __pyx_k_REGULAR[] = "REGULAR";
static const char __pyx_k_REQUEST[] = "REQUEST";
static const char __pyx_k_Request[] = "Request";
static const char __pyx_k_SERVICE[] = "SERVICE";
static const char __pyx_k_SESSION[] = "SESSION";
static const char __pyx_k_SETSTAT[] = "SETSTAT";
static const char __pyx_k_SFTP_AT[] = "SFTP_AT";
static const char __pyx_k_SFTP_FX[] = "SFTP_FX";
static const char __pyx_k_SPECIAL[] = "SPECIAL";
static const char __pyx_k_SSH_DIR[] = "SSH_DIR";
static const char __pyx_k_SUCCESS[] = "SUCCESS";
static const char __pyx_k_SYMLINK[] = "SYMLINK";
static const char __pyx_k_TIMEOUT[] = "TIMEOUT";
static const char __pyx_k_UNKNOWN[] = "UNKNOWN";
static const char __pyx_k_VERSION[] = "VERSION";
static const char __pyx_k_WARNING[] = "WARNING";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_BINDADDR[] = "BINDADDR";
static const char __pyx_k_COMP_C_S[] = "COMP_C_S";
static const char __pyx_k_COMP_S_C[] = "COMP_S_C";
static const char __pyx_k_EXTENDED[] = "EXTENDED";
static const char __pyx_k_FSETSTAT[] = "FSETSTAT";
static const char __pyx_k_HMAC_C_S[] = "HMAC_C_S";
static const char __pyx_k_HMAC_S_C[] = "HMAC_S_C";
static const char __pyx_k_HOSTKEYS[] = "HOSTKEYS";
static const char __pyx_k_IDENTITY[] = "IDENTITY";
static const char __pyx_k_KNOWN_OK[] = "KNOWN_OK";
static const char __pyx_k_LANG_C_S[] = "LANG_C_S";
static const char __pyx_k_LANG_S_C[] = "LANG_S_C";
static const char __pyx_k_NO_ERROR[] = "NO_ERROR";
static const char __pyx_k_NO_MEDIA[] = "NO_MEDIA";
static const char __pyx_k_O_NOCTTY[] = "O_NOCTTY";
static const char __pyx_k_O_RDONLY[] = "O_RDONLY";
static const char __pyx_k_O_WRONLY[] = "O_WRONLY";
static const char __pyx_k_PASSWORD[] = "PASSWORD";
static const char __pyx_k_PORT_STR[] = "PORT_STR";
static const char __pyx_k_PROTOCOL[] = "PROTOCOL";
static const char __pyx_k_READLINK[] = "READLINK";
static const char __pyx_k_REALPATH[] = "REALPATH";
static const char __pyx_k_RESERVED[] = "RESERVED";
static const char __pyx_k_SFTP_FXE[] = "SFTP_FXE";
static const char __pyx_k_SFTP_FXF[] = "SFTP_FXF";
static const char __pyx_k_SFTP_FXP[] = "SFTP_FXP";
static const char __pyx_k_SSH2_Msg[] = "SSH2_Msg";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_ACMODTIME[] = "ACMODTIME";
static const char __pyx_k_CRYPT_C_S[] = "CRYPT_C_S";
static const char __pyx_k_CRYPT_S_C[] = "CRYPT_S_C";
static const char __pyx_k_Connector[] = "Connector";
static const char __pyx_k_DIRECTORY[] = "DIRECTORY";
static const char __pyx_k_ECDH_INIT[] = "ECDH_INIT";
static const char __pyx_k_FUNCTIONS[] = "FUNCTIONS";
static const char __pyx_k_HOSTBASED[] = "HOSTBASED";
static const char __pyx_k_KEEPALIVE[] = "KEEPALIVE";
static const char __pyx_k_MAC_ERROR[] = "MAC_ERROR";
static const char __pyx_k_NOT_KNOWN[] = "NOT_KNOWN";
static const char __pyx_k_OVERWRITE[] = "OVERWRITE";
static const char __pyx_k_PUBLICKEY[] = "PUBLICKEY";
static const char __pyx_k_RECURSIVE[] = "RECURSIVE";
static const char __pyx_k_SSH2_Open[] = "SSH2_Open";
static const char __pyx_k_SUBSYSTEM[] = "SUBSYSTEM";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_ACCESSTIME[] = "ACCESSTIME";
static const char __pyx_k_AUTH_AGENT[] = "AUTH_AGENT";
static const char __pyx_k_CREATETIME[] = "CREATETIME";
static const char __pyx_k_DISCONNECT[] = "DISCONNECT";
static const char __pyx_k_DSS_CERT01[] = "DSS_CERT01";
static const char __pyx_k_ECDH_REPLY[] = "ECDH_REPLY";
static const char __pyx_k_ECDSA_P256[] = "ECDSA_P256";
static const char __pyx_k_ECDSA_P384[] = "ECDSA_P384";
static const char __pyx_k_ECDSA_P521[] = "ECDSA_P521";
static const char __pyx_k_ECMQV_INIT[] = "ECMQV_INIT";
static const char __pyx_k_ErrorTypes[] = "ErrorTypes";
static const char __pyx_k_GSSAPI_MIC[] = "GSSAPI_MIC";
static const char __pyx_k_KEXDH_INIT[] = "KEXDH_INIT";
static const char __pyx_k_KNOWNHOSTS[] = "KNOWNHOSTS";
static const char __pyx_k_MODIFYTIME[] = "MODIFYTIME";
static const char __pyx_k_OWNERGROUP[] = "OWNERGROUP";
static const char __pyx_k_O_NOFOLLOW[] = "O_NOFOLLOW";
static const char __pyx_k_RSA_CERT01[] = "RSA_CERT01";
static const char __pyx_k_Auth_Method[] = "Auth_Method";
static const char __pyx_k_BAD_MESSAGE[] = "BAD_MESSAGE";
static const char __pyx_k_CIPHERS_C_S[] = "CIPHERS_C_S";
static const char __pyx_k_CIPHERS_S_C[] = "CIPHERS_S_C";
static const char __pyx_k_COMPRESSION[] = "COMPRESSION";
static const char __pyx_k_DH_GEX_INIT[] = "DH_GEX_INIT";
static const char __pyx_k_ECMQV_REPLY[] = "ECMQV_REPLY";
static const char __pyx_k_FOUND_OTHER[] = "FOUND_OTHER";
static const char __pyx_k_GSSAPI_AUTH[] = "GSSAPI_AUTH";
static const char __pyx_k_INTERACTIVE[] = "INTERACTIVE";
static const char __pyx_k_KBDINT_AUTH[] = "KBDINT_AUTH";
static const char __pyx_k_KEXDH_REPLY[] = "KEXDH_REPLY";
static const char __pyx_k_PERMISSIONS[] = "PERMISSIONS";
static const char __pyx_k_PUBKEY_AUTH[] = "PUBKEY_AUTH";
static const char __pyx_k_SCP_Request[] = "SCP_Request";
static const char __pyx_k_ADD_IDENTITY[] = "ADD_IDENTITY";
static const char __pyx_k_CHANNEL_OPEN[] = "CHANNEL_OPEN";
static const char __pyx_k_CLOSED_ERROR[] = "CLOSED_ERROR";
static const char __pyx_k_CONNECTED_OK[] = "CONNECTED_OK";
static const char __pyx_k_DH_GEX_GROUP[] = "DH_GEX_GROUP";
static const char __pyx_k_DH_GEX_REPLY[] = "DH_GEX_REPLY";
static const char __pyx_k_DIRECT_TCPIP[] = "DIRECT_TCPIP";
static const char __pyx_k_GSSAPI_ERROR[] = "GSSAPI_ERROR";
static const char __pyx_k_GSSAPI_TOKEN[] = "GSSAPI_TOKEN";
static const char __pyx_k_INFO_REQUEST[] = "INFO_REQUEST";
static const char __pyx_k_KEY_EXCHANGE[] = "KEY_EXCHANGE";
static const char __pyx_k_NO_SUCH_FILE[] = "NO_SUCH_FILE";
static const char __pyx_k_NO_SUCH_PATH[] = "NO_SUCH_PATH";
static const char __pyx_k_OPEN_FAILURE[] = "OPEN_FAILURE";
static const char __pyx_k_PROXYCOMMAND[] = "PROXYCOMMAND";
static const char __pyx_k_READ_PENDING[] = "READ_PENDING";
static const char __pyx_k_SSH2_Msg_Kex[] = "SSH2_Msg_Kex";
static const char __pyx_k_TIMEOUT_USEC[] = "TIMEOUT_USEC";
static const char __pyx_k_EXCEPTION_EOF[] = "EXCEPTION_EOF";
static const char __pyx_k_EXTENDED_DATA[] = "EXTENDED_DATA";
static const char __pyx_k_GSSAPI_ERRTOK[] = "GSSAPI_ERRTOK";
static const char __pyx_k_INFO_RESPONSE[] = "INFO_RESPONSE";
static const char __pyx_k_KNOWN_CHANGED[] = "KNOWN_CHANGED";
static const char __pyx_k_LOG_VERBOSITY[] = "LOG_VERBOSITY";
static const char __pyx_k_NO_CONNECTION[] = "NO_CONNECTION";
static const char __pyx_k_PASSWORD_AUTH[] = "PASSWORD_AUTH";
static const char __pyx_k_TCPIP_FORWARD[] = "TCPIP_FORWARD";
static const char __pyx_k_UNIMPLEMENTED[] = "UNIMPLEMENTED";
static const char __pyx_k_WINDOW_ADJUST[] = "WINDOW_ADJUST";
static const char __pyx_k_WINDOW_CHANGE[] = "WINDOW_CHANGE";
static const char __pyx_k_WRITE_PENDING[] = "WRITE_PENDING";
static const char __pyx_k_WRITE_PROTECT[] = "WRITE_PROTECT";
static const char __pyx_k_BY_APPLICATION[] = "BY_APPLICATION";
static const char __pyx_k_CONNECT_FAILED[] = "CONNECT_FAILED";
static const char __pyx_k_DH_GEX_REQUEST[] = "DH_GEX_REQUEST";
static const char __pyx_k_ED25519_CERT01[] = "ED25519_CERT01";
static const char __pyx_k_EXTENDED_REPLY[] = "EXTENDED_REPLY";
static const char __pyx_k_FILE_NOT_FOUND[] = "FILE_NOT_FOUND";
static const char __pyx_k_Global_Request[] = "Global_Request";
static const char __pyx_k_INVALID_HANDLE[] = "INVALID_HANDLE";
static const char __pyx_k_INVALID_SOCKET[] = "INVALID_SOCKET";
static const char __pyx_k_MD5_DIGEST_LEN[] = "MD5_DIGEST_LEN";
static const char __pyx_k_OP_UNSUPPORTED[] = "OP_UNSUPPORTED";
static const char __pyx_k_PROTOCOL_ERROR[] = "PROTOCOL_ERROR";
static const char __pyx_k_PublicKeyState[] = "PublicKeyState";
static const char __pyx_k_REQUEST_DENIED[] = "REQUEST_DENIED";
static const char __pyx_k_SERVICE_ACCEPT[] = "SERVICE_ACCEPT";
static const char __pyx_k_COMPRESSION_C_S[] = "COMPRESSION_C_S";
static const char __pyx_k_COMPRESSION_S_C[] = "COMPRESSION_S_C";
static const char __pyx_k_CONNECTED_ERROR[] = "CONNECTED_ERROR";
static const char __pyx_k_CONNECTION_LOST[] = "CONNECTION_LOST";
static const char __pyx_k_Channel_Request[] = "Channel_Request";
static const char __pyx_k_EXCEPTION_ERROR[] = "EXCEPTION_ERROR";
static const char __pyx_k_FORWARDED_TCPIP[] = "FORWARDED_TCPIP";
static const char __pyx_k_GSSAPI_RESPONSE[] = "GSSAPI_RESPONSE";
static const char __pyx_k_SERVICE_REQUEST[] = "SERVICE_REQUEST";
static const char __pyx_k_SFTP_FXF_Rename[] = "SFTP_FXF_Rename";
static const char __pyx_k_SSH2_Disconnect[] = "SSH2_Disconnect";
static const char __pyx_k_SSH2_Msg_Global[] = "SSH2_Msg_Global";
static const char __pyx_k_SUBSECOND_TIMES[] = "SUBSECOND_TIMES";
static const char __pyx_k_Callbacks_Socket[] = "Callbacks_Socket";
static const char __pyx_k_PASSWD_CHANGEREQ[] = "PASSWD_CHANGEREQ";
static const char __pyx_k_SSH2_Msg_Channel[] = "SSH2_Msg_Channel";
static const char __pyx_k_SSH2_Msg_Request[] = "SSH2_Msg_Request";
static const char __pyx_k_ssh_libssh_enums[] = "ssh.libssh_enums";
static const char __pyx_k_COMPRESSION_ERROR[] = "COMPRESSION_ERROR";
static const char __pyx_k_COMPRESSION_LEVEL[] = "COMPRESSION_LEVEL";
static const char __pyx_k_CONNECTED_TIMEOUT[] = "CONNECTED_TIMEOUT";
static const char __pyx_k_ECDSA_P256_CERT01[] = "ECDSA_P256_CERT01";
static const char __pyx_k_ECDSA_P384_CERT01[] = "ECDSA_P384_CERT01";
static const char __pyx_k_ECDSA_P521_CERT01[] = "ECDSA_P521_CERT01";
static const char __pyx_k_GLOBAL_KNOWNHOSTS[] = "GLOBAL_KNOWNHOSTS";
static const char __pyx_k_ILLEGAL_USER_NAME[] = "ILLEGAL_USER_NAME";
static const char __pyx_k_LOG_VERBOSITY_STR[] = "LOG_VERBOSITY_STR";
static const char __pyx_k_OPEN_CONFIRMATION[] = "OPEN_CONFIRMATION";
static const char __pyx_k_PERMISSION_DENIED[] = "PERMISSION_DENIED";
static const char __pyx_k_RESOURCE_SHORTAGE[] = "RESOURCE_SHORTAGE";
static const char __pyx_k_SSH2_ExtendedData[] = "SSH2_ExtendedData";
static const char __pyx_k_SSH2_Msg_UserAuth[] = "SSH2_Msg_UserAuth";
static const char __pyx_k_STATVFS_ST_NOSUID[] = "STATVFS_ST_NOSUID";
static const char __pyx_k_STATVFS_ST_RDONLY[] = "STATVFS_ST_RDONLY";
static const char __pyx_k_DH_GEX_REQUEST_OLD[] = "DH_GEX_REQUEST_OLD";
static const char __pyx_k_SFTP_FileXfer_Attr[] = "SFTP_FileXfer_Attr";
static const char __pyx_k_SFTP_FileXfer_Type[] = "SFTP_FileXfer_Type";
static const char __pyx_k_STRICTHOSTKEYCHECK[] = "STRICTHOSTKEYCHECK";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_FILE_ALREADY_EXISTS[] = "FILE_ALREADY_EXISTS";
static const char __pyx_k_FLOW_WRITEWILLBLOCK[] = "FLOW_WRITEWILLBLOCK";
static const char __pyx_k_FLOW_WRITEWONTBLOCK[] = "FLOW_WRITEWONTBLOCK";
static const char __pyx_k_KEY_EXCHANGE_FAILED[] = "KEY_EXCHANGE_FAILED";
static const char __pyx_k_CANCEL_TCPIP_FORWARD[] = "CANCEL_TCPIP_FORWARD";
static const char __pyx_k_TOO_MANY_CONNECTIONS[] = "TOO_MANY_CONNECTIONS";
static const char __pyx_k_UNKNOWN_CHANNEL_TYPE[] = "UNKNOWN_CHANNEL_TYPE";
static const char __pyx_k_SERVICE_NOT_AVAILABLE[] = "SERVICE_NOT_AVAILABLE";
static const char __pyx_k_AUTH_CANCELLED_BY_USER[] = "AUTH_CANCELLED_BY_USER";
static const char __pyx_k_GSSAPI_CLIENT_IDENTITY[] = "GSSAPI_CLIENT_IDENTITY";
static const char __pyx_k_GSSAPI_SERVER_IDENTITY[] = "GSSAPI_SERVER_IDENTITY";
static const char __pyx_k_HOST_KEY_NOT_VERIFIABLE[] = "HOST_KEY_NOT_VERIFIABLE";
static const char __pyx_k_GSSAPI_EXCHANGE_COMPLETE[] = "GSSAPI_EXCHANGE_COMPLETE";
static const char __pyx_k_HOST_AUTHENTICATION_FAILED[] = "HOST_AUTHENTICATION_FAILED";
static const char __pyx_k_ADMINISTRATIVELY_PROHIBITED[] = "ADMINISTRATIVELY_PROHIBITED";
static const char __pyx_k_GSSAPI_DELEGATE_CREDENTIALS[] = "GSSAPI_DELEGATE_CREDENTIALS";
static const char __pyx_k_HOST_NOT_ALLOWED_TO_CONNECT[] = "HOST_NOT_ALLOWED_TO_CONNECT";
static const char __pyx_k_NO_MORE_AUTH_METHODS_AVAILABLE[] = "NO_MORE_AUTH_METHODS_AVAILABLE";
static const char __pyx_k_PROTOCOL_VERSION_NOT_SUPPORTED[] = "PROTOCOL_VERSION_NOT_SUPPORTED";
static PyObject *__pyx_n_s_ACCESSTIME;
static PyObject *__pyx_n_s_ACL;
static PyObject *__pyx_n_s_ACMODTIME;
static PyObject *__pyx_n_s_ADD_IDENTITY;
static PyObject *__pyx_n_s_ADMINISTRATIVELY_PROHIBITED;
static PyObject *__pyx_n_s_AGAIN;
static PyObject *__pyx_n_s_APPEND;
static PyObject *__pyx_n_s_ATOMIC;
static PyObject *__pyx_n_s_ATTRS;
static PyObject *__pyx_n_s_AUTH;
static PyObject *__pyx_n_s_AUTH_AGENT;
static PyObject *__pyx_n_s_AUTH_CANCELLED_BY_USER;
static PyObject *__pyx_n_s_Auth;
static PyObject *__pyx_n_s_Auth_Method;
static PyObject *__pyx_n_s_BAD_MESSAGE;
static PyObject *__pyx_n_s_BANNER;
static PyObject *__pyx_n_s_BINDADDR;
static PyObject *__pyx_n_s_BOTH;
static PyObject *__pyx_n_s_BY_APPLICATION;
static PyObject *__pyx_n_s_CANCEL_TCPIP_FORWARD;
static PyObject *__pyx_n_s_CHANNEL;
static PyObject *__pyx_n_s_CHANNEL_OPEN;
static PyObject *__pyx_n_s_CIPHERS_C_S;
static PyObject *__pyx_n_s_CIPHERS_S_C;
static PyObject *__pyx_n_s_CLOSE;
static PyObject *__pyx_n_s_CLOSED;
static PyObject *__pyx_n_s_CLOSED_ERROR;
static PyObject *__pyx_n_s_COMP;
static PyObject *__pyx_n_s_COMPRESSION;
static PyObject *__pyx_n_s_COMPRESSION_C_S;
static PyObject *__pyx_n_s_COMPRESSION_ERROR;
static PyObject *__pyx_n_s_COMPRESSION_LEVEL;
static PyObject *__pyx_n_s_COMPRESSION_S_C;
static PyObject *__pyx_n_s_COMP_C_S;
static PyObject *__pyx_n_s_COMP_S_C;
static PyObject *__pyx_n_s_CONNECTED_ERROR;
static PyObject *__pyx_n_s_CONNECTED_OK;
static PyObject *__pyx_n_s_CONNECTED_TIMEOUT;
static PyObject *__pyx_n_s_CONNECTION_LOST;
static PyObject *__pyx_n_s_CONNECT_FAILED;
static PyObject *__pyx_n_s_CREAT;
static PyObject *__pyx_n_s_CREATETIME;
static PyObject *__pyx_n_s_CRYPT;
static PyObject *__pyx_n_s_CRYPT_C_S;
static PyObject *__pyx_n_s_CRYPT_S_C;
static PyObject *__pyx_n_s_Callbacks_Socket;
static PyObject *__pyx_n_s_Channel;
static PyObject *__pyx_n_s_Channel_Request;
static PyObject *__pyx_n_s_Connector;
static PyObject *__pyx_n_s_DATA;
static PyObject *__pyx_n_s_DEBUG;
static PyObject *__pyx_n_s_DENIED;
static PyObject *__pyx_n_s_DH_GEX_GROUP;
static PyObject *__pyx_n_s_DH_GEX_INIT;
static PyObject *__pyx_n_s_DH_GEX_REPLY;
static PyObject *__pyx_n_s_DH_GEX_REQUEST;
static PyObject *__pyx_n_s_DH_GEX_REQUEST_OLD;
static PyObject *__pyx_n_s_DIRECTORY;
static PyObject *__pyx_n_s_DIRECT_TCPIP;
static PyObject *__pyx_n_s_DISCONNECT;
static PyObject *__pyx_n_s_DSS;
static PyObject *__pyx_n_s_DSS_CERT01;
static PyObject *__pyx_n_s_ECDH_INIT;
static PyObject *__pyx_n_s_ECDH_REPLY;
static PyObject *__pyx_n_s_ECDSA;
static PyObject *__pyx_n_s_ECDSA_P256;
static PyObject *__pyx_n_s_ECDSA_P256_CERT01;
static PyObject *__pyx_n_s_ECDSA_P384;
static PyObject *__pyx_n_s_ECDSA_P384_CERT01;
static PyObject *__pyx_n_s_ECDSA_P521;
static PyObject *__pyx_n_s_ECDSA_P521_CERT01;
static PyObject *__pyx_n_s_ECMQV_INIT;
static PyObject *__pyx_n_s_ECMQV_REPLY;
static PyObject *__pyx_n_s_ED25519;
static PyObject *__pyx_n_s_ED25519_CERT01;
static PyObject *__pyx_n_s_EINTR;
static PyObject *__pyx_n_s_ENDDIR;
static PyObject *__pyx_n_s_ENV;
static PyObject *__pyx_n_s_EOF;
static PyObject *__pyx_n_s_ERROR;
static PyObject *__pyx_n_s_EXCEPTION_EOF;
static PyObject *__pyx_n_s_EXCEPTION_ERROR;
static PyObject *__pyx_n_s_EXCL;
static PyObject *__pyx_n_s_EXEC;
static PyObject *__pyx_n_s_EXTENDED;
static PyObject *__pyx_n_s_EXTENDED_DATA;
static PyObject *__pyx_n_s_EXTENDED_REPLY;
static PyObject *__pyx_n_s_Enum;
static PyObject *__pyx_n_s_ErrorTypes;
static PyObject *__pyx_n_s_FAILURE;
static PyObject *__pyx_n_s_FATAL;
static PyObject *__pyx_n_s_FD;
static PyObject *__pyx_n_s_FILE_ALREADY_EXISTS;
static PyObject *__pyx_n_s_FILE_NOT_FOUND;
static PyObject *__pyx_n_s_FLOW_WRITEWILLBLOCK;
static PyObject *__pyx_n_s_FLOW_WRITEWONTBLOCK;
static PyObject *__pyx_n_s_FORWARDED_TCPIP;
static PyObject *__pyx_n_s_FOUND_OTHER;
static PyObject *__pyx_n_s_FSETSTAT;
static PyObject *__pyx_n_s_FSTAT;
static PyObject *__pyx_n_s_FUNCTIONS;
static PyObject *__pyx_n_s_GLOBAL;
static PyObject *__pyx_n_s_GLOBAL_KNOWNHOSTS;
static PyObject *__pyx_n_s_GSSAPI_AUTH;
static PyObject *__pyx_n_s_GSSAPI_CLIENT_IDENTITY;
static PyObject *__pyx_n_s_GSSAPI_DELEGATE_CREDENTIALS;
static PyObject *__pyx_n_s_GSSAPI_ERROR;
static PyObject *__pyx_n_s_GSSAPI_ERRTOK;
static PyObject *__pyx_n_s_GSSAPI_EXCHANGE_COMPLETE;
static PyObject *__pyx_n_s_GSSAPI_MIC;
static PyObject *__pyx_n_s_GSSAPI_RESPONSE;
static PyObject *__pyx_n_s_GSSAPI_SERVER_IDENTITY;
static PyObject *__pyx_n_s_GSSAPI_TOKEN;
static PyObject *__pyx_n_s_Global_Request;
static PyObject *__pyx_n_s_HANDLE;
static PyObject *__pyx_n_s_HMAC_C_S;
static PyObject *__pyx_n_s_HMAC_S_C;
static PyObject *__pyx_n_s_HOST;
static PyObject *__pyx_n_s_HOSTBASED;
static PyObject *__pyx_n_s_HOSTKEYS;
static PyObject *__pyx_n_s_HOST_AUTHENTICATION_FAILED;
static PyObject *__pyx_n_s_HOST_KEY_NOT_VERIFIABLE;
static PyObject *__pyx_n_s_HOST_NOT_ALLOWED_TO_CONNECT;
static PyObject *__pyx_n_s_IDENTITY;
static PyObject *__pyx_n_s_IFBLK;
static PyObject *__pyx_n_s_IFCHR;
static PyObject *__pyx_n_s_IFDIR;
static PyObject *__pyx_n_s_IFIFO;
static PyObject *__pyx_n_s_IFLNK;
static PyObject *__pyx_n_s_IFMT;
static PyObject *__pyx_n_s_IFREG;
static PyObject *__pyx_n_s_IFSOCK;
static PyObject *__pyx_n_s_IGNORE;
static PyObject *__pyx_n_s_ILLEGAL_USER_NAME;
static PyObject *__pyx_n_s_INFO;
static PyObject *__pyx_n_s_INFO_REQUEST;
static PyObject *__pyx_n_s_INFO_RESPONSE;
static PyObject *__pyx_n_s_INIT;
static PyObject *__pyx_n_s_INTERACTIVE;
static PyObject *__pyx_n_s_INVALID_HANDLE;
static PyObject *__pyx_n_s_INVALID_SOCKET;
static PyObject *__pyx_n_s_KBDINT_AUTH;
static PyObject *__pyx_n_s_KEEPALIVE;
static PyObject *__pyx_n_s_KEX;
static PyObject *__pyx_n_s_KEXDH_INIT;
static PyObject *__pyx_n_s_KEXDH_REPLY;
static PyObject *__pyx_n_s_KEXINIT;
static PyObject *__pyx_n_s_KEY_EXCHANGE;
static PyObject *__pyx_n_s_KEY_EXCHANGE_FAILED;
static PyObject *__pyx_n_s_KNOWNHOSTS;
static PyObject *__pyx_n_s_KNOWN_CHANGED;
static PyObject *__pyx_n_s_KNOWN_OK;
static PyObject *__pyx_n_s_Kex;
static PyObject *__pyx_n_s_KeyCMP;
static PyObject *__pyx_n_s_KeyType;
static PyObject *__pyx_n_s_LANG;
static PyObject *__pyx_n_s_LANG_C_S;
static PyObject *__pyx_n_s_LANG_S_C;
static PyObject *__pyx_n_s_LOG_VERBOSITY;
static PyObject *__pyx_n_s_LOG_VERBOSITY_STR;
static PyObject *__pyx_n_s_LSTAT;
static PyObject *__pyx_n_s_Log;
static PyObject *__pyx_n_s_MAC;
static PyObject *__pyx_n_s_MAC_C_S;
static PyObject *__pyx_n_s_MAC_ERROR;
static PyObject *__pyx_n_s_MAC_S_C;
static PyObject *__pyx_n_s_MD5_DIGEST_LEN;
static PyObject *__pyx_n_s_MKDIR;
static PyObject *__pyx_n_s_MODIFYTIME;
static PyObject *__pyx_n_s_NAME;
static PyObject *__pyx_n_s_NATIVE;
static PyObject *__pyx_n_s_NEWDIR;
static PyObject *__pyx_n_s_NEWFILE;
static PyObject *__pyx_n_s_NEWKEYS;
static PyObject *__pyx_n_s_NODELAY;
static PyObject *__pyx_n_s_NOLOG;
static PyObject *__pyx_n_s_NONE;
static PyObject *__pyx_n_s_NOT_KNOWN;
static PyObject *__pyx_n_s_NO_CONNECTION;
static PyObject *__pyx_n_s_NO_ERROR;
static PyObject *__pyx_n_s_NO_MEDIA;
static PyObject *__pyx_n_s_NO_MORE_AUTH_METHODS_AVAILABLE;
static PyObject *__pyx_n_s_NO_SUCH_FILE;
static PyObject *__pyx_n_s_NO_SUCH_PATH;
static PyObject *__pyx_n_s_OK;
static PyObject *__pyx_n_s_OPEN;
static PyObject *__pyx_n_s_OPENDIR;
static PyObject *__pyx_n_s_OPEN_CONFIRMATION;
static PyObject *__pyx_n_s_OPEN_FAILURE;
static PyObject *__pyx_n_s_OP_UNSUPPORTED;
static PyObject *__pyx_n_s_OVERWRITE;
static PyObject *__pyx_n_s_OWNERGROUP;
static PyObject *__pyx_n_s_O_CREAT;
static PyObject *__pyx_n_s_O_EXCL;
static PyObject *__pyx_n_s_O_NOCTTY;
static PyObject *__pyx_n_s_O_NOFOLLOW;
static PyObject *__pyx_n_s_O_RDONLY;
static PyObject *__pyx_n_s_O_RDWR;
static PyObject *__pyx_n_s_O_TRUNC;
static PyObject *__pyx_n_s_O_WRONLY;
static PyObject *__pyx_n_s_Options;
static PyObject *__pyx_n_s_PACKET;
static PyObject *__pyx_n_s_PARTIAL;
static PyObject *__pyx_n_s_PASSWD_CHANGEREQ;
static PyObject *__pyx_n_s_PASSWORD;
static PyObject *__pyx_n_s_PASSWORD_AUTH;
static PyObject *__pyx_n_s_PERMISSIONS;
static PyObject *__pyx_n_s_PERMISSION_DENIED;
static PyObject *__pyx_n_s_PK_OK;
static PyObject *__pyx_n_s_PORT;
static PyObject *__pyx_n_s_PORT_STR;
static PyObject *__pyx_n_s_PRIVATE;
static PyObject *__pyx_n_s_PROTOCOL;
static PyObject *__pyx_n_s_PROTOCOL_ERROR;
static PyObject *__pyx_n_s_PROTOCOL_VERSION_NOT_SUPPORTED;
static PyObject *__pyx_n_s_PROXYCOMMAND;
static PyObject *__pyx_n_s_PTY;
static PyObject *__pyx_n_s_PUBKEY_AUTH;
static PyObject *__pyx_n_s_PUBLIC;
static PyObject *__pyx_n_s_PUBLICKEY;
static PyObject *__pyx_n_s_PublicKeyState;
static PyObject *__pyx_n_s_RARE;
static PyObject *__pyx_n_s_READ;
static PyObject *__pyx_n_s_READDIR;
static PyObject *__pyx_n_s_READLINK;
static PyObject *__pyx_n_s_READ_PENDING;
static PyObject *__pyx_n_s_REALPATH;
static PyObject *__pyx_n_s_RECURSIVE;
static PyObject *__pyx_n_s_REGULAR;
static PyObject *__pyx_n_s_REMOVE;
static PyObject *__pyx_n_s_RENAME;
static PyObject *__pyx_n_s_REQUEST;
static PyObject *__pyx_n_s_REQUEST_DENIED;
static PyObject *__pyx_n_s_RESERVED;
static PyObject *__pyx_n_s_RESOURCE_SHORTAGE;
static PyObject *__pyx_n_s_RMDIR;
static PyObject *__pyx_n_s_RSA;
static PyObject *__pyx_n_s_RSA1;
static PyObject *__pyx_n_s_RSA_CERT01;
static PyObject *__pyx_n_s_Request;
static PyObject *__pyx_n_s_SCP;
static PyObject *__pyx_n_s_SCP_Request;
static PyObject *__pyx_n_s_SERVICE;
static PyObject *__pyx_n_s_SERVICE_ACCEPT;
static PyObject *__pyx_n_s_SERVICE_NOT_AVAILABLE;
static PyObject *__pyx_n_s_SERVICE_REQUEST;
static PyObject *__pyx_n_s_SESSION;
static PyObject *__pyx_n_s_SETSTAT;
static PyObject *__pyx_n_s_SFTP;
static PyObject *__pyx_n_s_SFTP_AT;
static PyObject *__pyx_n_s_SFTP_FX;
static PyObject *__pyx_n_s_SFTP_FXE;
static PyObject *__pyx_n_s_SFTP_FXF;
static PyObject *__pyx_n_s_SFTP_FXF_Rename;
static PyObject *__pyx_n_s_SFTP_FXP;
static PyObject *__pyx_n_s_SFTP_FileXfer_Attr;
static PyObject *__pyx_n_s_SFTP_FileXfer_Type;
static PyObject *__pyx_n_s_SFTP_S;
static PyObject *__pyx_n_s_SHELL;
static PyObject *__pyx_n_s_SIZE;
static PyObject *__pyx_n_s_SPECIAL;
static PyObject *__pyx_n_s_SSH;
static PyObject *__pyx_n_s_SSH1;
static PyObject *__pyx_n_s_SSH2;
static PyObject *__pyx_n_s_SSH2_Disconnect;
static PyObject *__pyx_n_s_SSH2_ExtendedData;
static PyObject *__pyx_n_s_SSH2_Msg;
static PyObject *__pyx_n_s_SSH2_Msg_Channel;
static PyObject *__pyx_n_s_SSH2_Msg_Global;
static PyObject *__pyx_n_s_SSH2_Msg_Kex;
static PyObject *__pyx_n_s_SSH2_Msg_Request;
static PyObject *__pyx_n_s_SSH2_Msg_UserAuth;
static PyObject *__pyx_n_s_SSH2_Open;
static PyObject *__pyx_n_s_SSH_DIR;
static PyObject *__pyx_n_s_STAT;
static PyObject *__pyx_n_s_STATUS;
static PyObject *__pyx_n_s_STATVFS_ST_NOSUID;
static PyObject *__pyx_n_s_STATVFS_ST_RDONLY;
static PyObject *__pyx_n_s_STDERR;
static PyObject *__pyx_n_s_STDOUT;
static PyObject *__pyx_n_s_STRICTHOSTKEYCHECK;
static PyObject *__pyx_n_s_SUBSECOND_TIMES;
static PyObject *__pyx_n_s_SUBSYSTEM;
static PyObject *__pyx_n_s_SUCCESS;
static PyObject *__pyx_n_s_SYMLINK;
static PyObject *__pyx_n_s_Server;
static PyObject *__pyx_n_s_TCPIP_FORWARD;
static PyObject *__pyx_n_s_TEXT;
static PyObject *__pyx_n_s_TIMEOUT;
static PyObject *__pyx_n_s_TIMEOUT_USEC;
static PyObject *__pyx_n_s_TOO_MANY_CONNECTIONS;
static PyObject *__pyx_n_s_TRACE;
static PyObject *__pyx_n_s_TRUNC;
static PyObject *__pyx_n_s_UIDGID;
static PyObject *__pyx_n_s_UNIMPLEMENTED;
static PyObject *__pyx_n_s_UNKNOWN;
static PyObject *__pyx_n_s_UNKNOWN_CHANNEL_TYPE;
static PyObject *__pyx_n_s_USER;
static PyObject *__pyx_n_s_VALID;
static PyObject *__pyx_n_s_VERSION;
static PyObject *__pyx_n_s_WARN;
static PyObject *__pyx_n_s_WARNING;
static PyObject *__pyx_n_s_WINDOW_ADJUST;
static PyObject *__pyx_n_s_WINDOW_CHANGE;
static PyObject *__pyx_n_s_WRITE;
static PyObject *__pyx_n_s_WRITE_PENDING;
static PyObject *__pyx_n_s_WRITE_PROTECT;
static PyObject *__pyx_n_s_WRONG;
static PyObject *__pyx_n_s_X11;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_enum;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_ssh_libssh_enums;
static PyObject *__pyx_n_s_test;
/* Late includes */

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_libssh_enums(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_libssh_enums},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "libssh_enums",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_ACCESSTIME, __pyx_k_ACCESSTIME, sizeof(__pyx_k_ACCESSTIME), 0, 0, 1, 1},
  {&__pyx_n_s_ACL, __pyx_k_ACL, sizeof(__pyx_k_ACL), 0, 0, 1, 1},
  {&__pyx_n_s_ACMODTIME, __pyx_k_ACMODTIME, sizeof(__pyx_k_ACMODTIME), 0, 0, 1, 1},
  {&__pyx_n_s_ADD_IDENTITY, __pyx_k_ADD_IDENTITY, sizeof(__pyx_k_ADD_IDENTITY), 0, 0, 1, 1},
  {&__pyx_n_s_ADMINISTRATIVELY_PROHIBITED, __pyx_k_ADMINISTRATIVELY_PROHIBITED, sizeof(__pyx_k_ADMINISTRATIVELY_PROHIBITED), 0, 0, 1, 1},
  {&__pyx_n_s_AGAIN, __pyx_k_AGAIN, sizeof(__pyx_k_AGAIN), 0, 0, 1, 1},
  {&__pyx_n_s_APPEND, __pyx_k_APPEND, sizeof(__pyx_k_APPEND), 0, 0, 1, 1},
  {&__pyx_n_s_ATOMIC, __pyx_k_ATOMIC, sizeof(__pyx_k_ATOMIC), 0, 0, 1, 1},
  {&__pyx_n_s_ATTRS, __pyx_k_ATTRS, sizeof(__pyx_k_ATTRS), 0, 0, 1, 1},
  {&__pyx_n_s_AUTH, __pyx_k_AUTH, sizeof(__pyx_k_AUTH), 0, 0, 1, 1},
  {&__pyx_n_s_AUTH_AGENT, __pyx_k_AUTH_AGENT, sizeof(__pyx_k_AUTH_AGENT), 0, 0, 1, 1},
  {&__pyx_n_s_AUTH_CANCELLED_BY_USER, __pyx_k_AUTH_CANCELLED_BY_USER, sizeof(__pyx_k_AUTH_CANCELLED_BY_USER), 0, 0, 1, 1},
  {&__pyx_n_s_Auth, __pyx_k_Auth, sizeof(__pyx_k_Auth), 0, 0, 1, 1},
  {&__pyx_n_s_Auth_Method, __pyx_k_Auth_Method, sizeof(__pyx_k_Auth_Method), 0, 0, 1, 1},
  {&__pyx_n_s_BAD_MESSAGE, __pyx_k_BAD_MESSAGE, sizeof(__pyx_k_BAD_MESSAGE), 0, 0, 1, 1},
  {&__pyx_n_s_BANNER, __pyx_k_BANNER, sizeof(__pyx_k_BANNER), 0, 0, 1, 1},
  {&__pyx_n_s_BINDADDR, __pyx_k_BINDADDR, sizeof(__pyx_k_BINDADDR), 0, 0, 1, 1},
  {&__pyx_n_s_BOTH, __pyx_k_BOTH, sizeof(__pyx_k_BOTH), 0, 0, 1, 1},
  {&__pyx_n_s_BY_APPLICATION, __pyx_k_BY_APPLICATION, sizeof(__pyx_k_BY_APPLICATION), 0, 0, 1, 1},
  {&__pyx_n_s_CANCEL_TCPIP_FORWARD, __pyx_k_CANCEL_TCPIP_FORWARD, sizeof(__pyx_k_CANCEL_TCPIP_FORWARD), 0, 0, 1, 1},
  {&__pyx_n_s_CHANNEL, __pyx_k_CHANNEL, sizeof(__pyx_k_CHANNEL), 0, 0, 1, 1},
  {&__pyx_n_s_CHANNEL_OPEN, __pyx_k_CHANNEL_OPEN, sizeof(__pyx_k_CHANNEL_OPEN), 0, 0, 1, 1},
  {&__pyx_n_s_CIPHERS_C_S, __pyx_k_CIPHERS_C_S, sizeof(__pyx_k_CIPHERS_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_CIPHERS_S_C, __pyx_k_CIPHERS_S_C, sizeof(__pyx_k_CIPHERS_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_CLOSE, __pyx_k_CLOSE, sizeof(__pyx_k_CLOSE), 0, 0, 1, 1},
  {&__pyx_n_s_CLOSED, __pyx_k_CLOSED, sizeof(__pyx_k_CLOSED), 0, 0, 1, 1},
  {&__pyx_n_s_CLOSED_ERROR, __pyx_k_CLOSED_ERROR, sizeof(__pyx_k_CLOSED_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_COMP, __pyx_k_COMP, sizeof(__pyx_k_COMP), 0, 0, 1, 1},
  {&__pyx_n_s_COMPRESSION, __pyx_k_COMPRESSION, sizeof(__pyx_k_COMPRESSION), 0, 0, 1, 1},
  {&__pyx_n_s_COMPRESSION_C_S, __pyx_k_COMPRESSION_C_S, sizeof(__pyx_k_COMPRESSION_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_COMPRESSION_ERROR, __pyx_k_COMPRESSION_ERROR, sizeof(__pyx_k_COMPRESSION_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_COMPRESSION_LEVEL, __pyx_k_COMPRESSION_LEVEL, sizeof(__pyx_k_COMPRESSION_LEVEL), 0, 0, 1, 1},
  {&__pyx_n_s_COMPRESSION_S_C, __pyx_k_COMPRESSION_S_C, sizeof(__pyx_k_COMPRESSION_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_COMP_C_S, __pyx_k_COMP_C_S, sizeof(__pyx_k_COMP_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_COMP_S_C, __pyx_k_COMP_S_C, sizeof(__pyx_k_COMP_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_CONNECTED_ERROR, __pyx_k_CONNECTED_ERROR, sizeof(__pyx_k_CONNECTED_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_CONNECTED_OK, __pyx_k_CONNECTED_OK, sizeof(__pyx_k_CONNECTED_OK), 0, 0, 1, 1},
  {&__pyx_n_s_CONNECTED_TIMEOUT, __pyx_k_CONNECTED_TIMEOUT, sizeof(__pyx_k_CONNECTED_TIMEOUT), 0, 0, 1, 1},
  {&__pyx_n_s_CONNECTION_LOST, __pyx_k_CONNECTION_LOST, sizeof(__pyx_k_CONNECTION_LOST), 0, 0, 1, 1},
  {&__pyx_n_s_CONNECT_FAILED, __pyx_k_CONNECT_FAILED, sizeof(__pyx_k_CONNECT_FAILED), 0, 0, 1, 1},
  {&__pyx_n_s_CREAT, __pyx_k_CREAT, sizeof(__pyx_k_CREAT), 0, 0, 1, 1},
  {&__pyx_n_s_CREATETIME, __pyx_k_CREATETIME, sizeof(__pyx_k_CREATETIME), 0, 0, 1, 1},
  {&__pyx_n_s_CRYPT, __pyx_k_CRYPT, sizeof(__pyx_k_CRYPT), 0, 0, 1, 1},
  {&__pyx_n_s_CRYPT_C_S, __pyx_k_CRYPT_C_S, sizeof(__pyx_k_CRYPT_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_CRYPT_S_C, __pyx_k_CRYPT_S_C, sizeof(__pyx_k_CRYPT_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_Callbacks_Socket, __pyx_k_Callbacks_Socket, sizeof(__pyx_k_Callbacks_Socket), 0, 0, 1, 1},
  {&__pyx_n_s_Channel, __pyx_k_Channel, sizeof(__pyx_k_Channel), 0, 0, 1, 1},
  {&__pyx_n_s_Channel_Request, __pyx_k_Channel_Request, sizeof(__pyx_k_Channel_Request), 0, 0, 1, 1},
  {&__pyx_n_s_Connector, __pyx_k_Connector, sizeof(__pyx_k_Connector), 0, 0, 1, 1},
  {&__pyx_n_s_DATA, __pyx_k_DATA, sizeof(__pyx_k_DATA), 0, 0, 1, 1},
  {&__pyx_n_s_DEBUG, __pyx_k_DEBUG, sizeof(__pyx_k_DEBUG), 0, 0, 1, 1},
  {&__pyx_n_s_DENIED, __pyx_k_DENIED, sizeof(__pyx_k_DENIED), 0, 0, 1, 1},
  {&__pyx_n_s_DH_GEX_GROUP, __pyx_k_DH_GEX_GROUP, sizeof(__pyx_k_DH_GEX_GROUP), 0, 0, 1, 1},
  {&__pyx_n_s_DH_GEX_INIT, __pyx_k_DH_GEX_INIT, sizeof(__pyx_k_DH_GEX_INIT), 0, 0, 1, 1},
  {&__pyx_n_s_DH_GEX_REPLY, __pyx_k_DH_GEX_REPLY, sizeof(__pyx_k_DH_GEX_REPLY), 0, 0, 1, 1},
  {&__pyx_n_s_DH_GEX_REQUEST, __pyx_k_DH_GEX_REQUEST, sizeof(__pyx_k_DH_GEX_REQUEST), 0, 0, 1, 1},
  {&__pyx_n_s_DH_GEX_REQUEST_OLD, __pyx_k_DH_GEX_REQUEST_OLD, sizeof(__pyx_k_DH_GEX_REQUEST_OLD), 0, 0, 1, 1},
  {&__pyx_n_s_DIRECTORY, __pyx_k_DIRECTORY, sizeof(__pyx_k_DIRECTORY), 0, 0, 1, 1},
  {&__pyx_n_s_DIRECT_TCPIP, __pyx_k_DIRECT_TCPIP, sizeof(__pyx_k_DIRECT_TCPIP), 0, 0, 1, 1},
  {&__pyx_n_s_DISCONNECT, __pyx_k_DISCONNECT, sizeof(__pyx_k_DISCONNECT), 0, 0, 1, 1},
  {&__pyx_n_s_DSS, __pyx_k_DSS, sizeof(__pyx_k_DSS), 0, 0, 1, 1},
  {&__pyx_n_s_DSS_CERT01, __pyx_k_DSS_CERT01, sizeof(__pyx_k_DSS_CERT01), 0, 0, 1, 1},
  {&__pyx_n_s_ECDH_INIT, __pyx_k_ECDH_INIT, sizeof(__pyx_k_ECDH_INIT), 0, 0, 1, 1},
  {&__pyx_n_s_ECDH_REPLY, __pyx_k_ECDH_REPLY, sizeof(__pyx_k_ECDH_REPLY), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA, __pyx_k_ECDSA, sizeof(__pyx_k_ECDSA), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA_P256, __pyx_k_ECDSA_P256, sizeof(__pyx_k_ECDSA_P256), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA_P256_CERT01, __pyx_k_ECDSA_P256_CERT01, sizeof(__pyx_k_ECDSA_P256_CERT01), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA_P384, __pyx_k_ECDSA_P384, sizeof(__pyx_k_ECDSA_P384), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA_P384_CERT01, __pyx_k_ECDSA_P384_CERT01, sizeof(__pyx_k_ECDSA_P384_CERT01), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA_P521, __pyx_k_ECDSA_P521, sizeof(__pyx_k_ECDSA_P521), 0, 0, 1, 1},
  {&__pyx_n_s_ECDSA_P521_CERT01, __pyx_k_ECDSA_P521_CERT01, sizeof(__pyx_k_ECDSA_P521_CERT01), 0, 0, 1, 1},
  {&__pyx_n_s_ECMQV_INIT, __pyx_k_ECMQV_INIT, sizeof(__pyx_k_ECMQV_INIT), 0, 0, 1, 1},
  {&__pyx_n_s_ECMQV_REPLY, __pyx_k_ECMQV_REPLY, sizeof(__pyx_k_ECMQV_REPLY), 0, 0, 1, 1},
  {&__pyx_n_s_ED25519, __pyx_k_ED25519, sizeof(__pyx_k_ED25519), 0, 0, 1, 1},
  {&__pyx_n_s_ED25519_CERT01, __pyx_k_ED25519_CERT01, sizeof(__pyx_k_ED25519_CERT01), 0, 0, 1, 1},
  {&__pyx_n_s_EINTR, __pyx_k_EINTR, sizeof(__pyx_k_EINTR), 0, 0, 1, 1},
  {&__pyx_n_s_ENDDIR, __pyx_k_ENDDIR, sizeof(__pyx_k_ENDDIR), 0, 0, 1, 1},
  {&__pyx_n_s_ENV, __pyx_k_ENV, sizeof(__pyx_k_ENV), 0, 0, 1, 1},
  {&__pyx_n_s_EOF, __pyx_k_EOF, sizeof(__pyx_k_EOF), 0, 0, 1, 1},
  {&__pyx_n_s_ERROR, __pyx_k_ERROR, sizeof(__pyx_k_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_EXCEPTION_EOF, __pyx_k_EXCEPTION_EOF, sizeof(__pyx_k_EXCEPTION_EOF), 0, 0, 1, 1},
  {&__pyx_n_s_EXCEPTION_ERROR, __pyx_k_EXCEPTION_ERROR, sizeof(__pyx_k_EXCEPTION_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_EXCL, __pyx_k_EXCL, sizeof(__pyx_k_EXCL), 0, 0, 1, 1},
  {&__pyx_n_s_EXEC, __pyx_k_EXEC, sizeof(__pyx_k_EXEC), 0, 0, 1, 1},
  {&__pyx_n_s_EXTENDED, __pyx_k_EXTENDED, sizeof(__pyx_k_EXTENDED), 0, 0, 1, 1},
  {&__pyx_n_s_EXTENDED_DATA, __pyx_k_EXTENDED_DATA, sizeof(__pyx_k_EXTENDED_DATA), 0, 0, 1, 1},
  {&__pyx_n_s_EXTENDED_REPLY, __pyx_k_EXTENDED_REPLY, sizeof(__pyx_k_EXTENDED_REPLY), 0, 0, 1, 1},
  {&__pyx_n_s_Enum, __pyx_k_Enum, sizeof(__pyx_k_Enum), 0, 0, 1, 1},
  {&__pyx_n_s_ErrorTypes, __pyx_k_ErrorTypes, sizeof(__pyx_k_ErrorTypes), 0, 0, 1, 1},
  {&__pyx_n_s_FAILURE, __pyx_k_FAILURE, sizeof(__pyx_k_FAILURE), 0, 0, 1, 1},
  {&__pyx_n_s_FATAL, __pyx_k_FATAL, sizeof(__pyx_k_FATAL), 0, 0, 1, 1},
  {&__pyx_n_s_FD, __pyx_k_FD, sizeof(__pyx_k_FD), 0, 0, 1, 1},
  {&__pyx_n_s_FILE_ALREADY_EXISTS, __pyx_k_FILE_ALREADY_EXISTS, sizeof(__pyx_k_FILE_ALREADY_EXISTS), 0, 0, 1, 1},
  {&__pyx_n_s_FILE_NOT_FOUND, __pyx_k_FILE_NOT_FOUND, sizeof(__pyx_k_FILE_NOT_FOUND), 0, 0, 1, 1},
  {&__pyx_n_s_FLOW_WRITEWILLBLOCK, __pyx_k_FLOW_WRITEWILLBLOCK, sizeof(__pyx_k_FLOW_WRITEWILLBLOCK), 0, 0, 1, 1},
  {&__pyx_n_s_FLOW_WRITEWONTBLOCK, __pyx_k_FLOW_WRITEWONTBLOCK, sizeof(__pyx_k_FLOW_WRITEWONTBLOCK), 0, 0, 1, 1},
  {&__pyx_n_s_FORWARDED_TCPIP, __pyx_k_FORWARDED_TCPIP, sizeof(__pyx_k_FORWARDED_TCPIP), 0, 0, 1, 1},
  {&__pyx_n_s_FOUND_OTHER, __pyx_k_FOUND_OTHER, sizeof(__pyx_k_FOUND_OTHER), 0, 0, 1, 1},
  {&__pyx_n_s_FSETSTAT, __pyx_k_FSETSTAT, sizeof(__pyx_k_FSETSTAT), 0, 0, 1, 1},
  {&__pyx_n_s_FSTAT, __pyx_k_FSTAT, sizeof(__pyx_k_FSTAT), 0, 0, 1, 1},
  {&__pyx_n_s_FUNCTIONS, __pyx_k_FUNCTIONS, sizeof(__pyx_k_FUNCTIONS), 0, 0, 1, 1},
  {&__pyx_n_s_GLOBAL, __pyx_k_GLOBAL, sizeof(__pyx_k_GLOBAL), 0, 0, 1, 1},
  {&__pyx_n_s_GLOBAL_KNOWNHOSTS, __pyx_k_GLOBAL_KNOWNHOSTS, sizeof(__pyx_k_GLOBAL_KNOWNHOSTS), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_AUTH, __pyx_k_GSSAPI_AUTH, sizeof(__pyx_k_GSSAPI_AUTH), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_CLIENT_IDENTITY, __pyx_k_GSSAPI_CLIENT_IDENTITY, sizeof(__pyx_k_GSSAPI_CLIENT_IDENTITY), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_DELEGATE_CREDENTIALS, __pyx_k_GSSAPI_DELEGATE_CREDENTIALS, sizeof(__pyx_k_GSSAPI_DELEGATE_CREDENTIALS), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_ERROR, __pyx_k_GSSAPI_ERROR, sizeof(__pyx_k_GSSAPI_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_ERRTOK, __pyx_k_GSSAPI_ERRTOK, sizeof(__pyx_k_GSSAPI_ERRTOK), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_EXCHANGE_COMPLETE, __pyx_k_GSSAPI_EXCHANGE_COMPLETE, sizeof(__pyx_k_GSSAPI_EXCHANGE_COMPLETE), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_MIC, __pyx_k_GSSAPI_MIC, sizeof(__pyx_k_GSSAPI_MIC), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_RESPONSE, __pyx_k_GSSAPI_RESPONSE, sizeof(__pyx_k_GSSAPI_RESPONSE), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_SERVER_IDENTITY, __pyx_k_GSSAPI_SERVER_IDENTITY, sizeof(__pyx_k_GSSAPI_SERVER_IDENTITY), 0, 0, 1, 1},
  {&__pyx_n_s_GSSAPI_TOKEN, __pyx_k_GSSAPI_TOKEN, sizeof(__pyx_k_GSSAPI_TOKEN), 0, 0, 1, 1},
  {&__pyx_n_s_Global_Request, __pyx_k_Global_Request, sizeof(__pyx_k_Global_Request), 0, 0, 1, 1},
  {&__pyx_n_s_HANDLE, __pyx_k_HANDLE, sizeof(__pyx_k_HANDLE), 0, 0, 1, 1},
  {&__pyx_n_s_HMAC_C_S, __pyx_k_HMAC_C_S, sizeof(__pyx_k_HMAC_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_HMAC_S_C, __pyx_k_HMAC_S_C, sizeof(__pyx_k_HMAC_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_HOST, __pyx_k_HOST, sizeof(__pyx_k_HOST), 0, 0, 1, 1},
  {&__pyx_n_s_HOSTBASED, __pyx_k_HOSTBASED, sizeof(__pyx_k_HOSTBASED), 0, 0, 1, 1},
  {&__pyx_n_s_HOSTKEYS, __pyx_k_HOSTKEYS, sizeof(__pyx_k_HOSTKEYS), 0, 0, 1, 1},
  {&__pyx_n_s_HOST_AUTHENTICATION_FAILED, __pyx_k_HOST_AUTHENTICATION_FAILED, sizeof(__pyx_k_HOST_AUTHENTICATION_FAILED), 0, 0, 1, 1},
  {&__pyx_n_s_HOST_KEY_NOT_VERIFIABLE, __pyx_k_HOST_KEY_NOT_VERIFIABLE, sizeof(__pyx_k_HOST_KEY_NOT_VERIFIABLE), 0, 0, 1, 1},
  {&__pyx_n_s_HOST_NOT_ALLOWED_TO_CONNECT, __pyx_k_HOST_NOT_ALLOWED_TO_CONNECT, sizeof(__pyx_k_HOST_NOT_ALLOWED_TO_CONNECT), 0, 0, 1, 1},
  {&__pyx_n_s_IDENTITY, __pyx_k_IDENTITY, sizeof(__pyx_k_IDENTITY), 0, 0, 1, 1},
  {&__pyx_n_s_IFBLK, __pyx_k_IFBLK, sizeof(__pyx_k_IFBLK), 0, 0, 1, 1},
  {&__pyx_n_s_IFCHR, __pyx_k_IFCHR, sizeof(__pyx_k_IFCHR), 0, 0, 1, 1},
  {&__pyx_n_s_IFDIR, __pyx_k_IFDIR, sizeof(__pyx_k_IFDIR), 0, 0, 1, 1},
  {&__pyx_n_s_IFIFO, __pyx_k_IFIFO, sizeof(__pyx_k_IFIFO), 0, 0, 1, 1},
  {&__pyx_n_s_IFLNK, __pyx_k_IFLNK, sizeof(__pyx_k_IFLNK), 0, 0, 1, 1},
  {&__pyx_n_s_IFMT, __pyx_k_IFMT, sizeof(__pyx_k_IFMT), 0, 0, 1, 1},
  {&__pyx_n_s_IFREG, __pyx_k_IFREG, sizeof(__pyx_k_IFREG), 0, 0, 1, 1},
  {&__pyx_n_s_IFSOCK, __pyx_k_IFSOCK, sizeof(__pyx_k_IFSOCK), 0, 0, 1, 1},
  {&__pyx_n_s_IGNORE, __pyx_k_IGNORE, sizeof(__pyx_k_IGNORE), 0, 0, 1, 1},
  {&__pyx_n_s_ILLEGAL_USER_NAME, __pyx_k_ILLEGAL_USER_NAME, sizeof(__pyx_k_ILLEGAL_USER_NAME), 0, 0, 1, 1},
  {&__pyx_n_s_INFO, __pyx_k_INFO, sizeof(__pyx_k_INFO), 0, 0, 1, 1},
  {&__pyx_n_s_INFO_REQUEST, __pyx_k_INFO_REQUEST, sizeof(__pyx_k_INFO_REQUEST), 0, 0, 1, 1},
  {&__pyx_n_s_INFO_RESPONSE, __pyx_k_INFO_RESPONSE, sizeof(__pyx_k_INFO_RESPONSE), 0, 0, 1, 1},
  {&__pyx_n_s_INIT, __pyx_k_INIT, sizeof(__pyx_k_INIT), 0, 0, 1, 1},
  {&__pyx_n_s_INTERACTIVE, __pyx_k_INTERACTIVE, sizeof(__pyx_k_INTERACTIVE), 0, 0, 1, 1},
  {&__pyx_n_s_INVALID_HANDLE, __pyx_k_INVALID_HANDLE, sizeof(__pyx_k_INVALID_HANDLE), 0, 0, 1, 1},
  {&__pyx_n_s_INVALID_SOCKET, __pyx_k_INVALID_SOCKET, sizeof(__pyx_k_INVALID_SOCKET), 0, 0, 1, 1},
  {&__pyx_n_s_KBDINT_AUTH, __pyx_k_KBDINT_AUTH, sizeof(__pyx_k_KBDINT_AUTH), 0, 0, 1, 1},
  {&__pyx_n_s_KEEPALIVE, __pyx_k_KEEPALIVE, sizeof(__pyx_k_KEEPALIVE), 0, 0, 1, 1},
  {&__pyx_n_s_KEX, __pyx_k_KEX, sizeof(__pyx_k_KEX), 0, 0, 1, 1},
  {&__pyx_n_s_KEXDH_INIT, __pyx_k_KEXDH_INIT, sizeof(__pyx_k_KEXDH_INIT), 0, 0, 1, 1},
  {&__pyx_n_s_KEXDH_REPLY, __pyx_k_KEXDH_REPLY, sizeof(__pyx_k_KEXDH_REPLY), 0, 0, 1, 1},
  {&__pyx_n_s_KEXINIT, __pyx_k_KEXINIT, sizeof(__pyx_k_KEXINIT), 0, 0, 1, 1},
  {&__pyx_n_s_KEY_EXCHANGE, __pyx_k_KEY_EXCHANGE, sizeof(__pyx_k_KEY_EXCHANGE), 0, 0, 1, 1},
  {&__pyx_n_s_KEY_EXCHANGE_FAILED, __pyx_k_KEY_EXCHANGE_FAILED, sizeof(__pyx_k_KEY_EXCHANGE_FAILED), 0, 0, 1, 1},
  {&__pyx_n_s_KNOWNHOSTS, __pyx_k_KNOWNHOSTS, sizeof(__pyx_k_KNOWNHOSTS), 0, 0, 1, 1},
  {&__pyx_n_s_KNOWN_CHANGED, __pyx_k_KNOWN_CHANGED, sizeof(__pyx_k_KNOWN_CHANGED), 0, 0, 1, 1},
  {&__pyx_n_s_KNOWN_OK, __pyx_k_KNOWN_OK, sizeof(__pyx_k_KNOWN_OK), 0, 0, 1, 1},
  {&__pyx_n_s_Kex, __pyx_k_Kex, sizeof(__pyx_k_Kex), 0, 0, 1, 1},
  {&__pyx_n_s_KeyCMP, __pyx_k_KeyCMP, sizeof(__pyx_k_KeyCMP), 0, 0, 1, 1},
  {&__pyx_n_s_KeyType, __pyx_k_KeyType, sizeof(__pyx_k_KeyType), 0, 0, 1, 1},
  {&__pyx_n_s_LANG, __pyx_k_LANG, sizeof(__pyx_k_LANG), 0, 0, 1, 1},
  {&__pyx_n_s_LANG_C_S, __pyx_k_LANG_C_S, sizeof(__pyx_k_LANG_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_LANG_S_C, __pyx_k_LANG_S_C, sizeof(__pyx_k_LANG_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_LOG_VERBOSITY, __pyx_k_LOG_VERBOSITY, sizeof(__pyx_k_LOG_VERBOSITY), 0, 0, 1, 1},
  {&__pyx_n_s_LOG_VERBOSITY_STR, __pyx_k_LOG_VERBOSITY_STR, sizeof(__pyx_k_LOG_VERBOSITY_STR), 0, 0, 1, 1},
  {&__pyx_n_s_LSTAT, __pyx_k_LSTAT, sizeof(__pyx_k_LSTAT), 0, 0, 1, 1},
  {&__pyx_n_s_Log, __pyx_k_Log, sizeof(__pyx_k_Log), 0, 0, 1, 1},
  {&__pyx_n_s_MAC, __pyx_k_MAC, sizeof(__pyx_k_MAC), 0, 0, 1, 1},
  {&__pyx_n_s_MAC_C_S, __pyx_k_MAC_C_S, sizeof(__pyx_k_MAC_C_S), 0, 0, 1, 1},
  {&__pyx_n_s_MAC_ERROR, __pyx_k_MAC_ERROR, sizeof(__pyx_k_MAC_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_MAC_S_C, __pyx_k_MAC_S_C, sizeof(__pyx_k_MAC_S_C), 0, 0, 1, 1},
  {&__pyx_n_s_MD5_DIGEST_LEN, __pyx_k_MD5_DIGEST_LEN, sizeof(__pyx_k_MD5_DIGEST_LEN), 0, 0, 1, 1},
  {&__pyx_n_s_MKDIR, __pyx_k_MKDIR, sizeof(__pyx_k_MKDIR), 0, 0, 1, 1},
  {&__pyx_n_s_MODIFYTIME, __pyx_k_MODIFYTIME, sizeof(__pyx_k_MODIFYTIME), 0, 0, 1, 1},
  {&__pyx_n_s_NAME, __pyx_k_NAME, sizeof(__pyx_k_NAME), 0, 0, 1, 1},
  {&__pyx_n_s_NATIVE, __pyx_k_NATIVE, sizeof(__pyx_k_NATIVE), 0, 0, 1, 1},
  {&__pyx_n_s_NEWDIR, __pyx_k_NEWDIR, sizeof(__pyx_k_NEWDIR), 0, 0, 1, 1},
  {&__pyx_n_s_NEWFILE, __pyx_k_NEWFILE, sizeof(__pyx_k_NEWFILE), 0, 0, 1, 1},
  {&__pyx_n_s_NEWKEYS, __pyx_k_NEWKEYS, sizeof(__pyx_k_NEWKEYS), 0, 0, 1, 1},
  {&__pyx_n_s_NODELAY, __pyx_k_NODELAY, sizeof(__pyx_k_NODELAY), 0, 0, 1, 1},
  {&__pyx_n_s_NOLOG, __pyx_k_NOLOG, sizeof(__pyx_k_NOLOG), 0, 0, 1, 1},
  {&__pyx_n_s_NONE, __pyx_k_NONE, sizeof(__pyx_k_NONE), 0, 0, 1, 1},
  {&__pyx_n_s_NOT_KNOWN, __pyx_k_NOT_KNOWN, sizeof(__pyx_k_NOT_KNOWN), 0, 0, 1, 1},
  {&__pyx_n_s_NO_CONNECTION, __pyx_k_NO_CONNECTION, sizeof(__pyx_k_NO_CONNECTION), 0, 0, 1, 1},
  {&__pyx_n_s_NO_ERROR, __pyx_k_NO_ERROR, sizeof(__pyx_k_NO_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_NO_MEDIA, __pyx_k_NO_MEDIA, sizeof(__pyx_k_NO_MEDIA), 0, 0, 1, 1},
  {&__pyx_n_s_NO_MORE_AUTH_METHODS_AVAILABLE, __pyx_k_NO_MORE_AUTH_METHODS_AVAILABLE, sizeof(__pyx_k_NO_MORE_AUTH_METHODS_AVAILABLE), 0, 0, 1, 1},
  {&__pyx_n_s_NO_SUCH_FILE, __pyx_k_NO_SUCH_FILE, sizeof(__pyx_k_NO_SUCH_FILE), 0, 0, 1, 1},
  {&__pyx_n_s_NO_SUCH_PATH, __pyx_k_NO_SUCH_PATH, sizeof(__pyx_k_NO_SUCH_PATH), 0, 0, 1, 1},
  {&__pyx_n_s_OK, __pyx_k_OK, sizeof(__pyx_k_OK), 0, 0, 1, 1},
  {&__pyx_n_s_OPEN, __pyx_k_OPEN, sizeof(__pyx_k_OPEN), 0, 0, 1, 1},
  {&__pyx_n_s_OPENDIR, __pyx_k_OPENDIR, sizeof(__pyx_k_OPENDIR), 0, 0, 1, 1},
  {&__pyx_n_s_OPEN_CONFIRMATION, __pyx_k_OPEN_CONFIRMATION, sizeof(__pyx_k_OPEN_CONFIRMATION), 0, 0, 1, 1},
  {&__pyx_n_s_OPEN_FAILURE, __pyx_k_OPEN_FAILURE, sizeof(__pyx_k_OPEN_FAILURE), 0, 0, 1, 1},
  {&__pyx_n_s_OP_UNSUPPORTED, __pyx_k_OP_UNSUPPORTED, sizeof(__pyx_k_OP_UNSUPPORTED), 0, 0, 1, 1},
  {&__pyx_n_s_OVERWRITE, __pyx_k_OVERWRITE, sizeof(__pyx_k_OVERWRITE), 0, 0, 1, 1},
  {&__pyx_n_s_OWNERGROUP, __pyx_k_OWNERGROUP, sizeof(__pyx_k_OWNERGROUP), 0, 0, 1, 1},
  {&__pyx_n_s_O_CREAT, __pyx_k_O_CREAT, sizeof(__pyx_k_O_CREAT), 0, 0, 1, 1},
  {&__pyx_n_s_O_EXCL, __pyx_k_O_EXCL, sizeof(__pyx_k_O_EXCL), 0, 0, 1, 1},
  {&__pyx_n_s_O_NOCTTY, __pyx_k_O_NOCTTY, sizeof(__pyx_k_O_NOCTTY), 0, 0, 1, 1},
  {&__pyx_n_s_O_NOFOLLOW, __pyx_k_O_NOFOLLOW, sizeof(__pyx_k_O_NOFOLLOW), 0, 0, 1, 1},
  {&__pyx_n_s_O_RDONLY, __pyx_k_O_RDONLY, sizeof(__pyx_k_O_RDONLY), 0, 0, 1, 1},
  {&__pyx_n_s_O_RDWR, __pyx_k_O_RDWR, sizeof(__pyx_k_O_RDWR), 0, 0, 1, 1},
  {&__pyx_n_s_O_TRUNC, __pyx_k_O_TRUNC, sizeof(__pyx_k_O_TRUNC), 0, 0, 1, 1},
  {&__pyx_n_s_O_WRONLY, __pyx_k_O_WRONLY, sizeof(__pyx_k_O_WRONLY), 0, 0, 1, 1},
  {&__pyx_n_s_Options, __pyx_k_Options, sizeof(__pyx_k_Options), 0, 0, 1, 1},
  {&__pyx_n_s_PACKET, __pyx_k_PACKET, sizeof(__pyx_k_PACKET), 0, 0, 1, 1},
  {&__pyx_n_s_PARTIAL, __pyx_k_PARTIAL, sizeof(__pyx_k_PARTIAL), 0, 0, 1, 1},
  {&__pyx_n_s_PASSWD_CHANGEREQ, __pyx_k_PASSWD_CHANGEREQ, sizeof(__pyx_k_PASSWD_CHANGEREQ), 0, 0, 1, 1},
  {&__pyx_n_s_PASSWORD, __pyx_k_PASSWORD, sizeof(__pyx_k_PASSWORD), 0, 0, 1, 1},
  {&__pyx_n_s_PASSWORD_AUTH, __pyx_k_PASSWORD_AUTH, sizeof(__pyx_k_PASSWORD_AUTH), 0, 0, 1, 1},
  {&__pyx_n_s_PERMISSIONS, __pyx_k_PERMISSIONS, sizeof(__pyx_k_PERMISSIONS), 0, 0, 1, 1},
  {&__pyx_n_s_PERMISSION_DENIED, __pyx_k_PERMISSION_DENIED, sizeof(__pyx_k_PERMISSION_DENIED), 0, 0, 1, 1},
  {&__pyx_n_s_PK_OK, __pyx_k_PK_OK, sizeof(__pyx_k_PK_OK), 0, 0, 1, 1},
  {&__pyx_n_s_PORT, __pyx_k_PORT, sizeof(__pyx_k_PORT), 0, 0, 1, 1},
  {&__pyx_n_s_PORT_STR, __pyx_k_PORT_STR, sizeof(__pyx_k_PORT_STR), 0, 0, 1, 1},
  {&__pyx_n_s_PRIVATE, __pyx_k_PRIVATE, sizeof(__pyx_k_PRIVATE), 0, 0, 1, 1},
  {&__pyx_n_s_PROTOCOL, __pyx_k_PROTOCOL, sizeof(__pyx_k_PROTOCOL), 0, 0, 1, 1},
  {&__pyx_n_s_PROTOCOL_ERROR, __pyx_k_PROTOCOL_ERROR, sizeof(__pyx_k_PROTOCOL_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_PROTOCOL_VERSION_NOT_SUPPORTED, __pyx_k_PROTOCOL_VERSION_NOT_SUPPORTED, sizeof(__pyx_k_PROTOCOL_VERSION_NOT_SUPPORTED), 0, 0, 1, 1},
  {&__pyx_n_s_PROXYCOMMAND, __pyx_k_PROXYCOMMAND, sizeof(__pyx_k_PROXYCOMMAND), 0, 0, 1, 1},
  {&__pyx_n_s_PTY, __pyx_k_PTY, sizeof(__pyx_k_PTY), 0, 0, 1, 1},
  {&__pyx_n_s_PUBKEY_AUTH, __pyx_k_PUBKEY_AUTH, sizeof(__pyx_k_PUBKEY_AUTH), 0, 0, 1, 1},
  {&__pyx_n_s_PUBLIC, __pyx_k_PUBLIC, sizeof(__pyx_k_PUBLIC), 0, 0, 1, 1},
  {&__pyx_n_s_PUBLICKEY, __pyx_k_PUBLICKEY, sizeof(__pyx_k_PUBLICKEY), 0, 0, 1, 1},
  {&__pyx_n_s_PublicKeyState, __pyx_k_PublicKeyState, sizeof(__pyx_k_PublicKeyState), 0, 0, 1, 1},
  {&__pyx_n_s_RARE, __pyx_k_RARE, sizeof(__pyx_k_RARE), 0, 0, 1, 1},
  {&__pyx_n_s_READ, __pyx_k_READ, sizeof(__pyx_k_READ), 0, 0, 1, 1},
  {&__pyx_n_s_READDIR, __pyx_k_READDIR, sizeof(__pyx_k_READDIR), 0, 0, 1, 1},
  {&__pyx_n_s_READLINK, __pyx_k_READLINK, sizeof(__pyx_k_READLINK), 0, 0, 1, 1},
  {&__pyx_n_s_READ_PENDING, __pyx_k_READ_PENDING, sizeof(__pyx_k_READ_PENDING), 0, 0, 1, 1},
  {&__pyx_n_s_REALPATH, __pyx_k_REALPATH, sizeof(__pyx_k_REALPATH), 0, 0, 1, 1},
  {&__pyx_n_s_RECURSIVE, __pyx_k_RECURSIVE, sizeof(__pyx_k_RECURSIVE), 0, 0, 1, 1},
  {&__pyx_n_s_REGULAR, __pyx_k_REGULAR, sizeof(__pyx_k_REGULAR), 0, 0, 1, 1},
  {&__pyx_n_s_REMOVE, __pyx_k_REMOVE, sizeof(__pyx_k_REMOVE), 0, 0, 1, 1},
  {&__pyx_n_s_RENAME, __pyx_k_RENAME, sizeof(__pyx_k_RENAME), 0, 0, 1, 1},
  {&__pyx_n_s_REQUEST, __pyx_k_REQUEST, sizeof(__pyx_k_REQUEST), 0, 0, 1, 1},
  {&__pyx_n_s_REQUEST_DENIED, __pyx_k_REQUEST_DENIED, sizeof(__pyx_k_REQUEST_DENIED), 0, 0, 1, 1},
  {&__pyx_n_s_RESERVED, __pyx_k_RESERVED, sizeof(__pyx_k_RESERVED), 0, 0, 1, 1},
  {&__pyx_n_s_RESOURCE_SHORTAGE, __pyx_k_RESOURCE_SHORTAGE, sizeof(__pyx_k_RESOURCE_SHORTAGE), 0, 0, 1, 1},
  {&__pyx_n_s_RMDIR, __pyx_k_RMDIR, sizeof(__pyx_k_RMDIR), 0, 0, 1, 1},
  {&__pyx_n_s_RSA, __pyx_k_RSA, sizeof(__pyx_k_RSA), 0, 0, 1, 1},
  {&__pyx_n_s_RSA1, __pyx_k_RSA1, sizeof(__pyx_k_RSA1), 0, 0, 1, 1},
  {&__pyx_n_s_RSA_CERT01, __pyx_k_RSA_CERT01, sizeof(__pyx_k_RSA_CERT01), 0, 0, 1, 1},
  {&__pyx_n_s_Request, __pyx_k_Request, sizeof(__pyx_k_Request), 0, 0, 1, 1},
  {&__pyx_n_s_SCP, __pyx_k_SCP, sizeof(__pyx_k_SCP), 0, 0, 1, 1},
  {&__pyx_n_s_SCP_Request, __pyx_k_SCP_Request, sizeof(__pyx_k_SCP_Request), 0, 0, 1, 1},
  {&__pyx_n_s_SERVICE, __pyx_k_SERVICE, sizeof(__pyx_k_SERVICE), 0, 0, 1, 1},
  {&__pyx_n_s_SERVICE_ACCEPT, __pyx_k_SERVICE_ACCEPT, sizeof(__pyx_k_SERVICE_ACCEPT), 0, 0, 1, 1},
  {&__pyx_n_s_SERVICE_NOT_AVAILABLE, __pyx_k_SERVICE_NOT_AVAILABLE, sizeof(__pyx_k_SERVICE_NOT_AVAILABLE), 0, 0, 1, 1},
  {&__pyx_n_s_SERVICE_REQUEST, __pyx_k_SERVICE_REQUEST, sizeof(__pyx_k_SERVICE_REQUEST), 0, 0, 1, 1},
  {&__pyx_n_s_SESSION, __pyx_k_SESSION, sizeof(__pyx_k_SESSION), 0, 0, 1, 1},
  {&__pyx_n_s_SETSTAT, __pyx_k_SETSTAT, sizeof(__pyx_k_SETSTAT), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP, __pyx_k_SFTP, sizeof(__pyx_k_SFTP), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_AT, __pyx_k_SFTP_AT, sizeof(__pyx_k_SFTP_AT), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FX, __pyx_k_SFTP_FX, sizeof(__pyx_k_SFTP_FX), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FXE, __pyx_k_SFTP_FXE, sizeof(__pyx_k_SFTP_FXE), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FXF, __pyx_k_SFTP_FXF, sizeof(__pyx_k_SFTP_FXF), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FXF_Rename, __pyx_k_SFTP_FXF_Rename, sizeof(__pyx_k_SFTP_FXF_Rename), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FXP, __pyx_k_SFTP_FXP, sizeof(__pyx_k_SFTP_FXP), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FileXfer_Attr, __pyx_k_SFTP_FileXfer_Attr, sizeof(__pyx_k_SFTP_FileXfer_Attr), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_FileXfer_Type, __pyx_k_SFTP_FileXfer_Type, sizeof(__pyx_k_SFTP_FileXfer_Type), 0, 0, 1, 1},
  {&__pyx_n_s_SFTP_S, __pyx_k_SFTP_S, sizeof(__pyx_k_SFTP_S), 0, 0, 1, 1},
  {&__pyx_n_s_SHELL, __pyx_k_SHELL, sizeof(__pyx_k_SHELL), 0, 0, 1, 1},
  {&__pyx_n_s_SIZE, __pyx_k_SIZE, sizeof(__pyx_k_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_SPECIAL, __pyx_k_SPECIAL, sizeof(__pyx_k_SPECIAL), 0, 0, 1, 1},
  {&__pyx_n_s_SSH, __pyx_k_SSH, sizeof(__pyx_k_SSH), 0, 0, 1, 1},
  {&__pyx_n_s_SSH1, __pyx_k_SSH1, sizeof(__pyx_k_SSH1), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2, __pyx_k_SSH2, sizeof(__pyx_k_SSH2), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Disconnect, __pyx_k_SSH2_Disconnect, sizeof(__pyx_k_SSH2_Disconnect), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_ExtendedData, __pyx_k_SSH2_ExtendedData, sizeof(__pyx_k_SSH2_ExtendedData), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Msg, __pyx_k_SSH2_Msg, sizeof(__pyx_k_SSH2_Msg), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Msg_Channel, __pyx_k_SSH2_Msg_Channel, sizeof(__pyx_k_SSH2_Msg_Channel), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Msg_Global, __pyx_k_SSH2_Msg_Global, sizeof(__pyx_k_SSH2_Msg_Global), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Msg_Kex, __pyx_k_SSH2_Msg_Kex, sizeof(__pyx_k_SSH2_Msg_Kex), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Msg_Request, __pyx_k_SSH2_Msg_Request, sizeof(__pyx_k_SSH2_Msg_Request), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Msg_UserAuth, __pyx_k_SSH2_Msg_UserAuth, sizeof(__pyx_k_SSH2_Msg_UserAuth), 0, 0, 1, 1},
  {&__pyx_n_s_SSH2_Open, __pyx_k_SSH2_Open, sizeof(__pyx_k_SSH2_Open), 0, 0, 1, 1},
  {&__pyx_n_s_SSH_DIR, __pyx_k_SSH_DIR, sizeof(__pyx_k_SSH_DIR), 0, 0, 1, 1},
  {&__pyx_n_s_STAT, __pyx_k_STAT, sizeof(__pyx_k_STAT), 0, 0, 1, 1},
  {&__pyx_n_s_STATUS, __pyx_k_STATUS, sizeof(__pyx_k_STATUS), 0, 0, 1, 1},
  {&__pyx_n_s_STATVFS_ST_NOSUID, __pyx_k_STATVFS_ST_NOSUID, sizeof(__pyx_k_STATVFS_ST_NOSUID), 0, 0, 1, 1},
  {&__pyx_n_s_STATVFS_ST_RDONLY, __pyx_k_STATVFS_ST_RDONLY, sizeof(__pyx_k_STATVFS_ST_RDONLY), 0, 0, 1, 1},
  {&__pyx_n_s_STDERR, __pyx_k_STDERR, sizeof(__pyx_k_STDERR), 0, 0, 1, 1},
  {&__pyx_n_s_STDOUT, __pyx_k_STDOUT, sizeof(__pyx_k_STDOUT), 0, 0, 1, 1},
  {&__pyx_n_s_STRICTHOSTKEYCHECK, __pyx_k_STRICTHOSTKEYCHECK, sizeof(__pyx_k_STRICTHOSTKEYCHECK), 0, 0, 1, 1},
  {&__pyx_n_s_SUBSECOND_TIMES, __pyx_k_SUBSECOND_TIMES, sizeof(__pyx_k_SUBSECOND_TIMES), 0, 0, 1, 1},
  {&__pyx_n_s_SUBSYSTEM, __pyx_k_SUBSYSTEM, sizeof(__pyx_k_SUBSYSTEM), 0, 0, 1, 1},
  {&__pyx_n_s_SUCCESS, __pyx_k_SUCCESS, sizeof(__pyx_k_SUCCESS), 0, 0, 1, 1},
  {&__pyx_n_s_SYMLINK, __pyx_k_SYMLINK, sizeof(__pyx_k_SYMLINK), 0, 0, 1, 1},
  {&__pyx_n_s_Server, __pyx_k_Server, sizeof(__pyx_k_Server), 0, 0, 1, 1},
  {&__pyx_n_s_TCPIP_FORWARD, __pyx_k_TCPIP_FORWARD, sizeof(__pyx_k_TCPIP_FORWARD), 0, 0, 1, 1},
  {&__pyx_n_s_TEXT, __pyx_k_TEXT, sizeof(__pyx_k_TEXT), 0, 0, 1, 1},
  {&__pyx_n_s_TIMEOUT, __pyx_k_TIMEOUT, sizeof(__pyx_k_TIMEOUT), 0, 0, 1, 1},
  {&__pyx_n_s_TIMEOUT_USEC, __pyx_k_TIMEOUT_USEC, sizeof(__pyx_k_TIMEOUT_USEC), 0, 0, 1, 1},
  {&__pyx_n_s_TOO_MANY_CONNECTIONS, __pyx_k_TOO_MANY_CONNECTIONS, sizeof(__pyx_k_TOO_MANY_CONNECTIONS), 0, 0, 1, 1},
  {&__pyx_n_s_TRACE, __pyx_k_TRACE, sizeof(__pyx_k_TRACE), 0, 0, 1, 1},
  {&__pyx_n_s_TRUNC, __pyx_k_TRUNC, sizeof(__pyx_k_TRUNC), 0, 0, 1, 1},
  {&__pyx_n_s_UIDGID, __pyx_k_UIDGID, sizeof(__pyx_k_UIDGID), 0, 0, 1, 1},
  {&__pyx_n_s_UNIMPLEMENTED, __pyx_k_UNIMPLEMENTED, sizeof(__pyx_k_UNIMPLEMENTED), 0, 0, 1, 1},
  {&__pyx_n_s_UNKNOWN, __pyx_k_UNKNOWN, sizeof(__pyx_k_UNKNOWN), 0, 0, 1, 1},
  {&__pyx_n_s_UNKNOWN_CHANNEL_TYPE, __pyx_k_UNKNOWN_CHANNEL_TYPE, sizeof(__pyx_k_UNKNOWN_CHANNEL_TYPE), 0, 0, 1, 1},
  {&__pyx_n_s_USER, __pyx_k_USER, sizeof(__pyx_k_USER), 0, 0, 1, 1},
  {&__pyx_n_s_VALID, __pyx_k_VALID, sizeof(__pyx_k_VALID), 0, 0, 1, 1},
  {&__pyx_n_s_VERSION, __pyx_k_VERSION, sizeof(__pyx_k_VERSION), 0, 0, 1, 1},
  {&__pyx_n_s_WARN, __pyx_k_WARN, sizeof(__pyx_k_WARN), 0, 0, 1, 1},
  {&__pyx_n_s_WARNING, __pyx_k_WARNING, sizeof(__pyx_k_WARNING), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_ADJUST, __pyx_k_WINDOW_ADJUST, sizeof(__pyx_k_WINDOW_ADJUST), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_CHANGE, __pyx_k_WINDOW_CHANGE, sizeof(__pyx_k_WINDOW_CHANGE), 0, 0, 1, 1},
  {&__pyx_n_s_WRITE, __pyx_k_WRITE, sizeof(__pyx_k_WRITE), 0, 0, 1, 1},
  {&__pyx_n_s_WRITE_PENDING, __pyx_k_WRITE_PENDING, sizeof(__pyx_k_WRITE_PENDING), 0, 0, 1, 1},
  {&__pyx_n_s_WRITE_PROTECT, __pyx_k_WRITE_PROTECT, sizeof(__pyx_k_WRITE_PROTECT), 0, 0, 1, 1},
  {&__pyx_n_s_WRONG, __pyx_k_WRONG, sizeof(__pyx_k_WRONG), 0, 0, 1, 1},
  {&__pyx_n_s_X11, __pyx_k_X11, sizeof(__pyx_k_X11), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_enum, __pyx_k_enum, sizeof(__pyx_k_enum), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_ssh_libssh_enums, __pyx_k_ssh_libssh_enums, sizeof(__pyx_k_ssh_libssh_enums), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  return 0;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initlibssh_enums(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initlibssh_enums(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_libssh_enums(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_libssh_enums(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_libssh_enums(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'libssh_enums' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_libssh_enums(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("libssh_enums", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ssh__libssh_enums) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ssh.libssh_enums")) {
      if (unlikely(PyDict_SetItemString(modules, "ssh.libssh_enums", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ssh/libssh_enums.pyx":17
 * # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * import enum             # <<<<<<<<<<<<<<
 * 
 * from . cimport c_sftp
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_enum, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_enum, __pyx_t_1) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":25
 * 
 * #ssh
 * class SSH(enum.Enum):             # <<<<<<<<<<<<<<
 *     MD5_DIGEST_LEN = c_ssh.MD5_DIGEST_LEN
 *     CRYPT = c_ssh.SSH_CRYPT
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH, __pyx_n_s_SSH, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":26
 * #ssh
 * class SSH(enum.Enum):
 *     MD5_DIGEST_LEN = c_ssh.MD5_DIGEST_LEN             # <<<<<<<<<<<<<<
 *     CRYPT = c_ssh.SSH_CRYPT
 *     MAC = c_ssh.SSH_MAC
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(MD5_DIGEST_LEN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MD5_DIGEST_LEN, __pyx_t_4) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":27
 * class SSH(enum.Enum):
 *     MD5_DIGEST_LEN = c_ssh.MD5_DIGEST_LEN
 *     CRYPT = c_ssh.SSH_CRYPT             # <<<<<<<<<<<<<<
 *     MAC = c_ssh.SSH_MAC
 *     COMP = c_ssh.SSH_COMP
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_CRYPT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CRYPT, __pyx_t_4) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":28
 *     MD5_DIGEST_LEN = c_ssh.MD5_DIGEST_LEN
 *     CRYPT = c_ssh.SSH_CRYPT
 *     MAC = c_ssh.SSH_MAC             # <<<<<<<<<<<<<<
 *     COMP = c_ssh.SSH_COMP
 *     LANG = c_ssh.SSH_LANG
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_MAC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MAC, __pyx_t_4) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":29
 *     CRYPT = c_ssh.SSH_CRYPT
 *     MAC = c_ssh.SSH_MAC
 *     COMP = c_ssh.SSH_COMP             # <<<<<<<<<<<<<<
 *     LANG = c_ssh.SSH_LANG
 *     #Errors
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_COMP); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMP, __pyx_t_4) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":30
 *     MAC = c_ssh.SSH_MAC
 *     COMP = c_ssh.SSH_COMP
 *     LANG = c_ssh.SSH_LANG             # <<<<<<<<<<<<<<
 *     #Errors
 *     OK = c_ssh.SSH_OK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LANG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LANG, __pyx_t_4) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":32
 *     LANG = c_ssh.SSH_LANG
 *     #Errors
 *     OK = c_ssh.SSH_OK             # <<<<<<<<<<<<<<
 *     ERROR = c_ssh.SSH_ERROR
 *     AGAIN = c_ssh.SSH_AGAIN
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_OK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OK, __pyx_t_4) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":33
 *     #Errors
 *     OK = c_ssh.SSH_OK
 *     ERROR = c_ssh.SSH_ERROR             # <<<<<<<<<<<<<<
 *     AGAIN = c_ssh.SSH_AGAIN
 *     EOF = c_ssh.SSH_EOF
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":34
 *     OK = c_ssh.SSH_OK
 *     ERROR = c_ssh.SSH_ERROR
 *     AGAIN = c_ssh.SSH_AGAIN             # <<<<<<<<<<<<<<
 *     EOF = c_ssh.SSH_EOF
 *     INVALID_SOCKET = c_ssh.SSH_INVALID_SOCKET
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AGAIN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_AGAIN, __pyx_t_4) < 0) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":35
 *     ERROR = c_ssh.SSH_ERROR
 *     AGAIN = c_ssh.SSH_AGAIN
 *     EOF = c_ssh.SSH_EOF             # <<<<<<<<<<<<<<
 *     INVALID_SOCKET = c_ssh.SSH_INVALID_SOCKET
 *     #States
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_EOF); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EOF, __pyx_t_4) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":36
 *     AGAIN = c_ssh.SSH_AGAIN
 *     EOF = c_ssh.SSH_EOF
 *     INVALID_SOCKET = c_ssh.SSH_INVALID_SOCKET             # <<<<<<<<<<<<<<
 *     #States
 *     CLOSED = c_ssh.SSH_CLOSED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_INVALID_SOCKET); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INVALID_SOCKET, __pyx_t_4) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":38
 *     INVALID_SOCKET = c_ssh.SSH_INVALID_SOCKET
 *     #States
 *     CLOSED = c_ssh.SSH_CLOSED             # <<<<<<<<<<<<<<
 *     READ_PENDING = c_ssh.SSH_READ_PENDING
 *     CLOSED_ERROR = c_ssh.SSH_CLOSED_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_CLOSED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLOSED, __pyx_t_4) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":39
 *     #States
 *     CLOSED = c_ssh.SSH_CLOSED
 *     READ_PENDING = c_ssh.SSH_READ_PENDING             # <<<<<<<<<<<<<<
 *     CLOSED_ERROR = c_ssh.SSH_CLOSED_ERROR
 *     WRITE_PENDING = c_ssh.SSH_WRITE_PENDING
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_READ_PENDING); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READ_PENDING, __pyx_t_4) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":40
 *     CLOSED = c_ssh.SSH_CLOSED
 *     READ_PENDING = c_ssh.SSH_READ_PENDING
 *     CLOSED_ERROR = c_ssh.SSH_CLOSED_ERROR             # <<<<<<<<<<<<<<
 *     WRITE_PENDING = c_ssh.SSH_WRITE_PENDING
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_CLOSED_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLOSED_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":41
 *     READ_PENDING = c_ssh.SSH_READ_PENDING
 *     CLOSED_ERROR = c_ssh.SSH_CLOSED_ERROR
 *     WRITE_PENDING = c_ssh.SSH_WRITE_PENDING             # <<<<<<<<<<<<<<
 * 
 * class Kex(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_WRITE_PENDING); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRITE_PENDING, __pyx_t_4) < 0) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":25
 * 
 * #ssh
 * class SSH(enum.Enum):             # <<<<<<<<<<<<<<
 *     MD5_DIGEST_LEN = c_ssh.MD5_DIGEST_LEN
 *     CRYPT = c_ssh.SSH_CRYPT
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH, __pyx_t_4) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":43
 *     WRITE_PENDING = c_ssh.SSH_WRITE_PENDING
 * 
 * class Kex(enum.Enum):             # <<<<<<<<<<<<<<
 *     KEX = c_ssh.SSH_KEX
 *     HOSTKEYS = c_ssh.SSH_HOSTKEYS
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Kex, __pyx_n_s_Kex, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":44
 * 
 * class Kex(enum.Enum):
 *     KEX = c_ssh.SSH_KEX             # <<<<<<<<<<<<<<
 *     HOSTKEYS = c_ssh.SSH_HOSTKEYS
 *     CRYPT_C_S = c_ssh.SSH_CRYPT_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_KEX); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEX, __pyx_t_4) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":45
 * class Kex(enum.Enum):
 *     KEX = c_ssh.SSH_KEX
 *     HOSTKEYS = c_ssh.SSH_HOSTKEYS             # <<<<<<<<<<<<<<
 *     CRYPT_C_S = c_ssh.SSH_CRYPT_C_S
 *     CRYPT_S_C = c_ssh.SSH_CRYPT_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_HOSTKEYS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOSTKEYS, __pyx_t_4) < 0) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":46
 *     KEX = c_ssh.SSH_KEX
 *     HOSTKEYS = c_ssh.SSH_HOSTKEYS
 *     CRYPT_C_S = c_ssh.SSH_CRYPT_C_S             # <<<<<<<<<<<<<<
 *     CRYPT_S_C = c_ssh.SSH_CRYPT_S_C
 *     MAC_C_S = c_ssh.SSH_MAC_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_CRYPT_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CRYPT_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":47
 *     HOSTKEYS = c_ssh.SSH_HOSTKEYS
 *     CRYPT_C_S = c_ssh.SSH_CRYPT_C_S
 *     CRYPT_S_C = c_ssh.SSH_CRYPT_S_C             # <<<<<<<<<<<<<<
 *     MAC_C_S = c_ssh.SSH_MAC_C_S
 *     MAC_S_C = c_ssh.SSH_MAC_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_CRYPT_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CRYPT_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":48
 *     CRYPT_C_S = c_ssh.SSH_CRYPT_C_S
 *     CRYPT_S_C = c_ssh.SSH_CRYPT_S_C
 *     MAC_C_S = c_ssh.SSH_MAC_C_S             # <<<<<<<<<<<<<<
 *     MAC_S_C = c_ssh.SSH_MAC_S_C
 *     COMP_C_S = c_ssh.SSH_COMP_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_MAC_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MAC_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":49
 *     CRYPT_S_C = c_ssh.SSH_CRYPT_S_C
 *     MAC_C_S = c_ssh.SSH_MAC_C_S
 *     MAC_S_C = c_ssh.SSH_MAC_S_C             # <<<<<<<<<<<<<<
 *     COMP_C_S = c_ssh.SSH_COMP_C_S
 *     COMP_S_C = c_ssh.SSH_COMP_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_MAC_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MAC_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":50
 *     MAC_C_S = c_ssh.SSH_MAC_C_S
 *     MAC_S_C = c_ssh.SSH_MAC_S_C
 *     COMP_C_S = c_ssh.SSH_COMP_C_S             # <<<<<<<<<<<<<<
 *     COMP_S_C = c_ssh.SSH_COMP_S_C
 *     LANG_C_S = c_ssh.SSH_LANG_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_COMP_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMP_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":51
 *     MAC_S_C = c_ssh.SSH_MAC_S_C
 *     COMP_C_S = c_ssh.SSH_COMP_C_S
 *     COMP_S_C = c_ssh.SSH_COMP_S_C             # <<<<<<<<<<<<<<
 *     LANG_C_S = c_ssh.SSH_LANG_C_S
 *     LANG_S_C = c_ssh.SSH_LANG_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_COMP_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMP_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":52
 *     COMP_C_S = c_ssh.SSH_COMP_C_S
 *     COMP_S_C = c_ssh.SSH_COMP_S_C
 *     LANG_C_S = c_ssh.SSH_LANG_C_S             # <<<<<<<<<<<<<<
 *     LANG_S_C = c_ssh.SSH_LANG_S_C
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_LANG_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LANG_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":53
 *     COMP_S_C = c_ssh.SSH_COMP_S_C
 *     LANG_C_S = c_ssh.SSH_LANG_C_S
 *     LANG_S_C = c_ssh.SSH_LANG_S_C             # <<<<<<<<<<<<<<
 * 
 * class Auth(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_kex_types_e(SSH_LANG_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LANG_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":43
 *     WRITE_PENDING = c_ssh.SSH_WRITE_PENDING
 * 
 * class Kex(enum.Enum):             # <<<<<<<<<<<<<<
 *     KEX = c_ssh.SSH_KEX
 *     HOSTKEYS = c_ssh.SSH_HOSTKEYS
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Kex, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Kex, __pyx_t_4) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":55
 *     LANG_S_C = c_ssh.SSH_LANG_S_C
 * 
 * class Auth(enum.Enum):             # <<<<<<<<<<<<<<
 *     SUCCESS = c_ssh.SSH_AUTH_SUCCESS
 *     DENIED = c_ssh.SSH_AUTH_DENIED
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Auth, __pyx_n_s_Auth, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":56
 * 
 * class Auth(enum.Enum):
 *     SUCCESS = c_ssh.SSH_AUTH_SUCCESS             # <<<<<<<<<<<<<<
 *     DENIED = c_ssh.SSH_AUTH_DENIED
 *     PARTIAL = c_ssh.SSH_AUTH_PARTIAL
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_auth_e(SSH_AUTH_SUCCESS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SUCCESS, __pyx_t_4) < 0) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":57
 * class Auth(enum.Enum):
 *     SUCCESS = c_ssh.SSH_AUTH_SUCCESS
 *     DENIED = c_ssh.SSH_AUTH_DENIED             # <<<<<<<<<<<<<<
 *     PARTIAL = c_ssh.SSH_AUTH_PARTIAL
 *     INFO = c_ssh.SSH_AUTH_INFO
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_auth_e(SSH_AUTH_DENIED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DENIED, __pyx_t_4) < 0) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":58
 *     SUCCESS = c_ssh.SSH_AUTH_SUCCESS
 *     DENIED = c_ssh.SSH_AUTH_DENIED
 *     PARTIAL = c_ssh.SSH_AUTH_PARTIAL             # <<<<<<<<<<<<<<
 *     INFO = c_ssh.SSH_AUTH_INFO
 *     AGAIN = c_ssh.SSH_AUTH_AGAIN
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_auth_e(SSH_AUTH_PARTIAL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PARTIAL, __pyx_t_4) < 0) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":59
 *     DENIED = c_ssh.SSH_AUTH_DENIED
 *     PARTIAL = c_ssh.SSH_AUTH_PARTIAL
 *     INFO = c_ssh.SSH_AUTH_INFO             # <<<<<<<<<<<<<<
 *     AGAIN = c_ssh.SSH_AUTH_AGAIN
 *     ERROR = c_ssh.SSH_AUTH_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_auth_e(SSH_AUTH_INFO); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INFO, __pyx_t_4) < 0) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":60
 *     PARTIAL = c_ssh.SSH_AUTH_PARTIAL
 *     INFO = c_ssh.SSH_AUTH_INFO
 *     AGAIN = c_ssh.SSH_AUTH_AGAIN             # <<<<<<<<<<<<<<
 *     ERROR = c_ssh.SSH_AUTH_ERROR
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_auth_e(SSH_AUTH_AGAIN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_AGAIN, __pyx_t_4) < 0) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":61
 *     INFO = c_ssh.SSH_AUTH_INFO
 *     AGAIN = c_ssh.SSH_AUTH_AGAIN
 *     ERROR = c_ssh.SSH_AUTH_ERROR             # <<<<<<<<<<<<<<
 * 
 * class Auth_Method(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_auth_e(SSH_AUTH_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":55
 *     LANG_S_C = c_ssh.SSH_LANG_S_C
 * 
 * class Auth(enum.Enum):             # <<<<<<<<<<<<<<
 *     SUCCESS = c_ssh.SSH_AUTH_SUCCESS
 *     DENIED = c_ssh.SSH_AUTH_DENIED
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Auth, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Auth, __pyx_t_4) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":63
 *     ERROR = c_ssh.SSH_AUTH_ERROR
 * 
 * class Auth_Method(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_AUTH_METHOD_UNKNOWN
 *     NONE = c_ssh.SSH_AUTH_METHOD_NONE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Auth_Method, __pyx_n_s_Auth_Method, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":64
 * 
 * class Auth_Method(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_AUTH_METHOD_UNKNOWN             # <<<<<<<<<<<<<<
 *     NONE = c_ssh.SSH_AUTH_METHOD_NONE
 *     PASSWORD = c_ssh.SSH_AUTH_METHOD_PASSWORD
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_UNKNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":65
 * class Auth_Method(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_AUTH_METHOD_UNKNOWN
 *     NONE = c_ssh.SSH_AUTH_METHOD_NONE             # <<<<<<<<<<<<<<
 *     PASSWORD = c_ssh.SSH_AUTH_METHOD_PASSWORD
 *     PUBLICKEY = c_ssh.SSH_AUTH_METHOD_PUBLICKEY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NONE, __pyx_t_4) < 0) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":66
 *     UNKNOWN = c_ssh.SSH_AUTH_METHOD_UNKNOWN
 *     NONE = c_ssh.SSH_AUTH_METHOD_NONE
 *     PASSWORD = c_ssh.SSH_AUTH_METHOD_PASSWORD             # <<<<<<<<<<<<<<
 *     PUBLICKEY = c_ssh.SSH_AUTH_METHOD_PUBLICKEY
 *     HOSTBASED = c_ssh.SSH_AUTH_METHOD_HOSTBASED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_PASSWORD); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PASSWORD, __pyx_t_4) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":67
 *     NONE = c_ssh.SSH_AUTH_METHOD_NONE
 *     PASSWORD = c_ssh.SSH_AUTH_METHOD_PASSWORD
 *     PUBLICKEY = c_ssh.SSH_AUTH_METHOD_PUBLICKEY             # <<<<<<<<<<<<<<
 *     HOSTBASED = c_ssh.SSH_AUTH_METHOD_HOSTBASED
 *     INTERACTIVE = c_ssh.SSH_AUTH_METHOD_INTERACTIVE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_PUBLICKEY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PUBLICKEY, __pyx_t_4) < 0) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":68
 *     PASSWORD = c_ssh.SSH_AUTH_METHOD_PASSWORD
 *     PUBLICKEY = c_ssh.SSH_AUTH_METHOD_PUBLICKEY
 *     HOSTBASED = c_ssh.SSH_AUTH_METHOD_HOSTBASED             # <<<<<<<<<<<<<<
 *     INTERACTIVE = c_ssh.SSH_AUTH_METHOD_INTERACTIVE
 *     GSSAPI_MIC = c_ssh.SSH_AUTH_METHOD_GSSAPI_MIC
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_HOSTBASED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOSTBASED, __pyx_t_4) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":69
 *     PUBLICKEY = c_ssh.SSH_AUTH_METHOD_PUBLICKEY
 *     HOSTBASED = c_ssh.SSH_AUTH_METHOD_HOSTBASED
 *     INTERACTIVE = c_ssh.SSH_AUTH_METHOD_INTERACTIVE             # <<<<<<<<<<<<<<
 *     GSSAPI_MIC = c_ssh.SSH_AUTH_METHOD_GSSAPI_MIC
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_INTERACTIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INTERACTIVE, __pyx_t_4) < 0) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":70
 *     HOSTBASED = c_ssh.SSH_AUTH_METHOD_HOSTBASED
 *     INTERACTIVE = c_ssh.SSH_AUTH_METHOD_INTERACTIVE
 *     GSSAPI_MIC = c_ssh.SSH_AUTH_METHOD_GSSAPI_MIC             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_requests_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_AUTH_METHOD_GSSAPI_MIC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_MIC, __pyx_t_4) < 0) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":63
 *     ERROR = c_ssh.SSH_AUTH_ERROR
 * 
 * class Auth_Method(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_AUTH_METHOD_UNKNOWN
 *     NONE = c_ssh.SSH_AUTH_METHOD_NONE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Auth_Method, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Auth_Method, __pyx_t_4) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":73
 * 
 *     # enum ssh_requests_e:
 * class Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     AUTH = c_ssh.SSH_REQUEST_AUTH
 *     CHANNEL_OPEN = c_ssh.SSH_REQUEST_CHANNEL_OPEN
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Request, __pyx_n_s_Request, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":74
 *     # enum ssh_requests_e:
 * class Request(enum.Enum):
 *     AUTH = c_ssh.SSH_REQUEST_AUTH             # <<<<<<<<<<<<<<
 *     CHANNEL_OPEN = c_ssh.SSH_REQUEST_CHANNEL_OPEN
 *     CHANNEL = c_ssh.SSH_REQUEST_CHANNEL
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_requests_e(SSH_REQUEST_AUTH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_AUTH, __pyx_t_4) < 0) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":75
 * class Request(enum.Enum):
 *     AUTH = c_ssh.SSH_REQUEST_AUTH
 *     CHANNEL_OPEN = c_ssh.SSH_REQUEST_CHANNEL_OPEN             # <<<<<<<<<<<<<<
 *     CHANNEL = c_ssh.SSH_REQUEST_CHANNEL
 *     SERVICE = c_ssh.SSH_REQUEST_SERVICE
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_requests_e(SSH_REQUEST_CHANNEL_OPEN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CHANNEL_OPEN, __pyx_t_4) < 0) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":76
 *     AUTH = c_ssh.SSH_REQUEST_AUTH
 *     CHANNEL_OPEN = c_ssh.SSH_REQUEST_CHANNEL_OPEN
 *     CHANNEL = c_ssh.SSH_REQUEST_CHANNEL             # <<<<<<<<<<<<<<
 *     SERVICE = c_ssh.SSH_REQUEST_SERVICE
 *     GLOBAL = c_ssh.SSH_REQUEST_GLOBAL
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_requests_e(SSH_REQUEST_CHANNEL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CHANNEL, __pyx_t_4) < 0) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":77
 *     CHANNEL_OPEN = c_ssh.SSH_REQUEST_CHANNEL_OPEN
 *     CHANNEL = c_ssh.SSH_REQUEST_CHANNEL
 *     SERVICE = c_ssh.SSH_REQUEST_SERVICE             # <<<<<<<<<<<<<<
 *     GLOBAL = c_ssh.SSH_REQUEST_GLOBAL
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_requests_e(SSH_REQUEST_SERVICE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SERVICE, __pyx_t_4) < 0) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":78
 *     CHANNEL = c_ssh.SSH_REQUEST_CHANNEL
 *     SERVICE = c_ssh.SSH_REQUEST_SERVICE
 *     GLOBAL = c_ssh.SSH_REQUEST_GLOBAL             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_channel_type_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_requests_e(SSH_REQUEST_GLOBAL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GLOBAL, __pyx_t_4) < 0) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":73
 * 
 *     # enum ssh_requests_e:
 * class Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     AUTH = c_ssh.SSH_REQUEST_AUTH
 *     CHANNEL_OPEN = c_ssh.SSH_REQUEST_CHANNEL_OPEN
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Request, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Request, __pyx_t_4) < 0) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":81
 * 
 *     # enum ssh_channel_type_e:
 * class Channel(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_CHANNEL_UNKNOWN
 *     SESSION = c_ssh.SSH_CHANNEL_SESSION
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Channel, __pyx_n_s_Channel, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":82
 *     # enum ssh_channel_type_e:
 * class Channel(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_CHANNEL_UNKNOWN             # <<<<<<<<<<<<<<
 *     SESSION = c_ssh.SSH_CHANNEL_SESSION
 *     DIRECT_TCPIP = c_ssh.SSH_CHANNEL_DIRECT_TCPIP
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_type_e(SSH_CHANNEL_UNKNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":83
 * class Channel(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_CHANNEL_UNKNOWN
 *     SESSION = c_ssh.SSH_CHANNEL_SESSION             # <<<<<<<<<<<<<<
 *     DIRECT_TCPIP = c_ssh.SSH_CHANNEL_DIRECT_TCPIP
 *     FORWARDED_TCPIP = c_ssh.SSH_CHANNEL_FORWARDED_TCPIP
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_type_e(SSH_CHANNEL_SESSION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SESSION, __pyx_t_4) < 0) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":84
 *     UNKNOWN = c_ssh.SSH_CHANNEL_UNKNOWN
 *     SESSION = c_ssh.SSH_CHANNEL_SESSION
 *     DIRECT_TCPIP = c_ssh.SSH_CHANNEL_DIRECT_TCPIP             # <<<<<<<<<<<<<<
 *     FORWARDED_TCPIP = c_ssh.SSH_CHANNEL_FORWARDED_TCPIP
 *     X11 = c_ssh.SSH_CHANNEL_X11
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_type_e(SSH_CHANNEL_DIRECT_TCPIP); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DIRECT_TCPIP, __pyx_t_4) < 0) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":85
 *     SESSION = c_ssh.SSH_CHANNEL_SESSION
 *     DIRECT_TCPIP = c_ssh.SSH_CHANNEL_DIRECT_TCPIP
 *     FORWARDED_TCPIP = c_ssh.SSH_CHANNEL_FORWARDED_TCPIP             # <<<<<<<<<<<<<<
 *     X11 = c_ssh.SSH_CHANNEL_X11
 *     AUTH_AGENT = c_ssh.SSH_CHANNEL_AUTH_AGENT
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_type_e(SSH_CHANNEL_FORWARDED_TCPIP); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FORWARDED_TCPIP, __pyx_t_4) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":86
 *     DIRECT_TCPIP = c_ssh.SSH_CHANNEL_DIRECT_TCPIP
 *     FORWARDED_TCPIP = c_ssh.SSH_CHANNEL_FORWARDED_TCPIP
 *     X11 = c_ssh.SSH_CHANNEL_X11             # <<<<<<<<<<<<<<
 *     AUTH_AGENT = c_ssh.SSH_CHANNEL_AUTH_AGENT
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_type_e(SSH_CHANNEL_X11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_X11, __pyx_t_4) < 0) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":87
 *     FORWARDED_TCPIP = c_ssh.SSH_CHANNEL_FORWARDED_TCPIP
 *     X11 = c_ssh.SSH_CHANNEL_X11
 *     AUTH_AGENT = c_ssh.SSH_CHANNEL_AUTH_AGENT             # <<<<<<<<<<<<<<
 * 
 * # enum ssh_channel_requests_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_type_e(SSH_CHANNEL_AUTH_AGENT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_AUTH_AGENT, __pyx_t_4) < 0) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":81
 * 
 *     # enum ssh_channel_type_e:
 * class Channel(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_CHANNEL_UNKNOWN
 *     SESSION = c_ssh.SSH_CHANNEL_SESSION
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Channel, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Channel, __pyx_t_4) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":90
 * 
 * # enum ssh_channel_requests_e:
 * class Channel_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_CHANNEL_REQUEST_UNKNOWN
 *     PTY = c_ssh.SSH_CHANNEL_REQUEST_PTY
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Channel_Request, __pyx_n_s_Channel_Request, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":91
 * # enum ssh_channel_requests_e:
 * class Channel_Request(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_CHANNEL_REQUEST_UNKNOWN             # <<<<<<<<<<<<<<
 *     PTY = c_ssh.SSH_CHANNEL_REQUEST_PTY
 *     EXEC = c_ssh.SSH_CHANNEL_REQUEST_EXEC
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_UNKNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":92
 * class Channel_Request(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_CHANNEL_REQUEST_UNKNOWN
 *     PTY = c_ssh.SSH_CHANNEL_REQUEST_PTY             # <<<<<<<<<<<<<<
 *     EXEC = c_ssh.SSH_CHANNEL_REQUEST_EXEC
 *     SHELL = c_ssh.SSH_CHANNEL_REQUEST_SHELL
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_PTY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PTY, __pyx_t_4) < 0) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":93
 *     UNKNOWN = c_ssh.SSH_CHANNEL_REQUEST_UNKNOWN
 *     PTY = c_ssh.SSH_CHANNEL_REQUEST_PTY
 *     EXEC = c_ssh.SSH_CHANNEL_REQUEST_EXEC             # <<<<<<<<<<<<<<
 *     SHELL = c_ssh.SSH_CHANNEL_REQUEST_SHELL
 *     ENV = c_ssh.SSH_CHANNEL_REQUEST_ENV
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_EXEC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXEC, __pyx_t_4) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":94
 *     PTY = c_ssh.SSH_CHANNEL_REQUEST_PTY
 *     EXEC = c_ssh.SSH_CHANNEL_REQUEST_EXEC
 *     SHELL = c_ssh.SSH_CHANNEL_REQUEST_SHELL             # <<<<<<<<<<<<<<
 *     ENV = c_ssh.SSH_CHANNEL_REQUEST_ENV
 *     SUBSYSTEM = c_ssh.SSH_CHANNEL_REQUEST_SUBSYSTEM
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_SHELL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SHELL, __pyx_t_4) < 0) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":95
 *     EXEC = c_ssh.SSH_CHANNEL_REQUEST_EXEC
 *     SHELL = c_ssh.SSH_CHANNEL_REQUEST_SHELL
 *     ENV = c_ssh.SSH_CHANNEL_REQUEST_ENV             # <<<<<<<<<<<<<<
 *     SUBSYSTEM = c_ssh.SSH_CHANNEL_REQUEST_SUBSYSTEM
 *     WINDOW_CHANGE = c_ssh.SSH_CHANNEL_REQUEST_WINDOW_CHANGE
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_ENV); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ENV, __pyx_t_4) < 0) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":96
 *     SHELL = c_ssh.SSH_CHANNEL_REQUEST_SHELL
 *     ENV = c_ssh.SSH_CHANNEL_REQUEST_ENV
 *     SUBSYSTEM = c_ssh.SSH_CHANNEL_REQUEST_SUBSYSTEM             # <<<<<<<<<<<<<<
 *     WINDOW_CHANGE = c_ssh.SSH_CHANNEL_REQUEST_WINDOW_CHANGE
 *     X11 = c_ssh.SSH_CHANNEL_REQUEST_X11
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_SUBSYSTEM); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SUBSYSTEM, __pyx_t_4) < 0) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":97
 *     ENV = c_ssh.SSH_CHANNEL_REQUEST_ENV
 *     SUBSYSTEM = c_ssh.SSH_CHANNEL_REQUEST_SUBSYSTEM
 *     WINDOW_CHANGE = c_ssh.SSH_CHANNEL_REQUEST_WINDOW_CHANGE             # <<<<<<<<<<<<<<
 *     X11 = c_ssh.SSH_CHANNEL_REQUEST_X11
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_WINDOW_CHANGE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WINDOW_CHANGE, __pyx_t_4) < 0) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":98
 *     SUBSYSTEM = c_ssh.SSH_CHANNEL_REQUEST_SUBSYSTEM
 *     WINDOW_CHANGE = c_ssh.SSH_CHANNEL_REQUEST_WINDOW_CHANGE
 *     X11 = c_ssh.SSH_CHANNEL_REQUEST_X11             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_global_requests_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_channel_requests_e(SSH_CHANNEL_REQUEST_X11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_X11, __pyx_t_4) < 0) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":90
 * 
 * # enum ssh_channel_requests_e:
 * class Channel_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_CHANNEL_REQUEST_UNKNOWN
 *     PTY = c_ssh.SSH_CHANNEL_REQUEST_PTY
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Channel_Request, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Channel_Request, __pyx_t_4) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":101
 * 
 *     # enum ssh_global_requests_e:
 * class Global_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_GLOBAL_REQUEST_UNKNOWN
 *     TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_TCPIP_FORWARD
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Global_Request, __pyx_n_s_Global_Request, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":102
 *     # enum ssh_global_requests_e:
 * class Global_Request(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_GLOBAL_REQUEST_UNKNOWN             # <<<<<<<<<<<<<<
 *     TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_TCPIP_FORWARD
 *     CANCEL_TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_CANCEL_TCPIP_FORWARD
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_global_requests_e(SSH_GLOBAL_REQUEST_UNKNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":103
 * class Global_Request(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_GLOBAL_REQUEST_UNKNOWN
 *     TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_TCPIP_FORWARD             # <<<<<<<<<<<<<<
 *     CANCEL_TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_CANCEL_TCPIP_FORWARD
 *     KEEPALIVE = c_ssh.SSH_GLOBAL_REQUEST_KEEPALIVE
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_global_requests_e(SSH_GLOBAL_REQUEST_TCPIP_FORWARD); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TCPIP_FORWARD, __pyx_t_4) < 0) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":104
 *     UNKNOWN = c_ssh.SSH_GLOBAL_REQUEST_UNKNOWN
 *     TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_TCPIP_FORWARD
 *     CANCEL_TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_CANCEL_TCPIP_FORWARD             # <<<<<<<<<<<<<<
 *     KEEPALIVE = c_ssh.SSH_GLOBAL_REQUEST_KEEPALIVE
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_global_requests_e(SSH_GLOBAL_REQUEST_CANCEL_TCPIP_FORWARD); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CANCEL_TCPIP_FORWARD, __pyx_t_4) < 0) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":105
 *     TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_TCPIP_FORWARD
 *     CANCEL_TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_CANCEL_TCPIP_FORWARD
 *     KEEPALIVE = c_ssh.SSH_GLOBAL_REQUEST_KEEPALIVE             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_publickey_state_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_global_requests_e(SSH_GLOBAL_REQUEST_KEEPALIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEEPALIVE, __pyx_t_4) < 0) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":101
 * 
 *     # enum ssh_global_requests_e:
 * class Global_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_GLOBAL_REQUEST_UNKNOWN
 *     TCPIP_FORWARD = c_ssh.SSH_GLOBAL_REQUEST_TCPIP_FORWARD
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Global_Request, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Global_Request, __pyx_t_4) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":108
 * 
 *     # enum ssh_publickey_state_e:
 * class PublicKeyState(enum.Enum):             # <<<<<<<<<<<<<<
 *     ERROR = c_ssh.SSH_PUBLICKEY_STATE_ERROR
 *     NONE = c_ssh.SSH_PUBLICKEY_STATE_NONE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_PublicKeyState, __pyx_n_s_PublicKeyState, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":109
 *     # enum ssh_publickey_state_e:
 * class PublicKeyState(enum.Enum):
 *     ERROR = c_ssh.SSH_PUBLICKEY_STATE_ERROR             # <<<<<<<<<<<<<<
 *     NONE = c_ssh.SSH_PUBLICKEY_STATE_NONE
 *     VALID = c_ssh.SSH_PUBLICKEY_STATE_VALID
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_publickey_state_e(SSH_PUBLICKEY_STATE_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":110
 * class PublicKeyState(enum.Enum):
 *     ERROR = c_ssh.SSH_PUBLICKEY_STATE_ERROR
 *     NONE = c_ssh.SSH_PUBLICKEY_STATE_NONE             # <<<<<<<<<<<<<<
 *     VALID = c_ssh.SSH_PUBLICKEY_STATE_VALID
 *     WRONG = c_ssh.SSH_PUBLICKEY_STATE_WRONG
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_publickey_state_e(SSH_PUBLICKEY_STATE_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NONE, __pyx_t_4) < 0) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":111
 *     ERROR = c_ssh.SSH_PUBLICKEY_STATE_ERROR
 *     NONE = c_ssh.SSH_PUBLICKEY_STATE_NONE
 *     VALID = c_ssh.SSH_PUBLICKEY_STATE_VALID             # <<<<<<<<<<<<<<
 *     WRONG = c_ssh.SSH_PUBLICKEY_STATE_WRONG
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_publickey_state_e(SSH_PUBLICKEY_STATE_VALID); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_VALID, __pyx_t_4) < 0) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":112
 *     NONE = c_ssh.SSH_PUBLICKEY_STATE_NONE
 *     VALID = c_ssh.SSH_PUBLICKEY_STATE_VALID
 *     WRONG = c_ssh.SSH_PUBLICKEY_STATE_WRONG             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_server_known_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_publickey_state_e(SSH_PUBLICKEY_STATE_WRONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRONG, __pyx_t_4) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":108
 * 
 *     # enum ssh_publickey_state_e:
 * class PublicKeyState(enum.Enum):             # <<<<<<<<<<<<<<
 *     ERROR = c_ssh.SSH_PUBLICKEY_STATE_ERROR
 *     NONE = c_ssh.SSH_PUBLICKEY_STATE_NONE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_PublicKeyState, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PublicKeyState, __pyx_t_4) < 0) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":115
 * 
 *     # enum ssh_server_known_e:
 * class Server(enum.Enum):             # <<<<<<<<<<<<<<
 *     ERROR = c_ssh.SSH_SERVER_ERROR
 *     NOT_KNOWN = c_ssh.SSH_SERVER_NOT_KNOWN
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Server, __pyx_n_s_Server, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":116
 *     # enum ssh_server_known_e:
 * class Server(enum.Enum):
 *     ERROR = c_ssh.SSH_SERVER_ERROR             # <<<<<<<<<<<<<<
 *     NOT_KNOWN = c_ssh.SSH_SERVER_NOT_KNOWN
 *     KNOWN_OK = c_ssh.SSH_SERVER_KNOWN_OK
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_server_known_e(SSH_SERVER_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":117
 * class Server(enum.Enum):
 *     ERROR = c_ssh.SSH_SERVER_ERROR
 *     NOT_KNOWN = c_ssh.SSH_SERVER_NOT_KNOWN             # <<<<<<<<<<<<<<
 *     KNOWN_OK = c_ssh.SSH_SERVER_KNOWN_OK
 *     KNOWN_CHANGED = c_ssh.SSH_SERVER_KNOWN_CHANGED
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_server_known_e(SSH_SERVER_NOT_KNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NOT_KNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":118
 *     ERROR = c_ssh.SSH_SERVER_ERROR
 *     NOT_KNOWN = c_ssh.SSH_SERVER_NOT_KNOWN
 *     KNOWN_OK = c_ssh.SSH_SERVER_KNOWN_OK             # <<<<<<<<<<<<<<
 *     KNOWN_CHANGED = c_ssh.SSH_SERVER_KNOWN_CHANGED
 *     FOUND_OTHER = c_ssh.SSH_SERVER_FOUND_OTHER
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_server_known_e(SSH_SERVER_KNOWN_OK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KNOWN_OK, __pyx_t_4) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":119
 *     NOT_KNOWN = c_ssh.SSH_SERVER_NOT_KNOWN
 *     KNOWN_OK = c_ssh.SSH_SERVER_KNOWN_OK
 *     KNOWN_CHANGED = c_ssh.SSH_SERVER_KNOWN_CHANGED             # <<<<<<<<<<<<<<
 *     FOUND_OTHER = c_ssh.SSH_SERVER_FOUND_OTHER
 *     FILE_NOT_FOUND = c_ssh.SSH_SERVER_FILE_NOT_FOUND
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_server_known_e(SSH_SERVER_KNOWN_CHANGED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KNOWN_CHANGED, __pyx_t_4) < 0) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":120
 *     KNOWN_OK = c_ssh.SSH_SERVER_KNOWN_OK
 *     KNOWN_CHANGED = c_ssh.SSH_SERVER_KNOWN_CHANGED
 *     FOUND_OTHER = c_ssh.SSH_SERVER_FOUND_OTHER             # <<<<<<<<<<<<<<
 *     FILE_NOT_FOUND = c_ssh.SSH_SERVER_FILE_NOT_FOUND
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_server_known_e(SSH_SERVER_FOUND_OTHER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FOUND_OTHER, __pyx_t_4) < 0) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":121
 *     KNOWN_CHANGED = c_ssh.SSH_SERVER_KNOWN_CHANGED
 *     FOUND_OTHER = c_ssh.SSH_SERVER_FOUND_OTHER
 *     FILE_NOT_FOUND = c_ssh.SSH_SERVER_FILE_NOT_FOUND             # <<<<<<<<<<<<<<
 * 
 *     # Errors
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_server_known_e(SSH_SERVER_FILE_NOT_FOUND); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FILE_NOT_FOUND, __pyx_t_4) < 0) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":115
 * 
 *     # enum ssh_server_known_e:
 * class Server(enum.Enum):             # <<<<<<<<<<<<<<
 *     ERROR = c_ssh.SSH_SERVER_ERROR
 *     NOT_KNOWN = c_ssh.SSH_SERVER_NOT_KNOWN
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Server, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Server, __pyx_t_4) < 0) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":125
 *     # Errors
 *     # enum ssh_error_types_e:
 * class ErrorTypes(enum.Enum):             # <<<<<<<<<<<<<<
 *     NO_ERROR = c_ssh.SSH_NO_ERROR
 *     REQUEST_DENIED = c_ssh.SSH_REQUEST_DENIED
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_ErrorTypes, __pyx_n_s_ErrorTypes, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":126
 *     # enum ssh_error_types_e:
 * class ErrorTypes(enum.Enum):
 *     NO_ERROR = c_ssh.SSH_NO_ERROR             # <<<<<<<<<<<<<<
 *     REQUEST_DENIED = c_ssh.SSH_REQUEST_DENIED
 *     FATAL = c_ssh.SSH_FATAL
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_error_types_e(SSH_NO_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NO_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":127
 * class ErrorTypes(enum.Enum):
 *     NO_ERROR = c_ssh.SSH_NO_ERROR
 *     REQUEST_DENIED = c_ssh.SSH_REQUEST_DENIED             # <<<<<<<<<<<<<<
 *     FATAL = c_ssh.SSH_FATAL
 *     EINTR = c_ssh.SSH_EINTR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_error_types_e(SSH_REQUEST_DENIED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REQUEST_DENIED, __pyx_t_4) < 0) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":128
 *     NO_ERROR = c_ssh.SSH_NO_ERROR
 *     REQUEST_DENIED = c_ssh.SSH_REQUEST_DENIED
 *     FATAL = c_ssh.SSH_FATAL             # <<<<<<<<<<<<<<
 *     EINTR = c_ssh.SSH_EINTR
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_error_types_e(SSH_FATAL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FATAL, __pyx_t_4) < 0) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":129
 *     REQUEST_DENIED = c_ssh.SSH_REQUEST_DENIED
 *     FATAL = c_ssh.SSH_FATAL
 *     EINTR = c_ssh.SSH_EINTR             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_keytypes_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_error_types_e(SSH_EINTR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EINTR, __pyx_t_4) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":125
 *     # Errors
 *     # enum ssh_error_types_e:
 * class ErrorTypes(enum.Enum):             # <<<<<<<<<<<<<<
 *     NO_ERROR = c_ssh.SSH_NO_ERROR
 *     REQUEST_DENIED = c_ssh.SSH_REQUEST_DENIED
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_ErrorTypes, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ErrorTypes, __pyx_t_4) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":132
 * 
 *     # enum ssh_keytypes_e:
 * class KeyType(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_KEYTYPE_UNKNOWN
 *     DSS = c_ssh.SSH_KEYTYPE_DSS
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_KeyType, __pyx_n_s_KeyType, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":133
 *     # enum ssh_keytypes_e:
 * class KeyType(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_KEYTYPE_UNKNOWN             # <<<<<<<<<<<<<<
 *     DSS = c_ssh.SSH_KEYTYPE_DSS
 *     RSA = c_ssh.SSH_KEYTYPE_RSA
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_UNKNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":134
 * class KeyType(enum.Enum):
 *     UNKNOWN = c_ssh.SSH_KEYTYPE_UNKNOWN
 *     DSS = c_ssh.SSH_KEYTYPE_DSS             # <<<<<<<<<<<<<<
 *     RSA = c_ssh.SSH_KEYTYPE_RSA
 *     RSA1 = c_ssh.SSH_KEYTYPE_RSA1
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_DSS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DSS, __pyx_t_4) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":135
 *     UNKNOWN = c_ssh.SSH_KEYTYPE_UNKNOWN
 *     DSS = c_ssh.SSH_KEYTYPE_DSS
 *     RSA = c_ssh.SSH_KEYTYPE_RSA             # <<<<<<<<<<<<<<
 *     RSA1 = c_ssh.SSH_KEYTYPE_RSA1
 *     ECDSA = c_ssh.SSH_KEYTYPE_ECDSA  # Deprecated
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_RSA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RSA, __pyx_t_4) < 0) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":136
 *     DSS = c_ssh.SSH_KEYTYPE_DSS
 *     RSA = c_ssh.SSH_KEYTYPE_RSA
 *     RSA1 = c_ssh.SSH_KEYTYPE_RSA1             # <<<<<<<<<<<<<<
 *     ECDSA = c_ssh.SSH_KEYTYPE_ECDSA  # Deprecated
 *     ED25519 = c_ssh.SSH_KEYTYPE_ED25519
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_RSA1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RSA1, __pyx_t_4) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":137
 *     RSA = c_ssh.SSH_KEYTYPE_RSA
 *     RSA1 = c_ssh.SSH_KEYTYPE_RSA1
 *     ECDSA = c_ssh.SSH_KEYTYPE_ECDSA  # Deprecated             # <<<<<<<<<<<<<<
 *     ED25519 = c_ssh.SSH_KEYTYPE_ED25519
 *     DSS_CERT01 = c_ssh.SSH_KEYTYPE_DSS_CERT01
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA, __pyx_t_4) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":138
 *     RSA1 = c_ssh.SSH_KEYTYPE_RSA1
 *     ECDSA = c_ssh.SSH_KEYTYPE_ECDSA  # Deprecated
 *     ED25519 = c_ssh.SSH_KEYTYPE_ED25519             # <<<<<<<<<<<<<<
 *     DSS_CERT01 = c_ssh.SSH_KEYTYPE_DSS_CERT01
 *     RSA_CERT01 = c_ssh.SSH_KEYTYPE_RSA_CERT01
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ED25519); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ED25519, __pyx_t_4) < 0) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":139
 *     ECDSA = c_ssh.SSH_KEYTYPE_ECDSA  # Deprecated
 *     ED25519 = c_ssh.SSH_KEYTYPE_ED25519
 *     DSS_CERT01 = c_ssh.SSH_KEYTYPE_DSS_CERT01             # <<<<<<<<<<<<<<
 *     RSA_CERT01 = c_ssh.SSH_KEYTYPE_RSA_CERT01
 *     ECDSA_P256 = c_ssh.SSH_KEYTYPE_ECDSA_P256
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_DSS_CERT01); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DSS_CERT01, __pyx_t_4) < 0) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":140
 *     ED25519 = c_ssh.SSH_KEYTYPE_ED25519
 *     DSS_CERT01 = c_ssh.SSH_KEYTYPE_DSS_CERT01
 *     RSA_CERT01 = c_ssh.SSH_KEYTYPE_RSA_CERT01             # <<<<<<<<<<<<<<
 *     ECDSA_P256 = c_ssh.SSH_KEYTYPE_ECDSA_P256
 *     ECDSA_P384 = c_ssh.SSH_KEYTYPE_ECDSA_P384
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_RSA_CERT01); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RSA_CERT01, __pyx_t_4) < 0) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":141
 *     DSS_CERT01 = c_ssh.SSH_KEYTYPE_DSS_CERT01
 *     RSA_CERT01 = c_ssh.SSH_KEYTYPE_RSA_CERT01
 *     ECDSA_P256 = c_ssh.SSH_KEYTYPE_ECDSA_P256             # <<<<<<<<<<<<<<
 *     ECDSA_P384 = c_ssh.SSH_KEYTYPE_ECDSA_P384
 *     ECDSA_P521 = c_ssh.SSH_KEYTYPE_ECDSA_P521
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA_P256); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA_P256, __pyx_t_4) < 0) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":142
 *     RSA_CERT01 = c_ssh.SSH_KEYTYPE_RSA_CERT01
 *     ECDSA_P256 = c_ssh.SSH_KEYTYPE_ECDSA_P256
 *     ECDSA_P384 = c_ssh.SSH_KEYTYPE_ECDSA_P384             # <<<<<<<<<<<<<<
 *     ECDSA_P521 = c_ssh.SSH_KEYTYPE_ECDSA_P521
 *     ECDSA_P256_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P256_CERT01
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA_P384); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA_P384, __pyx_t_4) < 0) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":143
 *     ECDSA_P256 = c_ssh.SSH_KEYTYPE_ECDSA_P256
 *     ECDSA_P384 = c_ssh.SSH_KEYTYPE_ECDSA_P384
 *     ECDSA_P521 = c_ssh.SSH_KEYTYPE_ECDSA_P521             # <<<<<<<<<<<<<<
 *     ECDSA_P256_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P256_CERT01
 *     ECDSA_P384_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P384_CERT01
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA_P521); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA_P521, __pyx_t_4) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":144
 *     ECDSA_P384 = c_ssh.SSH_KEYTYPE_ECDSA_P384
 *     ECDSA_P521 = c_ssh.SSH_KEYTYPE_ECDSA_P521
 *     ECDSA_P256_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P256_CERT01             # <<<<<<<<<<<<<<
 *     ECDSA_P384_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P384_CERT01
 *     ECDSA_P521_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P521_CERT01
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA_P256_CERT01); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA_P256_CERT01, __pyx_t_4) < 0) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":145
 *     ECDSA_P521 = c_ssh.SSH_KEYTYPE_ECDSA_P521
 *     ECDSA_P256_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P256_CERT01
 *     ECDSA_P384_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P384_CERT01             # <<<<<<<<<<<<<<
 *     ECDSA_P521_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P521_CERT01
 *     ED25519_CERT01 = c_ssh.SSH_KEYTYPE_ED25519_CERT01
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA_P384_CERT01); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA_P384_CERT01, __pyx_t_4) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":146
 *     ECDSA_P256_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P256_CERT01
 *     ECDSA_P384_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P384_CERT01
 *     ECDSA_P521_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P521_CERT01             # <<<<<<<<<<<<<<
 *     ED25519_CERT01 = c_ssh.SSH_KEYTYPE_ED25519_CERT01
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ECDSA_P521_CERT01); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDSA_P521_CERT01, __pyx_t_4) < 0) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":147
 *     ECDSA_P384_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P384_CERT01
 *     ECDSA_P521_CERT01 = c_ssh.SSH_KEYTYPE_ECDSA_P521_CERT01
 *     ED25519_CERT01 = c_ssh.SSH_KEYTYPE_ED25519_CERT01             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_keycmp_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keytypes_e(SSH_KEYTYPE_ED25519_CERT01); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ED25519_CERT01, __pyx_t_4) < 0) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":132
 * 
 *     # enum ssh_keytypes_e:
 * class KeyType(enum.Enum):             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_ssh.SSH_KEYTYPE_UNKNOWN
 *     DSS = c_ssh.SSH_KEYTYPE_DSS
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_KeyType, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_KeyType, __pyx_t_4) < 0) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":150
 * 
 *     # enum ssh_keycmp_e:
 * class KeyCMP(enum.Enum):             # <<<<<<<<<<<<<<
 *     PUBLIC = c_ssh.SSH_KEY_CMP_PUBLIC
 *     PRIVATE = c_ssh.SSH_KEY_CMP_PRIVATE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_KeyCMP, __pyx_n_s_KeyCMP, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":151
 *     # enum ssh_keycmp_e:
 * class KeyCMP(enum.Enum):
 *     PUBLIC = c_ssh.SSH_KEY_CMP_PUBLIC             # <<<<<<<<<<<<<<
 *     PRIVATE = c_ssh.SSH_KEY_CMP_PRIVATE
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keycmp_e(SSH_KEY_CMP_PUBLIC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PUBLIC, __pyx_t_4) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":152
 * class KeyCMP(enum.Enum):
 *     PUBLIC = c_ssh.SSH_KEY_CMP_PUBLIC
 *     PRIVATE = c_ssh.SSH_KEY_CMP_PRIVATE             # <<<<<<<<<<<<<<
 * 
 * class Log(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_keycmp_e(SSH_KEY_CMP_PRIVATE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PRIVATE, __pyx_t_4) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":150
 * 
 *     # enum ssh_keycmp_e:
 * class KeyCMP(enum.Enum):             # <<<<<<<<<<<<<<
 *     PUBLIC = c_ssh.SSH_KEY_CMP_PUBLIC
 *     PRIVATE = c_ssh.SSH_KEY_CMP_PRIVATE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_KeyCMP, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_KeyCMP, __pyx_t_4) < 0) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":154
 *     PRIVATE = c_ssh.SSH_KEY_CMP_PRIVATE
 * 
 * class Log(enum.Enum):             # <<<<<<<<<<<<<<
 *     NOLOG = c_ssh.SSH_LOG_NOLOG
 *     PROTOCOL = c_ssh.SSH_LOG_PROTOCOL
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Log, __pyx_n_s_Log, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":155
 * 
 * class Log(enum.Enum):
 *     NOLOG = c_ssh.SSH_LOG_NOLOG             # <<<<<<<<<<<<<<
 *     PROTOCOL = c_ssh.SSH_LOG_PROTOCOL
 *     PACKET = c_ssh.SSH_LOG_PACKET
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_NOLOG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NOLOG, __pyx_t_4) < 0) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":156
 * class Log(enum.Enum):
 *     NOLOG = c_ssh.SSH_LOG_NOLOG
 *     PROTOCOL = c_ssh.SSH_LOG_PROTOCOL             # <<<<<<<<<<<<<<
 *     PACKET = c_ssh.SSH_LOG_PACKET
 *     FUNCTIONS = c_ssh.SSH_LOG_FUNCTIONS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_PROTOCOL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PROTOCOL, __pyx_t_4) < 0) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":157
 *     NOLOG = c_ssh.SSH_LOG_NOLOG
 *     PROTOCOL = c_ssh.SSH_LOG_PROTOCOL
 *     PACKET = c_ssh.SSH_LOG_PACKET             # <<<<<<<<<<<<<<
 *     FUNCTIONS = c_ssh.SSH_LOG_FUNCTIONS
 *     RARE = c_ssh.SSH_LOG_RARE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_PACKET); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PACKET, __pyx_t_4) < 0) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":158
 *     PROTOCOL = c_ssh.SSH_LOG_PROTOCOL
 *     PACKET = c_ssh.SSH_LOG_PACKET
 *     FUNCTIONS = c_ssh.SSH_LOG_FUNCTIONS             # <<<<<<<<<<<<<<
 *     RARE = c_ssh.SSH_LOG_RARE
 *     NONE = c_ssh.SSH_LOG_NONE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_FUNCTIONS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FUNCTIONS, __pyx_t_4) < 0) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":159
 *     PACKET = c_ssh.SSH_LOG_PACKET
 *     FUNCTIONS = c_ssh.SSH_LOG_FUNCTIONS
 *     RARE = c_ssh.SSH_LOG_RARE             # <<<<<<<<<<<<<<
 *     NONE = c_ssh.SSH_LOG_NONE
 *     WARN = c_ssh.SSH_LOG_WARN
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_RARE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RARE, __pyx_t_4) < 0) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":160
 *     FUNCTIONS = c_ssh.SSH_LOG_FUNCTIONS
 *     RARE = c_ssh.SSH_LOG_RARE
 *     NONE = c_ssh.SSH_LOG_NONE             # <<<<<<<<<<<<<<
 *     WARN = c_ssh.SSH_LOG_WARN
 *     INFO = c_ssh.SSH_LOG_INFO
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NONE, __pyx_t_4) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":161
 *     RARE = c_ssh.SSH_LOG_RARE
 *     NONE = c_ssh.SSH_LOG_NONE
 *     WARN = c_ssh.SSH_LOG_WARN             # <<<<<<<<<<<<<<
 *     INFO = c_ssh.SSH_LOG_INFO
 *     DEBUG = c_ssh.SSH_LOG_DEBUG
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_WARN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WARN, __pyx_t_4) < 0) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":162
 *     NONE = c_ssh.SSH_LOG_NONE
 *     WARN = c_ssh.SSH_LOG_WARN
 *     INFO = c_ssh.SSH_LOG_INFO             # <<<<<<<<<<<<<<
 *     DEBUG = c_ssh.SSH_LOG_DEBUG
 *     TRACE = c_ssh.SSH_LOG_TRACE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_INFO); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INFO, __pyx_t_4) < 0) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":163
 *     WARN = c_ssh.SSH_LOG_WARN
 *     INFO = c_ssh.SSH_LOG_INFO
 *     DEBUG = c_ssh.SSH_LOG_DEBUG             # <<<<<<<<<<<<<<
 *     TRACE = c_ssh.SSH_LOG_TRACE
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_DEBUG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DEBUG, __pyx_t_4) < 0) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":164
 *     INFO = c_ssh.SSH_LOG_INFO
 *     DEBUG = c_ssh.SSH_LOG_DEBUG
 *     TRACE = c_ssh.SSH_LOG_TRACE             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_options_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_LOG_TRACE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TRACE, __pyx_t_4) < 0) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":154
 *     PRIVATE = c_ssh.SSH_KEY_CMP_PRIVATE
 * 
 * class Log(enum.Enum):             # <<<<<<<<<<<<<<
 *     NOLOG = c_ssh.SSH_LOG_NOLOG
 *     PROTOCOL = c_ssh.SSH_LOG_PROTOCOL
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Log, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Log, __pyx_t_4) < 0) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":167
 * 
 *     # enum ssh_options_e:
 * class Options(enum.Enum):             # <<<<<<<<<<<<<<
 *     HOST = c_ssh.SSH_OPTIONS_HOST
 *     PORT = c_ssh.SSH_OPTIONS_PORT
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Options, __pyx_n_s_Options, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":168
 *     # enum ssh_options_e:
 * class Options(enum.Enum):
 *     HOST = c_ssh.SSH_OPTIONS_HOST             # <<<<<<<<<<<<<<
 *     PORT = c_ssh.SSH_OPTIONS_PORT
 *     PORT_STR = c_ssh.SSH_OPTIONS_PORT_STR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_HOST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOST, __pyx_t_4) < 0) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":169
 * class Options(enum.Enum):
 *     HOST = c_ssh.SSH_OPTIONS_HOST
 *     PORT = c_ssh.SSH_OPTIONS_PORT             # <<<<<<<<<<<<<<
 *     PORT_STR = c_ssh.SSH_OPTIONS_PORT_STR
 *     FD = c_ssh.SSH_OPTIONS_FD
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_PORT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PORT, __pyx_t_4) < 0) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":170
 *     HOST = c_ssh.SSH_OPTIONS_HOST
 *     PORT = c_ssh.SSH_OPTIONS_PORT
 *     PORT_STR = c_ssh.SSH_OPTIONS_PORT_STR             # <<<<<<<<<<<<<<
 *     FD = c_ssh.SSH_OPTIONS_FD
 *     USER = c_ssh.SSH_OPTIONS_USER
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_PORT_STR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PORT_STR, __pyx_t_4) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":171
 *     PORT = c_ssh.SSH_OPTIONS_PORT
 *     PORT_STR = c_ssh.SSH_OPTIONS_PORT_STR
 *     FD = c_ssh.SSH_OPTIONS_FD             # <<<<<<<<<<<<<<
 *     USER = c_ssh.SSH_OPTIONS_USER
 *     SSH_DIR = c_ssh.SSH_OPTIONS_SSH_DIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_FD); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FD, __pyx_t_4) < 0) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":172
 *     PORT_STR = c_ssh.SSH_OPTIONS_PORT_STR
 *     FD = c_ssh.SSH_OPTIONS_FD
 *     USER = c_ssh.SSH_OPTIONS_USER             # <<<<<<<<<<<<<<
 *     SSH_DIR = c_ssh.SSH_OPTIONS_SSH_DIR
 *     IDENTITY = c_ssh.SSH_OPTIONS_IDENTITY
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_USER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_USER, __pyx_t_4) < 0) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":173
 *     FD = c_ssh.SSH_OPTIONS_FD
 *     USER = c_ssh.SSH_OPTIONS_USER
 *     SSH_DIR = c_ssh.SSH_OPTIONS_SSH_DIR             # <<<<<<<<<<<<<<
 *     IDENTITY = c_ssh.SSH_OPTIONS_IDENTITY
 *     ADD_IDENTITY = c_ssh.SSH_OPTIONS_ADD_IDENTITY
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_SSH_DIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SSH_DIR, __pyx_t_4) < 0) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":174
 *     USER = c_ssh.SSH_OPTIONS_USER
 *     SSH_DIR = c_ssh.SSH_OPTIONS_SSH_DIR
 *     IDENTITY = c_ssh.SSH_OPTIONS_IDENTITY             # <<<<<<<<<<<<<<
 *     ADD_IDENTITY = c_ssh.SSH_OPTIONS_ADD_IDENTITY
 *     KNOWNHOSTS = c_ssh.SSH_OPTIONS_KNOWNHOSTS
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_IDENTITY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IDENTITY, __pyx_t_4) < 0) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":175
 *     SSH_DIR = c_ssh.SSH_OPTIONS_SSH_DIR
 *     IDENTITY = c_ssh.SSH_OPTIONS_IDENTITY
 *     ADD_IDENTITY = c_ssh.SSH_OPTIONS_ADD_IDENTITY             # <<<<<<<<<<<<<<
 *     KNOWNHOSTS = c_ssh.SSH_OPTIONS_KNOWNHOSTS
 *     TIMEOUT = c_ssh.SSH_OPTIONS_TIMEOUT
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_ADD_IDENTITY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ADD_IDENTITY, __pyx_t_4) < 0) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":176
 *     IDENTITY = c_ssh.SSH_OPTIONS_IDENTITY
 *     ADD_IDENTITY = c_ssh.SSH_OPTIONS_ADD_IDENTITY
 *     KNOWNHOSTS = c_ssh.SSH_OPTIONS_KNOWNHOSTS             # <<<<<<<<<<<<<<
 *     TIMEOUT = c_ssh.SSH_OPTIONS_TIMEOUT
 *     TIMEOUT_USEC = c_ssh.SSH_OPTIONS_TIMEOUT_USEC
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_KNOWNHOSTS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KNOWNHOSTS, __pyx_t_4) < 0) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":177
 *     ADD_IDENTITY = c_ssh.SSH_OPTIONS_ADD_IDENTITY
 *     KNOWNHOSTS = c_ssh.SSH_OPTIONS_KNOWNHOSTS
 *     TIMEOUT = c_ssh.SSH_OPTIONS_TIMEOUT             # <<<<<<<<<<<<<<
 *     TIMEOUT_USEC = c_ssh.SSH_OPTIONS_TIMEOUT_USEC
 *     SSH1 = c_ssh.SSH_OPTIONS_SSH1
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_TIMEOUT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TIMEOUT, __pyx_t_4) < 0) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":178
 *     KNOWNHOSTS = c_ssh.SSH_OPTIONS_KNOWNHOSTS
 *     TIMEOUT = c_ssh.SSH_OPTIONS_TIMEOUT
 *     TIMEOUT_USEC = c_ssh.SSH_OPTIONS_TIMEOUT_USEC             # <<<<<<<<<<<<<<
 *     SSH1 = c_ssh.SSH_OPTIONS_SSH1
 *     SSH2 = c_ssh.SSH_OPTIONS_SSH2
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_TIMEOUT_USEC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TIMEOUT_USEC, __pyx_t_4) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":179
 *     TIMEOUT = c_ssh.SSH_OPTIONS_TIMEOUT
 *     TIMEOUT_USEC = c_ssh.SSH_OPTIONS_TIMEOUT_USEC
 *     SSH1 = c_ssh.SSH_OPTIONS_SSH1             # <<<<<<<<<<<<<<
 *     SSH2 = c_ssh.SSH_OPTIONS_SSH2
 *     LOG_VERBOSITY = c_ssh.SSH_OPTIONS_LOG_VERBOSITY
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_SSH1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SSH1, __pyx_t_4) < 0) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":180
 *     TIMEOUT_USEC = c_ssh.SSH_OPTIONS_TIMEOUT_USEC
 *     SSH1 = c_ssh.SSH_OPTIONS_SSH1
 *     SSH2 = c_ssh.SSH_OPTIONS_SSH2             # <<<<<<<<<<<<<<
 *     LOG_VERBOSITY = c_ssh.SSH_OPTIONS_LOG_VERBOSITY
 *     LOG_VERBOSITY_STR = c_ssh.SSH_OPTIONS_LOG_VERBOSITY_STR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_SSH2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SSH2, __pyx_t_4) < 0) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":181
 *     SSH1 = c_ssh.SSH_OPTIONS_SSH1
 *     SSH2 = c_ssh.SSH_OPTIONS_SSH2
 *     LOG_VERBOSITY = c_ssh.SSH_OPTIONS_LOG_VERBOSITY             # <<<<<<<<<<<<<<
 *     LOG_VERBOSITY_STR = c_ssh.SSH_OPTIONS_LOG_VERBOSITY_STR
 *     CIPHERS_C_S = c_ssh.SSH_OPTIONS_CIPHERS_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_LOG_VERBOSITY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LOG_VERBOSITY, __pyx_t_4) < 0) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":182
 *     SSH2 = c_ssh.SSH_OPTIONS_SSH2
 *     LOG_VERBOSITY = c_ssh.SSH_OPTIONS_LOG_VERBOSITY
 *     LOG_VERBOSITY_STR = c_ssh.SSH_OPTIONS_LOG_VERBOSITY_STR             # <<<<<<<<<<<<<<
 *     CIPHERS_C_S = c_ssh.SSH_OPTIONS_CIPHERS_C_S
 *     CIPHERS_S_C = c_ssh.SSH_OPTIONS_CIPHERS_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_LOG_VERBOSITY_STR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LOG_VERBOSITY_STR, __pyx_t_4) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":183
 *     LOG_VERBOSITY = c_ssh.SSH_OPTIONS_LOG_VERBOSITY
 *     LOG_VERBOSITY_STR = c_ssh.SSH_OPTIONS_LOG_VERBOSITY_STR
 *     CIPHERS_C_S = c_ssh.SSH_OPTIONS_CIPHERS_C_S             # <<<<<<<<<<<<<<
 *     CIPHERS_S_C = c_ssh.SSH_OPTIONS_CIPHERS_S_C
 *     COMPRESSION_C_S = c_ssh.SSH_OPTIONS_COMPRESSION_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_CIPHERS_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CIPHERS_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":184
 *     LOG_VERBOSITY_STR = c_ssh.SSH_OPTIONS_LOG_VERBOSITY_STR
 *     CIPHERS_C_S = c_ssh.SSH_OPTIONS_CIPHERS_C_S
 *     CIPHERS_S_C = c_ssh.SSH_OPTIONS_CIPHERS_S_C             # <<<<<<<<<<<<<<
 *     COMPRESSION_C_S = c_ssh.SSH_OPTIONS_COMPRESSION_C_S
 *     COMPRESSION_S_C = c_ssh.SSH_OPTIONS_COMPRESSION_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_CIPHERS_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CIPHERS_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":185
 *     CIPHERS_C_S = c_ssh.SSH_OPTIONS_CIPHERS_C_S
 *     CIPHERS_S_C = c_ssh.SSH_OPTIONS_CIPHERS_S_C
 *     COMPRESSION_C_S = c_ssh.SSH_OPTIONS_COMPRESSION_C_S             # <<<<<<<<<<<<<<
 *     COMPRESSION_S_C = c_ssh.SSH_OPTIONS_COMPRESSION_S_C
 *     PROXYCOMMAND = c_ssh.SSH_OPTIONS_PROXYCOMMAND
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_COMPRESSION_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMPRESSION_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":186
 *     CIPHERS_S_C = c_ssh.SSH_OPTIONS_CIPHERS_S_C
 *     COMPRESSION_C_S = c_ssh.SSH_OPTIONS_COMPRESSION_C_S
 *     COMPRESSION_S_C = c_ssh.SSH_OPTIONS_COMPRESSION_S_C             # <<<<<<<<<<<<<<
 *     PROXYCOMMAND = c_ssh.SSH_OPTIONS_PROXYCOMMAND
 *     BINDADDR = c_ssh.SSH_OPTIONS_BINDADDR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_COMPRESSION_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMPRESSION_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":187
 *     COMPRESSION_C_S = c_ssh.SSH_OPTIONS_COMPRESSION_C_S
 *     COMPRESSION_S_C = c_ssh.SSH_OPTIONS_COMPRESSION_S_C
 *     PROXYCOMMAND = c_ssh.SSH_OPTIONS_PROXYCOMMAND             # <<<<<<<<<<<<<<
 *     BINDADDR = c_ssh.SSH_OPTIONS_BINDADDR
 *     STRICTHOSTKEYCHECK = c_ssh.SSH_OPTIONS_STRICTHOSTKEYCHECK
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_PROXYCOMMAND); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PROXYCOMMAND, __pyx_t_4) < 0) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":188
 *     COMPRESSION_S_C = c_ssh.SSH_OPTIONS_COMPRESSION_S_C
 *     PROXYCOMMAND = c_ssh.SSH_OPTIONS_PROXYCOMMAND
 *     BINDADDR = c_ssh.SSH_OPTIONS_BINDADDR             # <<<<<<<<<<<<<<
 *     STRICTHOSTKEYCHECK = c_ssh.SSH_OPTIONS_STRICTHOSTKEYCHECK
 *     COMPRESSION = c_ssh.SSH_OPTIONS_COMPRESSION
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_BINDADDR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_BINDADDR, __pyx_t_4) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":189
 *     PROXYCOMMAND = c_ssh.SSH_OPTIONS_PROXYCOMMAND
 *     BINDADDR = c_ssh.SSH_OPTIONS_BINDADDR
 *     STRICTHOSTKEYCHECK = c_ssh.SSH_OPTIONS_STRICTHOSTKEYCHECK             # <<<<<<<<<<<<<<
 *     COMPRESSION = c_ssh.SSH_OPTIONS_COMPRESSION
 *     COMPRESSION_LEVEL = c_ssh.SSH_OPTIONS_COMPRESSION_LEVEL
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_STRICTHOSTKEYCHECK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STRICTHOSTKEYCHECK, __pyx_t_4) < 0) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":190
 *     BINDADDR = c_ssh.SSH_OPTIONS_BINDADDR
 *     STRICTHOSTKEYCHECK = c_ssh.SSH_OPTIONS_STRICTHOSTKEYCHECK
 *     COMPRESSION = c_ssh.SSH_OPTIONS_COMPRESSION             # <<<<<<<<<<<<<<
 *     COMPRESSION_LEVEL = c_ssh.SSH_OPTIONS_COMPRESSION_LEVEL
 *     KEY_EXCHANGE = c_ssh.SSH_OPTIONS_KEY_EXCHANGE
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_COMPRESSION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMPRESSION, __pyx_t_4) < 0) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":191
 *     STRICTHOSTKEYCHECK = c_ssh.SSH_OPTIONS_STRICTHOSTKEYCHECK
 *     COMPRESSION = c_ssh.SSH_OPTIONS_COMPRESSION
 *     COMPRESSION_LEVEL = c_ssh.SSH_OPTIONS_COMPRESSION_LEVEL             # <<<<<<<<<<<<<<
 *     KEY_EXCHANGE = c_ssh.SSH_OPTIONS_KEY_EXCHANGE
 *     HOSTKEYS = c_ssh.SSH_OPTIONS_HOSTKEYS
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_COMPRESSION_LEVEL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMPRESSION_LEVEL, __pyx_t_4) < 0) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":192
 *     COMPRESSION = c_ssh.SSH_OPTIONS_COMPRESSION
 *     COMPRESSION_LEVEL = c_ssh.SSH_OPTIONS_COMPRESSION_LEVEL
 *     KEY_EXCHANGE = c_ssh.SSH_OPTIONS_KEY_EXCHANGE             # <<<<<<<<<<<<<<
 *     HOSTKEYS = c_ssh.SSH_OPTIONS_HOSTKEYS
 *     GSSAPI_SERVER_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_SERVER_IDENTITY
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_KEY_EXCHANGE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEY_EXCHANGE, __pyx_t_4) < 0) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":193
 *     COMPRESSION_LEVEL = c_ssh.SSH_OPTIONS_COMPRESSION_LEVEL
 *     KEY_EXCHANGE = c_ssh.SSH_OPTIONS_KEY_EXCHANGE
 *     HOSTKEYS = c_ssh.SSH_OPTIONS_HOSTKEYS             # <<<<<<<<<<<<<<
 *     GSSAPI_SERVER_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_SERVER_IDENTITY
 *     GSSAPI_CLIENT_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_CLIENT_IDENTITY
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_HOSTKEYS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOSTKEYS, __pyx_t_4) < 0) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":194
 *     KEY_EXCHANGE = c_ssh.SSH_OPTIONS_KEY_EXCHANGE
 *     HOSTKEYS = c_ssh.SSH_OPTIONS_HOSTKEYS
 *     GSSAPI_SERVER_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_SERVER_IDENTITY             # <<<<<<<<<<<<<<
 *     GSSAPI_CLIENT_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_CLIENT_IDENTITY
 *     GSSAPI_DELEGATE_CREDENTIALS = c_ssh.SSH_OPTIONS_GSSAPI_DELEGATE_CREDENTIALS
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_GSSAPI_SERVER_IDENTITY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_SERVER_IDENTITY, __pyx_t_4) < 0) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":195
 *     HOSTKEYS = c_ssh.SSH_OPTIONS_HOSTKEYS
 *     GSSAPI_SERVER_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_SERVER_IDENTITY
 *     GSSAPI_CLIENT_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_CLIENT_IDENTITY             # <<<<<<<<<<<<<<
 *     GSSAPI_DELEGATE_CREDENTIALS = c_ssh.SSH_OPTIONS_GSSAPI_DELEGATE_CREDENTIALS
 *     HMAC_C_S = c_ssh.SSH_OPTIONS_HMAC_C_S
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_GSSAPI_CLIENT_IDENTITY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_CLIENT_IDENTITY, __pyx_t_4) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":196
 *     GSSAPI_SERVER_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_SERVER_IDENTITY
 *     GSSAPI_CLIENT_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_CLIENT_IDENTITY
 *     GSSAPI_DELEGATE_CREDENTIALS = c_ssh.SSH_OPTIONS_GSSAPI_DELEGATE_CREDENTIALS             # <<<<<<<<<<<<<<
 *     HMAC_C_S = c_ssh.SSH_OPTIONS_HMAC_C_S
 *     HMAC_S_C = c_ssh.SSH_OPTIONS_HMAC_S_C
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_GSSAPI_DELEGATE_CREDENTIALS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_DELEGATE_CREDENTIALS, __pyx_t_4) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":197
 *     GSSAPI_CLIENT_IDENTITY = c_ssh.SSH_OPTIONS_GSSAPI_CLIENT_IDENTITY
 *     GSSAPI_DELEGATE_CREDENTIALS = c_ssh.SSH_OPTIONS_GSSAPI_DELEGATE_CREDENTIALS
 *     HMAC_C_S = c_ssh.SSH_OPTIONS_HMAC_C_S             # <<<<<<<<<<<<<<
 *     HMAC_S_C = c_ssh.SSH_OPTIONS_HMAC_S_C
 *     PASSWORD_AUTH = c_ssh.SSH_OPTIONS_PASSWORD_AUTH
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_HMAC_C_S); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HMAC_C_S, __pyx_t_4) < 0) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":198
 *     GSSAPI_DELEGATE_CREDENTIALS = c_ssh.SSH_OPTIONS_GSSAPI_DELEGATE_CREDENTIALS
 *     HMAC_C_S = c_ssh.SSH_OPTIONS_HMAC_C_S
 *     HMAC_S_C = c_ssh.SSH_OPTIONS_HMAC_S_C             # <<<<<<<<<<<<<<
 *     PASSWORD_AUTH = c_ssh.SSH_OPTIONS_PASSWORD_AUTH
 *     PUBKEY_AUTH = c_ssh.SSH_OPTIONS_PUBKEY_AUTH
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_HMAC_S_C); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HMAC_S_C, __pyx_t_4) < 0) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":199
 *     HMAC_C_S = c_ssh.SSH_OPTIONS_HMAC_C_S
 *     HMAC_S_C = c_ssh.SSH_OPTIONS_HMAC_S_C
 *     PASSWORD_AUTH = c_ssh.SSH_OPTIONS_PASSWORD_AUTH             # <<<<<<<<<<<<<<
 *     PUBKEY_AUTH = c_ssh.SSH_OPTIONS_PUBKEY_AUTH
 *     KBDINT_AUTH = c_ssh.SSH_OPTIONS_KBDINT_AUTH
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_PASSWORD_AUTH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PASSWORD_AUTH, __pyx_t_4) < 0) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":200
 *     HMAC_S_C = c_ssh.SSH_OPTIONS_HMAC_S_C
 *     PASSWORD_AUTH = c_ssh.SSH_OPTIONS_PASSWORD_AUTH
 *     PUBKEY_AUTH = c_ssh.SSH_OPTIONS_PUBKEY_AUTH             # <<<<<<<<<<<<<<
 *     KBDINT_AUTH = c_ssh.SSH_OPTIONS_KBDINT_AUTH
 *     GSSAPI_AUTH = c_ssh.SSH_OPTIONS_GSSAPI_AUTH
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_PUBKEY_AUTH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PUBKEY_AUTH, __pyx_t_4) < 0) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":201
 *     PASSWORD_AUTH = c_ssh.SSH_OPTIONS_PASSWORD_AUTH
 *     PUBKEY_AUTH = c_ssh.SSH_OPTIONS_PUBKEY_AUTH
 *     KBDINT_AUTH = c_ssh.SSH_OPTIONS_KBDINT_AUTH             # <<<<<<<<<<<<<<
 *     GSSAPI_AUTH = c_ssh.SSH_OPTIONS_GSSAPI_AUTH
 *     GLOBAL_KNOWNHOSTS = c_ssh.SSH_OPTIONS_GLOBAL_KNOWNHOSTS
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_KBDINT_AUTH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KBDINT_AUTH, __pyx_t_4) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":202
 *     PUBKEY_AUTH = c_ssh.SSH_OPTIONS_PUBKEY_AUTH
 *     KBDINT_AUTH = c_ssh.SSH_OPTIONS_KBDINT_AUTH
 *     GSSAPI_AUTH = c_ssh.SSH_OPTIONS_GSSAPI_AUTH             # <<<<<<<<<<<<<<
 *     GLOBAL_KNOWNHOSTS = c_ssh.SSH_OPTIONS_GLOBAL_KNOWNHOSTS
 *     NODELAY = c_ssh.SSH_OPTIONS_NODELAY
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_GSSAPI_AUTH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_AUTH, __pyx_t_4) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":203
 *     KBDINT_AUTH = c_ssh.SSH_OPTIONS_KBDINT_AUTH
 *     GSSAPI_AUTH = c_ssh.SSH_OPTIONS_GSSAPI_AUTH
 *     GLOBAL_KNOWNHOSTS = c_ssh.SSH_OPTIONS_GLOBAL_KNOWNHOSTS             # <<<<<<<<<<<<<<
 *     NODELAY = c_ssh.SSH_OPTIONS_NODELAY
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_GLOBAL_KNOWNHOSTS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GLOBAL_KNOWNHOSTS, __pyx_t_4) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":204
 *     GSSAPI_AUTH = c_ssh.SSH_OPTIONS_GSSAPI_AUTH
 *     GLOBAL_KNOWNHOSTS = c_ssh.SSH_OPTIONS_GLOBAL_KNOWNHOSTS
 *     NODELAY = c_ssh.SSH_OPTIONS_NODELAY             # <<<<<<<<<<<<<<
 * 
 * class SCP(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_options_e(SSH_OPTIONS_NODELAY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NODELAY, __pyx_t_4) < 0) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":167
 * 
 *     # enum ssh_options_e:
 * class Options(enum.Enum):             # <<<<<<<<<<<<<<
 *     HOST = c_ssh.SSH_OPTIONS_HOST
 *     PORT = c_ssh.SSH_OPTIONS_PORT
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Options, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Options, __pyx_t_4) < 0) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":206
 *     NODELAY = c_ssh.SSH_OPTIONS_NODELAY
 * 
 * class SCP(enum.Enum):             # <<<<<<<<<<<<<<
 *     WRITE = c_ssh.SSH_SCP_WRITE
 *     READ = c_ssh.SSH_SCP_READ
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SCP, __pyx_n_s_SCP, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":207
 * 
 * class SCP(enum.Enum):
 *     WRITE = c_ssh.SSH_SCP_WRITE             # <<<<<<<<<<<<<<
 *     READ = c_ssh.SSH_SCP_READ
 *     RECURSIVE = c_ssh.SSH_SCP_RECURSIVE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SCP_WRITE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRITE, __pyx_t_4) < 0) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":208
 * class SCP(enum.Enum):
 *     WRITE = c_ssh.SSH_SCP_WRITE
 *     READ = c_ssh.SSH_SCP_READ             # <<<<<<<<<<<<<<
 *     RECURSIVE = c_ssh.SSH_SCP_RECURSIVE
 *         # enum ssh_scp_request_types:
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SCP_READ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READ, __pyx_t_4) < 0) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":209
 *     WRITE = c_ssh.SSH_SCP_WRITE
 *     READ = c_ssh.SSH_SCP_READ
 *     RECURSIVE = c_ssh.SSH_SCP_RECURSIVE             # <<<<<<<<<<<<<<
 *         # enum ssh_scp_request_types:
 * class SCP_Request(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SCP_RECURSIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RECURSIVE, __pyx_t_4) < 0) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":206
 *     NODELAY = c_ssh.SSH_OPTIONS_NODELAY
 * 
 * class SCP(enum.Enum):             # <<<<<<<<<<<<<<
 *     WRITE = c_ssh.SSH_SCP_WRITE
 *     READ = c_ssh.SSH_SCP_READ
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SCP, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SCP, __pyx_t_4) < 0) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":211
 *     RECURSIVE = c_ssh.SSH_SCP_RECURSIVE
 *         # enum ssh_scp_request_types:
 * class SCP_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     NEWDIR = c_ssh.SSH_SCP_REQUEST_NEWDIR
 *     NEWFILE = c_ssh.SSH_SCP_REQUEST_NEWFILE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SCP_Request, __pyx_n_s_SCP_Request, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":212
 *         # enum ssh_scp_request_types:
 * class SCP_Request(enum.Enum):
 *     NEWDIR = c_ssh.SSH_SCP_REQUEST_NEWDIR             # <<<<<<<<<<<<<<
 *     NEWFILE = c_ssh.SSH_SCP_REQUEST_NEWFILE
 *     EOF = c_ssh.SSH_SCP_REQUEST_EOF
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_scp_request_types(SSH_SCP_REQUEST_NEWDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NEWDIR, __pyx_t_4) < 0) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":213
 * class SCP_Request(enum.Enum):
 *     NEWDIR = c_ssh.SSH_SCP_REQUEST_NEWDIR
 *     NEWFILE = c_ssh.SSH_SCP_REQUEST_NEWFILE             # <<<<<<<<<<<<<<
 *     EOF = c_ssh.SSH_SCP_REQUEST_EOF
 *     ENDDIR = c_ssh.SSH_SCP_REQUEST_ENDDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_scp_request_types(SSH_SCP_REQUEST_NEWFILE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NEWFILE, __pyx_t_4) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":214
 *     NEWDIR = c_ssh.SSH_SCP_REQUEST_NEWDIR
 *     NEWFILE = c_ssh.SSH_SCP_REQUEST_NEWFILE
 *     EOF = c_ssh.SSH_SCP_REQUEST_EOF             # <<<<<<<<<<<<<<
 *     ENDDIR = c_ssh.SSH_SCP_REQUEST_ENDDIR
 *     WARNING = c_ssh.SSH_SCP_REQUEST_WARNING
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_scp_request_types(SSH_SCP_REQUEST_EOF); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EOF, __pyx_t_4) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":215
 *     NEWFILE = c_ssh.SSH_SCP_REQUEST_NEWFILE
 *     EOF = c_ssh.SSH_SCP_REQUEST_EOF
 *     ENDDIR = c_ssh.SSH_SCP_REQUEST_ENDDIR             # <<<<<<<<<<<<<<
 *     WARNING = c_ssh.SSH_SCP_REQUEST_WARNING
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_scp_request_types(SSH_SCP_REQUEST_ENDDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ENDDIR, __pyx_t_4) < 0) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":216
 *     EOF = c_ssh.SSH_SCP_REQUEST_EOF
 *     ENDDIR = c_ssh.SSH_SCP_REQUEST_ENDDIR
 *     WARNING = c_ssh.SSH_SCP_REQUEST_WARNING             # <<<<<<<<<<<<<<
 * 
 *     # enum ssh_connector_flags_e:
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_scp_request_types(SSH_SCP_REQUEST_WARNING); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WARNING, __pyx_t_4) < 0) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":211
 *     RECURSIVE = c_ssh.SSH_SCP_RECURSIVE
 *         # enum ssh_scp_request_types:
 * class SCP_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     NEWDIR = c_ssh.SSH_SCP_REQUEST_NEWDIR
 *     NEWFILE = c_ssh.SSH_SCP_REQUEST_NEWFILE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SCP_Request, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SCP_Request, __pyx_t_4) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":219
 * 
 *     # enum ssh_connector_flags_e:
 * class Connector(enum.Enum):             # <<<<<<<<<<<<<<
 *     STDOUT = c_ssh.SSH_CONNECTOR_STDOUT
 *     STDERR = c_ssh.SSH_CONNECTOR_STDERR
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Connector, __pyx_n_s_Connector, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":220
 *     # enum ssh_connector_flags_e:
 * class Connector(enum.Enum):
 *     STDOUT = c_ssh.SSH_CONNECTOR_STDOUT             # <<<<<<<<<<<<<<
 *     STDERR = c_ssh.SSH_CONNECTOR_STDERR
 *     BOTH = c_ssh.SSH_CONNECTOR_BOTH
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_connector_flags_e(SSH_CONNECTOR_STDOUT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STDOUT, __pyx_t_4) < 0) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":221
 * class Connector(enum.Enum):
 *     STDOUT = c_ssh.SSH_CONNECTOR_STDOUT
 *     STDERR = c_ssh.SSH_CONNECTOR_STDERR             # <<<<<<<<<<<<<<
 *     BOTH = c_ssh.SSH_CONNECTOR_BOTH
 * #callbacks
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_connector_flags_e(SSH_CONNECTOR_STDERR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STDERR, __pyx_t_4) < 0) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":222
 *     STDOUT = c_ssh.SSH_CONNECTOR_STDOUT
 *     STDERR = c_ssh.SSH_CONNECTOR_STDERR
 *     BOTH = c_ssh.SSH_CONNECTOR_BOTH             # <<<<<<<<<<<<<<
 * #callbacks
 * class Callbacks_Socket(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_enum__ssh_connector_flags_e(SSH_CONNECTOR_BOTH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_BOTH, __pyx_t_4) < 0) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":219
 * 
 *     # enum ssh_connector_flags_e:
 * class Connector(enum.Enum):             # <<<<<<<<<<<<<<
 *     STDOUT = c_ssh.SSH_CONNECTOR_STDOUT
 *     STDERR = c_ssh.SSH_CONNECTOR_STDERR
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Connector, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Connector, __pyx_t_4) < 0) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":224
 *     BOTH = c_ssh.SSH_CONNECTOR_BOTH
 * #callbacks
 * class Callbacks_Socket(enum.Enum):             # <<<<<<<<<<<<<<
 *     FLOW_WRITEWILLBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWILLBLOCK
 *     FLOW_WRITEWONTBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWONTBLOCK
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_Callbacks_Socket, __pyx_n_s_Callbacks_Socket, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":225
 * #callbacks
 * class Callbacks_Socket(enum.Enum):
 *     FLOW_WRITEWILLBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWILLBLOCK             # <<<<<<<<<<<<<<
 *     FLOW_WRITEWONTBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWONTBLOCK
 *     EXCEPTION_EOF = c_callbacks.SSH_SOCKET_EXCEPTION_EOF
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_FLOW_WRITEWILLBLOCK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FLOW_WRITEWILLBLOCK, __pyx_t_4) < 0) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":226
 * class Callbacks_Socket(enum.Enum):
 *     FLOW_WRITEWILLBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWILLBLOCK
 *     FLOW_WRITEWONTBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWONTBLOCK             # <<<<<<<<<<<<<<
 *     EXCEPTION_EOF = c_callbacks.SSH_SOCKET_EXCEPTION_EOF
 *     EXCEPTION_ERROR = c_callbacks.SSH_SOCKET_EXCEPTION_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_FLOW_WRITEWONTBLOCK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FLOW_WRITEWONTBLOCK, __pyx_t_4) < 0) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":227
 *     FLOW_WRITEWILLBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWILLBLOCK
 *     FLOW_WRITEWONTBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWONTBLOCK
 *     EXCEPTION_EOF = c_callbacks.SSH_SOCKET_EXCEPTION_EOF             # <<<<<<<<<<<<<<
 *     EXCEPTION_ERROR = c_callbacks.SSH_SOCKET_EXCEPTION_ERROR
 *     CONNECTED_OK = c_callbacks.SSH_SOCKET_CONNECTED_OK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_EXCEPTION_EOF); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXCEPTION_EOF, __pyx_t_4) < 0) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":228
 *     FLOW_WRITEWONTBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWONTBLOCK
 *     EXCEPTION_EOF = c_callbacks.SSH_SOCKET_EXCEPTION_EOF
 *     EXCEPTION_ERROR = c_callbacks.SSH_SOCKET_EXCEPTION_ERROR             # <<<<<<<<<<<<<<
 *     CONNECTED_OK = c_callbacks.SSH_SOCKET_CONNECTED_OK
 *     CONNECTED_ERROR = c_callbacks.SSH_SOCKET_CONNECTED_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_EXCEPTION_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXCEPTION_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":229
 *     EXCEPTION_EOF = c_callbacks.SSH_SOCKET_EXCEPTION_EOF
 *     EXCEPTION_ERROR = c_callbacks.SSH_SOCKET_EXCEPTION_ERROR
 *     CONNECTED_OK = c_callbacks.SSH_SOCKET_CONNECTED_OK             # <<<<<<<<<<<<<<
 *     CONNECTED_ERROR = c_callbacks.SSH_SOCKET_CONNECTED_ERROR
 *     CONNECTED_TIMEOUT = c_callbacks.SSH_SOCKET_CONNECTED_TIMEOUT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_CONNECTED_OK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CONNECTED_OK, __pyx_t_4) < 0) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":230
 *     EXCEPTION_ERROR = c_callbacks.SSH_SOCKET_EXCEPTION_ERROR
 *     CONNECTED_OK = c_callbacks.SSH_SOCKET_CONNECTED_OK
 *     CONNECTED_ERROR = c_callbacks.SSH_SOCKET_CONNECTED_ERROR             # <<<<<<<<<<<<<<
 *     CONNECTED_TIMEOUT = c_callbacks.SSH_SOCKET_CONNECTED_TIMEOUT
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_CONNECTED_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CONNECTED_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":231
 *     CONNECTED_OK = c_callbacks.SSH_SOCKET_CONNECTED_OK
 *     CONNECTED_ERROR = c_callbacks.SSH_SOCKET_CONNECTED_ERROR
 *     CONNECTED_TIMEOUT = c_callbacks.SSH_SOCKET_CONNECTED_TIMEOUT             # <<<<<<<<<<<<<<
 * 
 * #ssh2
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_SOCKET_CONNECTED_TIMEOUT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CONNECTED_TIMEOUT, __pyx_t_4) < 0) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":224
 *     BOTH = c_ssh.SSH_CONNECTOR_BOTH
 * #callbacks
 * class Callbacks_Socket(enum.Enum):             # <<<<<<<<<<<<<<
 *     FLOW_WRITEWILLBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWILLBLOCK
 *     FLOW_WRITEWONTBLOCK = c_callbacks.SSH_SOCKET_FLOW_WRITEWONTBLOCK
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_Callbacks_Socket, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Callbacks_Socket, __pyx_t_4) < 0) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":234
 * 
 * #ssh2
 * class SSH2_Msg(enum.Enum):             # <<<<<<<<<<<<<<
 *     DISCONNECT = c_ssh2.SSH2_MSG_DISCONNECT
 *     IGNORE = c_ssh2.SSH2_MSG_IGNORE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Msg, __pyx_n_s_SSH2_Msg, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":235
 * #ssh2
 * class SSH2_Msg(enum.Enum):
 *     DISCONNECT = c_ssh2.SSH2_MSG_DISCONNECT             # <<<<<<<<<<<<<<
 *     IGNORE = c_ssh2.SSH2_MSG_IGNORE
 *     UNIMPLEMENTED = c_ssh2.SSH2_MSG_UNIMPLEMENTED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_DISCONNECT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DISCONNECT, __pyx_t_4) < 0) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":236
 * class SSH2_Msg(enum.Enum):
 *     DISCONNECT = c_ssh2.SSH2_MSG_DISCONNECT
 *     IGNORE = c_ssh2.SSH2_MSG_IGNORE             # <<<<<<<<<<<<<<
 *     UNIMPLEMENTED = c_ssh2.SSH2_MSG_UNIMPLEMENTED
 *     DEBUG = c_ssh2.SSH2_MSG_DEBUG
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_IGNORE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IGNORE, __pyx_t_4) < 0) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":237
 *     DISCONNECT = c_ssh2.SSH2_MSG_DISCONNECT
 *     IGNORE = c_ssh2.SSH2_MSG_IGNORE
 *     UNIMPLEMENTED = c_ssh2.SSH2_MSG_UNIMPLEMENTED             # <<<<<<<<<<<<<<
 *     DEBUG = c_ssh2.SSH2_MSG_DEBUG
 *     SERVICE_REQUEST = c_ssh2.SSH2_MSG_SERVICE_REQUEST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_UNIMPLEMENTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNIMPLEMENTED, __pyx_t_4) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":238
 *     IGNORE = c_ssh2.SSH2_MSG_IGNORE
 *     UNIMPLEMENTED = c_ssh2.SSH2_MSG_UNIMPLEMENTED
 *     DEBUG = c_ssh2.SSH2_MSG_DEBUG             # <<<<<<<<<<<<<<
 *     SERVICE_REQUEST = c_ssh2.SSH2_MSG_SERVICE_REQUEST
 *     SERVICE_ACCEPT = c_ssh2.SSH2_MSG_SERVICE_ACCEPT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_DEBUG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DEBUG, __pyx_t_4) < 0) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":239
 *     UNIMPLEMENTED = c_ssh2.SSH2_MSG_UNIMPLEMENTED
 *     DEBUG = c_ssh2.SSH2_MSG_DEBUG
 *     SERVICE_REQUEST = c_ssh2.SSH2_MSG_SERVICE_REQUEST             # <<<<<<<<<<<<<<
 *     SERVICE_ACCEPT = c_ssh2.SSH2_MSG_SERVICE_ACCEPT
 *     KEXINIT = c_ssh2.SSH2_MSG_KEXINIT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_SERVICE_REQUEST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SERVICE_REQUEST, __pyx_t_4) < 0) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":240
 *     DEBUG = c_ssh2.SSH2_MSG_DEBUG
 *     SERVICE_REQUEST = c_ssh2.SSH2_MSG_SERVICE_REQUEST
 *     SERVICE_ACCEPT = c_ssh2.SSH2_MSG_SERVICE_ACCEPT             # <<<<<<<<<<<<<<
 *     KEXINIT = c_ssh2.SSH2_MSG_KEXINIT
 *     NEWKEYS = c_ssh2.SSH2_MSG_NEWKEYS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_SERVICE_ACCEPT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SERVICE_ACCEPT, __pyx_t_4) < 0) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":241
 *     SERVICE_REQUEST = c_ssh2.SSH2_MSG_SERVICE_REQUEST
 *     SERVICE_ACCEPT = c_ssh2.SSH2_MSG_SERVICE_ACCEPT
 *     KEXINIT = c_ssh2.SSH2_MSG_KEXINIT             # <<<<<<<<<<<<<<
 *     NEWKEYS = c_ssh2.SSH2_MSG_NEWKEYS
 *     ECMQV_INIT = c_ssh2.SSH2_MSG_ECMQV_INIT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEXINIT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEXINIT, __pyx_t_4) < 0) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":242
 *     SERVICE_ACCEPT = c_ssh2.SSH2_MSG_SERVICE_ACCEPT
 *     KEXINIT = c_ssh2.SSH2_MSG_KEXINIT
 *     NEWKEYS = c_ssh2.SSH2_MSG_NEWKEYS             # <<<<<<<<<<<<<<
 *     ECMQV_INIT = c_ssh2.SSH2_MSG_ECMQV_INIT
 *     ECMQV_REPLY = c_ssh2.SSH2_MSG_ECMQV_REPLY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_NEWKEYS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NEWKEYS, __pyx_t_4) < 0) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":243
 *     KEXINIT = c_ssh2.SSH2_MSG_KEXINIT
 *     NEWKEYS = c_ssh2.SSH2_MSG_NEWKEYS
 *     ECMQV_INIT = c_ssh2.SSH2_MSG_ECMQV_INIT             # <<<<<<<<<<<<<<
 *     ECMQV_REPLY = c_ssh2.SSH2_MSG_ECMQV_REPLY
 *     KEXDH_INIT = c_ssh2.SSH2_MSG_KEXDH_INIT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_ECMQV_INIT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECMQV_INIT, __pyx_t_4) < 0) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":244
 *     NEWKEYS = c_ssh2.SSH2_MSG_NEWKEYS
 *     ECMQV_INIT = c_ssh2.SSH2_MSG_ECMQV_INIT
 *     ECMQV_REPLY = c_ssh2.SSH2_MSG_ECMQV_REPLY             # <<<<<<<<<<<<<<
 *     KEXDH_INIT = c_ssh2.SSH2_MSG_KEXDH_INIT
 *     KEXDH_REPLY = c_ssh2.SSH2_MSG_KEXDH_REPLY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_ECMQV_REPLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECMQV_REPLY, __pyx_t_4) < 0) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":245
 *     ECMQV_INIT = c_ssh2.SSH2_MSG_ECMQV_INIT
 *     ECMQV_REPLY = c_ssh2.SSH2_MSG_ECMQV_REPLY
 *     KEXDH_INIT = c_ssh2.SSH2_MSG_KEXDH_INIT             # <<<<<<<<<<<<<<
 *     KEXDH_REPLY = c_ssh2.SSH2_MSG_KEXDH_REPLY
 * class SSH2_Msg_Kex(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEXDH_INIT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEXDH_INIT, __pyx_t_4) < 0) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":246
 *     ECMQV_REPLY = c_ssh2.SSH2_MSG_ECMQV_REPLY
 *     KEXDH_INIT = c_ssh2.SSH2_MSG_KEXDH_INIT
 *     KEXDH_REPLY = c_ssh2.SSH2_MSG_KEXDH_REPLY             # <<<<<<<<<<<<<<
 * class SSH2_Msg_Kex(enum.Enum):
 *     ECDH_INIT = c_ssh2.SSH2_MSG_KEX_ECDH_INIT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEXDH_REPLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEXDH_REPLY, __pyx_t_4) < 0) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":234
 * 
 * #ssh2
 * class SSH2_Msg(enum.Enum):             # <<<<<<<<<<<<<<
 *     DISCONNECT = c_ssh2.SSH2_MSG_DISCONNECT
 *     IGNORE = c_ssh2.SSH2_MSG_IGNORE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Msg, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Msg, __pyx_t_4) < 0) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":247
 *     KEXDH_INIT = c_ssh2.SSH2_MSG_KEXDH_INIT
 *     KEXDH_REPLY = c_ssh2.SSH2_MSG_KEXDH_REPLY
 * class SSH2_Msg_Kex(enum.Enum):             # <<<<<<<<<<<<<<
 *     ECDH_INIT = c_ssh2.SSH2_MSG_KEX_ECDH_INIT
 *     ECDH_REPLY = c_ssh2.SSH2_MSG_KEX_ECDH_REPLY
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Msg_Kex, __pyx_n_s_SSH2_Msg_Kex, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":248
 *     KEXDH_REPLY = c_ssh2.SSH2_MSG_KEXDH_REPLY
 * class SSH2_Msg_Kex(enum.Enum):
 *     ECDH_INIT = c_ssh2.SSH2_MSG_KEX_ECDH_INIT             # <<<<<<<<<<<<<<
 *     ECDH_REPLY = c_ssh2.SSH2_MSG_KEX_ECDH_REPLY
 *     DH_GEX_REQUEST_OLD = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST_OLD
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_ECDH_INIT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDH_INIT, __pyx_t_4) < 0) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":249
 * class SSH2_Msg_Kex(enum.Enum):
 *     ECDH_INIT = c_ssh2.SSH2_MSG_KEX_ECDH_INIT
 *     ECDH_REPLY = c_ssh2.SSH2_MSG_KEX_ECDH_REPLY             # <<<<<<<<<<<<<<
 *     DH_GEX_REQUEST_OLD = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST_OLD
 *     DH_GEX_GROUP = c_ssh2.SSH2_MSG_KEX_DH_GEX_GROUP
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_ECDH_REPLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ECDH_REPLY, __pyx_t_4) < 0) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":250
 *     ECDH_INIT = c_ssh2.SSH2_MSG_KEX_ECDH_INIT
 *     ECDH_REPLY = c_ssh2.SSH2_MSG_KEX_ECDH_REPLY
 *     DH_GEX_REQUEST_OLD = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST_OLD             # <<<<<<<<<<<<<<
 *     DH_GEX_GROUP = c_ssh2.SSH2_MSG_KEX_DH_GEX_GROUP
 *     DH_GEX_INIT = c_ssh2.SSH2_MSG_KEX_DH_GEX_INIT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_DH_GEX_REQUEST_OLD); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DH_GEX_REQUEST_OLD, __pyx_t_4) < 0) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":251
 *     ECDH_REPLY = c_ssh2.SSH2_MSG_KEX_ECDH_REPLY
 *     DH_GEX_REQUEST_OLD = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST_OLD
 *     DH_GEX_GROUP = c_ssh2.SSH2_MSG_KEX_DH_GEX_GROUP             # <<<<<<<<<<<<<<
 *     DH_GEX_INIT = c_ssh2.SSH2_MSG_KEX_DH_GEX_INIT
 *     DH_GEX_REPLY = c_ssh2.SSH2_MSG_KEX_DH_GEX_REPLY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_DH_GEX_GROUP); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DH_GEX_GROUP, __pyx_t_4) < 0) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":252
 *     DH_GEX_REQUEST_OLD = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST_OLD
 *     DH_GEX_GROUP = c_ssh2.SSH2_MSG_KEX_DH_GEX_GROUP
 *     DH_GEX_INIT = c_ssh2.SSH2_MSG_KEX_DH_GEX_INIT             # <<<<<<<<<<<<<<
 *     DH_GEX_REPLY = c_ssh2.SSH2_MSG_KEX_DH_GEX_REPLY
 *     DH_GEX_REQUEST = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_DH_GEX_INIT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DH_GEX_INIT, __pyx_t_4) < 0) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":253
 *     DH_GEX_GROUP = c_ssh2.SSH2_MSG_KEX_DH_GEX_GROUP
 *     DH_GEX_INIT = c_ssh2.SSH2_MSG_KEX_DH_GEX_INIT
 *     DH_GEX_REPLY = c_ssh2.SSH2_MSG_KEX_DH_GEX_REPLY             # <<<<<<<<<<<<<<
 *     DH_GEX_REQUEST = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST
 * class SSH2_Msg_UserAuth(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_DH_GEX_REPLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DH_GEX_REPLY, __pyx_t_4) < 0) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":254
 *     DH_GEX_INIT = c_ssh2.SSH2_MSG_KEX_DH_GEX_INIT
 *     DH_GEX_REPLY = c_ssh2.SSH2_MSG_KEX_DH_GEX_REPLY
 *     DH_GEX_REQUEST = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST             # <<<<<<<<<<<<<<
 * class SSH2_Msg_UserAuth(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_USERAUTH_REQUEST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_KEX_DH_GEX_REQUEST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DH_GEX_REQUEST, __pyx_t_4) < 0) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":247
 *     KEXDH_INIT = c_ssh2.SSH2_MSG_KEXDH_INIT
 *     KEXDH_REPLY = c_ssh2.SSH2_MSG_KEXDH_REPLY
 * class SSH2_Msg_Kex(enum.Enum):             # <<<<<<<<<<<<<<
 *     ECDH_INIT = c_ssh2.SSH2_MSG_KEX_ECDH_INIT
 *     ECDH_REPLY = c_ssh2.SSH2_MSG_KEX_ECDH_REPLY
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Msg_Kex, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Msg_Kex, __pyx_t_4) < 0) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":255
 *     DH_GEX_REPLY = c_ssh2.SSH2_MSG_KEX_DH_GEX_REPLY
 *     DH_GEX_REQUEST = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST
 * class SSH2_Msg_UserAuth(enum.Enum):             # <<<<<<<<<<<<<<
 *     REQUEST = c_ssh2.SSH2_MSG_USERAUTH_REQUEST
 *     FAILURE = c_ssh2.SSH2_MSG_USERAUTH_FAILURE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Msg_UserAuth, __pyx_n_s_SSH2_Msg_UserAuth, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":256
 *     DH_GEX_REQUEST = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST
 * class SSH2_Msg_UserAuth(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_USERAUTH_REQUEST             # <<<<<<<<<<<<<<
 *     FAILURE = c_ssh2.SSH2_MSG_USERAUTH_FAILURE
 *     SUCCESS = c_ssh2.SSH2_MSG_USERAUTH_SUCCESS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_REQUEST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REQUEST, __pyx_t_4) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":257
 * class SSH2_Msg_UserAuth(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_USERAUTH_REQUEST
 *     FAILURE = c_ssh2.SSH2_MSG_USERAUTH_FAILURE             # <<<<<<<<<<<<<<
 *     SUCCESS = c_ssh2.SSH2_MSG_USERAUTH_SUCCESS
 *     BANNER = c_ssh2.SSH2_MSG_USERAUTH_BANNER
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_FAILURE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FAILURE, __pyx_t_4) < 0) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":258
 *     REQUEST = c_ssh2.SSH2_MSG_USERAUTH_REQUEST
 *     FAILURE = c_ssh2.SSH2_MSG_USERAUTH_FAILURE
 *     SUCCESS = c_ssh2.SSH2_MSG_USERAUTH_SUCCESS             # <<<<<<<<<<<<<<
 *     BANNER = c_ssh2.SSH2_MSG_USERAUTH_BANNER
 *     PK_OK = c_ssh2.SSH2_MSG_USERAUTH_PK_OK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_SUCCESS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SUCCESS, __pyx_t_4) < 0) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":259
 *     FAILURE = c_ssh2.SSH2_MSG_USERAUTH_FAILURE
 *     SUCCESS = c_ssh2.SSH2_MSG_USERAUTH_SUCCESS
 *     BANNER = c_ssh2.SSH2_MSG_USERAUTH_BANNER             # <<<<<<<<<<<<<<
 *     PK_OK = c_ssh2.SSH2_MSG_USERAUTH_PK_OK
 *     PASSWD_CHANGEREQ = c_ssh2.SSH2_MSG_USERAUTH_PASSWD_CHANGEREQ
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_BANNER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_BANNER, __pyx_t_4) < 0) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":260
 *     SUCCESS = c_ssh2.SSH2_MSG_USERAUTH_SUCCESS
 *     BANNER = c_ssh2.SSH2_MSG_USERAUTH_BANNER
 *     PK_OK = c_ssh2.SSH2_MSG_USERAUTH_PK_OK             # <<<<<<<<<<<<<<
 *     PASSWD_CHANGEREQ = c_ssh2.SSH2_MSG_USERAUTH_PASSWD_CHANGEREQ
 *     INFO_REQUEST = c_ssh2.SSH2_MSG_USERAUTH_INFO_REQUEST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_PK_OK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PK_OK, __pyx_t_4) < 0) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":261
 *     BANNER = c_ssh2.SSH2_MSG_USERAUTH_BANNER
 *     PK_OK = c_ssh2.SSH2_MSG_USERAUTH_PK_OK
 *     PASSWD_CHANGEREQ = c_ssh2.SSH2_MSG_USERAUTH_PASSWD_CHANGEREQ             # <<<<<<<<<<<<<<
 *     INFO_REQUEST = c_ssh2.SSH2_MSG_USERAUTH_INFO_REQUEST
 *     GSSAPI_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_RESPONSE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_PASSWD_CHANGEREQ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PASSWD_CHANGEREQ, __pyx_t_4) < 0) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":262
 *     PK_OK = c_ssh2.SSH2_MSG_USERAUTH_PK_OK
 *     PASSWD_CHANGEREQ = c_ssh2.SSH2_MSG_USERAUTH_PASSWD_CHANGEREQ
 *     INFO_REQUEST = c_ssh2.SSH2_MSG_USERAUTH_INFO_REQUEST             # <<<<<<<<<<<<<<
 *     GSSAPI_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_RESPONSE
 *     INFO_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_INFO_RESPONSE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_INFO_REQUEST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INFO_REQUEST, __pyx_t_4) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":263
 *     PASSWD_CHANGEREQ = c_ssh2.SSH2_MSG_USERAUTH_PASSWD_CHANGEREQ
 *     INFO_REQUEST = c_ssh2.SSH2_MSG_USERAUTH_INFO_REQUEST
 *     GSSAPI_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_RESPONSE             # <<<<<<<<<<<<<<
 *     INFO_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_INFO_RESPONSE
 *     GSSAPI_TOKEN = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_TOKEN
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_GSSAPI_RESPONSE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_RESPONSE, __pyx_t_4) < 0) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":264
 *     INFO_REQUEST = c_ssh2.SSH2_MSG_USERAUTH_INFO_REQUEST
 *     GSSAPI_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_RESPONSE
 *     INFO_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_INFO_RESPONSE             # <<<<<<<<<<<<<<
 *     GSSAPI_TOKEN = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_TOKEN
 *     GSSAPI_EXCHANGE_COMPLETE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_INFO_RESPONSE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INFO_RESPONSE, __pyx_t_4) < 0) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":265
 *     GSSAPI_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_RESPONSE
 *     INFO_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_INFO_RESPONSE
 *     GSSAPI_TOKEN = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_TOKEN             # <<<<<<<<<<<<<<
 *     GSSAPI_EXCHANGE_COMPLETE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE
 *     GSSAPI_ERROR = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_GSSAPI_TOKEN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_TOKEN, __pyx_t_4) < 0) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":266
 *     INFO_RESPONSE = c_ssh2.SSH2_MSG_USERAUTH_INFO_RESPONSE
 *     GSSAPI_TOKEN = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_TOKEN
 *     GSSAPI_EXCHANGE_COMPLETE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE             # <<<<<<<<<<<<<<
 *     GSSAPI_ERROR = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERROR
 *     GSSAPI_ERRTOK = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERRTOK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_EXCHANGE_COMPLETE, __pyx_t_4) < 0) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":267
 *     GSSAPI_TOKEN = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_TOKEN
 *     GSSAPI_EXCHANGE_COMPLETE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE
 *     GSSAPI_ERROR = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERROR             # <<<<<<<<<<<<<<
 *     GSSAPI_ERRTOK = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERRTOK
 *     GSSAPI_MIC = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_MIC
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_GSSAPI_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":268
 *     GSSAPI_EXCHANGE_COMPLETE = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_EXCHANGE_COMPLETE
 *     GSSAPI_ERROR = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERROR
 *     GSSAPI_ERRTOK = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERRTOK             # <<<<<<<<<<<<<<
 *     GSSAPI_MIC = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_MIC
 * class SSH2_Msg_Global(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_GSSAPI_ERRTOK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_ERRTOK, __pyx_t_4) < 0) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":269
 *     GSSAPI_ERROR = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERROR
 *     GSSAPI_ERRTOK = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERRTOK
 *     GSSAPI_MIC = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_MIC             # <<<<<<<<<<<<<<
 * class SSH2_Msg_Global(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_USERAUTH_GSSAPI_MIC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GSSAPI_MIC, __pyx_t_4) < 0) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":255
 *     DH_GEX_REPLY = c_ssh2.SSH2_MSG_KEX_DH_GEX_REPLY
 *     DH_GEX_REQUEST = c_ssh2.SSH2_MSG_KEX_DH_GEX_REQUEST
 * class SSH2_Msg_UserAuth(enum.Enum):             # <<<<<<<<<<<<<<
 *     REQUEST = c_ssh2.SSH2_MSG_USERAUTH_REQUEST
 *     FAILURE = c_ssh2.SSH2_MSG_USERAUTH_FAILURE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Msg_UserAuth, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Msg_UserAuth, __pyx_t_4) < 0) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":270
 *     GSSAPI_ERRTOK = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERRTOK
 *     GSSAPI_MIC = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_MIC
 * class SSH2_Msg_Global(enum.Enum):             # <<<<<<<<<<<<<<
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST
 * class SSH2_Msg_Request(enum.Enum):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Msg_Global, __pyx_n_s_SSH2_Msg_Global, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":271
 *     GSSAPI_MIC = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_MIC
 * class SSH2_Msg_Global(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST             # <<<<<<<<<<<<<<
 * class SSH2_Msg_Request(enum.Enum):
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_GLOBAL_REQUEST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REQUEST, __pyx_t_4) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":270
 *     GSSAPI_ERRTOK = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_ERRTOK
 *     GSSAPI_MIC = c_ssh2.SSH2_MSG_USERAUTH_GSSAPI_MIC
 * class SSH2_Msg_Global(enum.Enum):             # <<<<<<<<<<<<<<
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST
 * class SSH2_Msg_Request(enum.Enum):
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Msg_Global, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Msg_Global, __pyx_t_4) < 0) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":272
 * class SSH2_Msg_Global(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST
 * class SSH2_Msg_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Msg_Request, __pyx_n_s_SSH2_Msg_Request, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":273
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST
 * class SSH2_Msg_Request(enum.Enum):
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS             # <<<<<<<<<<<<<<
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE
 * class SSH2_Msg_Channel(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_REQUEST_SUCCESS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SUCCESS, __pyx_t_4) < 0) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":274
 * class SSH2_Msg_Request(enum.Enum):
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE             # <<<<<<<<<<<<<<
 * class SSH2_Msg_Channel(enum.Enum):
 *     OPEN = c_ssh2.SSH2_MSG_CHANNEL_OPEN
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_REQUEST_FAILURE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FAILURE, __pyx_t_4) < 0) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":272
 * class SSH2_Msg_Global(enum.Enum):
 *     REQUEST = c_ssh2.SSH2_MSG_GLOBAL_REQUEST
 * class SSH2_Msg_Request(enum.Enum):             # <<<<<<<<<<<<<<
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Msg_Request, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Msg_Request, __pyx_t_4) < 0) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":275
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE
 * class SSH2_Msg_Channel(enum.Enum):             # <<<<<<<<<<<<<<
 *     OPEN = c_ssh2.SSH2_MSG_CHANNEL_OPEN
 *     OPEN_CONFIRMATION = c_ssh2.SSH2_MSG_CHANNEL_OPEN_CONFIRMATION
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Msg_Channel, __pyx_n_s_SSH2_Msg_Channel, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":276
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE
 * class SSH2_Msg_Channel(enum.Enum):
 *     OPEN = c_ssh2.SSH2_MSG_CHANNEL_OPEN             # <<<<<<<<<<<<<<
 *     OPEN_CONFIRMATION = c_ssh2.SSH2_MSG_CHANNEL_OPEN_CONFIRMATION
 *     OPEN_FAILURE = c_ssh2.SSH2_MSG_CHANNEL_OPEN_FAILURE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_OPEN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPEN, __pyx_t_4) < 0) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":277
 * class SSH2_Msg_Channel(enum.Enum):
 *     OPEN = c_ssh2.SSH2_MSG_CHANNEL_OPEN
 *     OPEN_CONFIRMATION = c_ssh2.SSH2_MSG_CHANNEL_OPEN_CONFIRMATION             # <<<<<<<<<<<<<<
 *     OPEN_FAILURE = c_ssh2.SSH2_MSG_CHANNEL_OPEN_FAILURE
 *     WINDOW_ADJUST = c_ssh2.SSH2_MSG_CHANNEL_WINDOW_ADJUST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_OPEN_CONFIRMATION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPEN_CONFIRMATION, __pyx_t_4) < 0) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":278
 *     OPEN = c_ssh2.SSH2_MSG_CHANNEL_OPEN
 *     OPEN_CONFIRMATION = c_ssh2.SSH2_MSG_CHANNEL_OPEN_CONFIRMATION
 *     OPEN_FAILURE = c_ssh2.SSH2_MSG_CHANNEL_OPEN_FAILURE             # <<<<<<<<<<<<<<
 *     WINDOW_ADJUST = c_ssh2.SSH2_MSG_CHANNEL_WINDOW_ADJUST
 *     DATA = c_ssh2.SSH2_MSG_CHANNEL_DATA
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_OPEN_FAILURE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPEN_FAILURE, __pyx_t_4) < 0) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":279
 *     OPEN_CONFIRMATION = c_ssh2.SSH2_MSG_CHANNEL_OPEN_CONFIRMATION
 *     OPEN_FAILURE = c_ssh2.SSH2_MSG_CHANNEL_OPEN_FAILURE
 *     WINDOW_ADJUST = c_ssh2.SSH2_MSG_CHANNEL_WINDOW_ADJUST             # <<<<<<<<<<<<<<
 *     DATA = c_ssh2.SSH2_MSG_CHANNEL_DATA
 *     EXTENDED_DATA = c_ssh2.SSH2_MSG_CHANNEL_EXTENDED_DATA
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_WINDOW_ADJUST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WINDOW_ADJUST, __pyx_t_4) < 0) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":280
 *     OPEN_FAILURE = c_ssh2.SSH2_MSG_CHANNEL_OPEN_FAILURE
 *     WINDOW_ADJUST = c_ssh2.SSH2_MSG_CHANNEL_WINDOW_ADJUST
 *     DATA = c_ssh2.SSH2_MSG_CHANNEL_DATA             # <<<<<<<<<<<<<<
 *     EXTENDED_DATA = c_ssh2.SSH2_MSG_CHANNEL_EXTENDED_DATA
 *     EOF = c_ssh2.SSH2_MSG_CHANNEL_EOF
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_DATA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DATA, __pyx_t_4) < 0) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":281
 *     WINDOW_ADJUST = c_ssh2.SSH2_MSG_CHANNEL_WINDOW_ADJUST
 *     DATA = c_ssh2.SSH2_MSG_CHANNEL_DATA
 *     EXTENDED_DATA = c_ssh2.SSH2_MSG_CHANNEL_EXTENDED_DATA             # <<<<<<<<<<<<<<
 *     EOF = c_ssh2.SSH2_MSG_CHANNEL_EOF
 *     CLOSE = c_ssh2.SSH2_MSG_CHANNEL_CLOSE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_EXTENDED_DATA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXTENDED_DATA, __pyx_t_4) < 0) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":282
 *     DATA = c_ssh2.SSH2_MSG_CHANNEL_DATA
 *     EXTENDED_DATA = c_ssh2.SSH2_MSG_CHANNEL_EXTENDED_DATA
 *     EOF = c_ssh2.SSH2_MSG_CHANNEL_EOF             # <<<<<<<<<<<<<<
 *     CLOSE = c_ssh2.SSH2_MSG_CHANNEL_CLOSE
 *     REQUEST = c_ssh2.SSH2_MSG_CHANNEL_REQUEST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_EOF); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EOF, __pyx_t_4) < 0) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":283
 *     EXTENDED_DATA = c_ssh2.SSH2_MSG_CHANNEL_EXTENDED_DATA
 *     EOF = c_ssh2.SSH2_MSG_CHANNEL_EOF
 *     CLOSE = c_ssh2.SSH2_MSG_CHANNEL_CLOSE             # <<<<<<<<<<<<<<
 *     REQUEST = c_ssh2.SSH2_MSG_CHANNEL_REQUEST
 *     SUCCESS = c_ssh2.SSH2_MSG_CHANNEL_SUCCESS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_CLOSE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLOSE, __pyx_t_4) < 0) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":284
 *     EOF = c_ssh2.SSH2_MSG_CHANNEL_EOF
 *     CLOSE = c_ssh2.SSH2_MSG_CHANNEL_CLOSE
 *     REQUEST = c_ssh2.SSH2_MSG_CHANNEL_REQUEST             # <<<<<<<<<<<<<<
 *     SUCCESS = c_ssh2.SSH2_MSG_CHANNEL_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_CHANNEL_FAILURE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_REQUEST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REQUEST, __pyx_t_4) < 0) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":285
 *     CLOSE = c_ssh2.SSH2_MSG_CHANNEL_CLOSE
 *     REQUEST = c_ssh2.SSH2_MSG_CHANNEL_REQUEST
 *     SUCCESS = c_ssh2.SSH2_MSG_CHANNEL_SUCCESS             # <<<<<<<<<<<<<<
 *     FAILURE = c_ssh2.SSH2_MSG_CHANNEL_FAILURE
 * class SSH2_Disconnect(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_SUCCESS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SUCCESS, __pyx_t_4) < 0) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":286
 *     REQUEST = c_ssh2.SSH2_MSG_CHANNEL_REQUEST
 *     SUCCESS = c_ssh2.SSH2_MSG_CHANNEL_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_CHANNEL_FAILURE             # <<<<<<<<<<<<<<
 * class SSH2_Disconnect(enum.Enum):
 *     HOST_NOT_ALLOWED_TO_CONNECT = c_ssh2.SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_MSG_CHANNEL_FAILURE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FAILURE, __pyx_t_4) < 0) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":275
 *     SUCCESS = c_ssh2.SSH2_MSG_REQUEST_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_REQUEST_FAILURE
 * class SSH2_Msg_Channel(enum.Enum):             # <<<<<<<<<<<<<<
 *     OPEN = c_ssh2.SSH2_MSG_CHANNEL_OPEN
 *     OPEN_CONFIRMATION = c_ssh2.SSH2_MSG_CHANNEL_OPEN_CONFIRMATION
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Msg_Channel, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Msg_Channel, __pyx_t_4) < 0) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":287
 *     SUCCESS = c_ssh2.SSH2_MSG_CHANNEL_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_CHANNEL_FAILURE
 * class SSH2_Disconnect(enum.Enum):             # <<<<<<<<<<<<<<
 *     HOST_NOT_ALLOWED_TO_CONNECT = c_ssh2.SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT
 *     PROTOCOL_ERROR = c_ssh2.SSH2_DISCONNECT_PROTOCOL_ERROR
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Disconnect, __pyx_n_s_SSH2_Disconnect, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":288
 *     FAILURE = c_ssh2.SSH2_MSG_CHANNEL_FAILURE
 * class SSH2_Disconnect(enum.Enum):
 *     HOST_NOT_ALLOWED_TO_CONNECT = c_ssh2.SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT             # <<<<<<<<<<<<<<
 *     PROTOCOL_ERROR = c_ssh2.SSH2_DISCONNECT_PROTOCOL_ERROR
 *     KEY_EXCHANGE_FAILED = c_ssh2.SSH2_DISCONNECT_KEY_EXCHANGE_FAILED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOST_NOT_ALLOWED_TO_CONNECT, __pyx_t_4) < 0) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":289
 * class SSH2_Disconnect(enum.Enum):
 *     HOST_NOT_ALLOWED_TO_CONNECT = c_ssh2.SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT
 *     PROTOCOL_ERROR = c_ssh2.SSH2_DISCONNECT_PROTOCOL_ERROR             # <<<<<<<<<<<<<<
 *     KEY_EXCHANGE_FAILED = c_ssh2.SSH2_DISCONNECT_KEY_EXCHANGE_FAILED
 *     HOST_AUTHENTICATION_FAILED = c_ssh2.SSH2_DISCONNECT_HOST_AUTHENTICATION_FAILED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_PROTOCOL_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PROTOCOL_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":290
 *     HOST_NOT_ALLOWED_TO_CONNECT = c_ssh2.SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT
 *     PROTOCOL_ERROR = c_ssh2.SSH2_DISCONNECT_PROTOCOL_ERROR
 *     KEY_EXCHANGE_FAILED = c_ssh2.SSH2_DISCONNECT_KEY_EXCHANGE_FAILED             # <<<<<<<<<<<<<<
 *     HOST_AUTHENTICATION_FAILED = c_ssh2.SSH2_DISCONNECT_HOST_AUTHENTICATION_FAILED
 *     RESERVED = c_ssh2.SSH2_DISCONNECT_RESERVED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_KEY_EXCHANGE_FAILED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_KEY_EXCHANGE_FAILED, __pyx_t_4) < 0) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":291
 *     PROTOCOL_ERROR = c_ssh2.SSH2_DISCONNECT_PROTOCOL_ERROR
 *     KEY_EXCHANGE_FAILED = c_ssh2.SSH2_DISCONNECT_KEY_EXCHANGE_FAILED
 *     HOST_AUTHENTICATION_FAILED = c_ssh2.SSH2_DISCONNECT_HOST_AUTHENTICATION_FAILED             # <<<<<<<<<<<<<<
 *     RESERVED = c_ssh2.SSH2_DISCONNECT_RESERVED
 *     MAC_ERROR = c_ssh2.SSH2_DISCONNECT_MAC_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_HOST_AUTHENTICATION_FAILED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOST_AUTHENTICATION_FAILED, __pyx_t_4) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":292
 *     KEY_EXCHANGE_FAILED = c_ssh2.SSH2_DISCONNECT_KEY_EXCHANGE_FAILED
 *     HOST_AUTHENTICATION_FAILED = c_ssh2.SSH2_DISCONNECT_HOST_AUTHENTICATION_FAILED
 *     RESERVED = c_ssh2.SSH2_DISCONNECT_RESERVED             # <<<<<<<<<<<<<<
 *     MAC_ERROR = c_ssh2.SSH2_DISCONNECT_MAC_ERROR
 *     COMPRESSION_ERROR = c_ssh2.SSH2_DISCONNECT_COMPRESSION_ERROR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_RESERVED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RESERVED, __pyx_t_4) < 0) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":293
 *     HOST_AUTHENTICATION_FAILED = c_ssh2.SSH2_DISCONNECT_HOST_AUTHENTICATION_FAILED
 *     RESERVED = c_ssh2.SSH2_DISCONNECT_RESERVED
 *     MAC_ERROR = c_ssh2.SSH2_DISCONNECT_MAC_ERROR             # <<<<<<<<<<<<<<
 *     COMPRESSION_ERROR = c_ssh2.SSH2_DISCONNECT_COMPRESSION_ERROR
 *     SERVICE_NOT_AVAILABLE = c_ssh2.SSH2_DISCONNECT_SERVICE_NOT_AVAILABLE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_MAC_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MAC_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 293, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":294
 *     RESERVED = c_ssh2.SSH2_DISCONNECT_RESERVED
 *     MAC_ERROR = c_ssh2.SSH2_DISCONNECT_MAC_ERROR
 *     COMPRESSION_ERROR = c_ssh2.SSH2_DISCONNECT_COMPRESSION_ERROR             # <<<<<<<<<<<<<<
 *     SERVICE_NOT_AVAILABLE = c_ssh2.SSH2_DISCONNECT_SERVICE_NOT_AVAILABLE
 *     PROTOCOL_VERSION_NOT_SUPPORTED = c_ssh2.SSH2_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_COMPRESSION_ERROR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_COMPRESSION_ERROR, __pyx_t_4) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":295
 *     MAC_ERROR = c_ssh2.SSH2_DISCONNECT_MAC_ERROR
 *     COMPRESSION_ERROR = c_ssh2.SSH2_DISCONNECT_COMPRESSION_ERROR
 *     SERVICE_NOT_AVAILABLE = c_ssh2.SSH2_DISCONNECT_SERVICE_NOT_AVAILABLE             # <<<<<<<<<<<<<<
 *     PROTOCOL_VERSION_NOT_SUPPORTED = c_ssh2.SSH2_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED
 *     HOST_KEY_NOT_VERIFIABLE = c_ssh2.SSH2_DISCONNECT_HOST_KEY_NOT_VERIFIABLE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_SERVICE_NOT_AVAILABLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SERVICE_NOT_AVAILABLE, __pyx_t_4) < 0) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":296
 *     COMPRESSION_ERROR = c_ssh2.SSH2_DISCONNECT_COMPRESSION_ERROR
 *     SERVICE_NOT_AVAILABLE = c_ssh2.SSH2_DISCONNECT_SERVICE_NOT_AVAILABLE
 *     PROTOCOL_VERSION_NOT_SUPPORTED = c_ssh2.SSH2_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED             # <<<<<<<<<<<<<<
 *     HOST_KEY_NOT_VERIFIABLE = c_ssh2.SSH2_DISCONNECT_HOST_KEY_NOT_VERIFIABLE
 *     CONNECTION_LOST = c_ssh2.SSH2_DISCONNECT_CONNECTION_LOST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PROTOCOL_VERSION_NOT_SUPPORTED, __pyx_t_4) < 0) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":297
 *     SERVICE_NOT_AVAILABLE = c_ssh2.SSH2_DISCONNECT_SERVICE_NOT_AVAILABLE
 *     PROTOCOL_VERSION_NOT_SUPPORTED = c_ssh2.SSH2_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED
 *     HOST_KEY_NOT_VERIFIABLE = c_ssh2.SSH2_DISCONNECT_HOST_KEY_NOT_VERIFIABLE             # <<<<<<<<<<<<<<
 *     CONNECTION_LOST = c_ssh2.SSH2_DISCONNECT_CONNECTION_LOST
 *     BY_APPLICATION = c_ssh2.SSH2_DISCONNECT_BY_APPLICATION
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_HOST_KEY_NOT_VERIFIABLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HOST_KEY_NOT_VERIFIABLE, __pyx_t_4) < 0) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":298
 *     PROTOCOL_VERSION_NOT_SUPPORTED = c_ssh2.SSH2_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED
 *     HOST_KEY_NOT_VERIFIABLE = c_ssh2.SSH2_DISCONNECT_HOST_KEY_NOT_VERIFIABLE
 *     CONNECTION_LOST = c_ssh2.SSH2_DISCONNECT_CONNECTION_LOST             # <<<<<<<<<<<<<<
 *     BY_APPLICATION = c_ssh2.SSH2_DISCONNECT_BY_APPLICATION
 *     TOO_MANY_CONNECTIONS = c_ssh2.SSH2_DISCONNECT_TOO_MANY_CONNECTIONS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_CONNECTION_LOST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CONNECTION_LOST, __pyx_t_4) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":299
 *     HOST_KEY_NOT_VERIFIABLE = c_ssh2.SSH2_DISCONNECT_HOST_KEY_NOT_VERIFIABLE
 *     CONNECTION_LOST = c_ssh2.SSH2_DISCONNECT_CONNECTION_LOST
 *     BY_APPLICATION = c_ssh2.SSH2_DISCONNECT_BY_APPLICATION             # <<<<<<<<<<<<<<
 *     TOO_MANY_CONNECTIONS = c_ssh2.SSH2_DISCONNECT_TOO_MANY_CONNECTIONS
 *     AUTH_CANCELLED_BY_USER = c_ssh2.SSH2_DISCONNECT_AUTH_CANCELLED_BY_USER
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_BY_APPLICATION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_BY_APPLICATION, __pyx_t_4) < 0) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":300
 *     CONNECTION_LOST = c_ssh2.SSH2_DISCONNECT_CONNECTION_LOST
 *     BY_APPLICATION = c_ssh2.SSH2_DISCONNECT_BY_APPLICATION
 *     TOO_MANY_CONNECTIONS = c_ssh2.SSH2_DISCONNECT_TOO_MANY_CONNECTIONS             # <<<<<<<<<<<<<<
 *     AUTH_CANCELLED_BY_USER = c_ssh2.SSH2_DISCONNECT_AUTH_CANCELLED_BY_USER
 *     NO_MORE_AUTH_METHODS_AVAILABLE = c_ssh2.SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_TOO_MANY_CONNECTIONS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TOO_MANY_CONNECTIONS, __pyx_t_4) < 0) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":301
 *     BY_APPLICATION = c_ssh2.SSH2_DISCONNECT_BY_APPLICATION
 *     TOO_MANY_CONNECTIONS = c_ssh2.SSH2_DISCONNECT_TOO_MANY_CONNECTIONS
 *     AUTH_CANCELLED_BY_USER = c_ssh2.SSH2_DISCONNECT_AUTH_CANCELLED_BY_USER             # <<<<<<<<<<<<<<
 *     NO_MORE_AUTH_METHODS_AVAILABLE = c_ssh2.SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE
 *     ILLEGAL_USER_NAME = c_ssh2.SSH2_DISCONNECT_ILLEGAL_USER_NAME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_AUTH_CANCELLED_BY_USER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_AUTH_CANCELLED_BY_USER, __pyx_t_4) < 0) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":302
 *     TOO_MANY_CONNECTIONS = c_ssh2.SSH2_DISCONNECT_TOO_MANY_CONNECTIONS
 *     AUTH_CANCELLED_BY_USER = c_ssh2.SSH2_DISCONNECT_AUTH_CANCELLED_BY_USER
 *     NO_MORE_AUTH_METHODS_AVAILABLE = c_ssh2.SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE             # <<<<<<<<<<<<<<
 *     ILLEGAL_USER_NAME = c_ssh2.SSH2_DISCONNECT_ILLEGAL_USER_NAME
 * class SSH2_Open(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NO_MORE_AUTH_METHODS_AVAILABLE, __pyx_t_4) < 0) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":303
 *     AUTH_CANCELLED_BY_USER = c_ssh2.SSH2_DISCONNECT_AUTH_CANCELLED_BY_USER
 *     NO_MORE_AUTH_METHODS_AVAILABLE = c_ssh2.SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE
 *     ILLEGAL_USER_NAME = c_ssh2.SSH2_DISCONNECT_ILLEGAL_USER_NAME             # <<<<<<<<<<<<<<
 * class SSH2_Open(enum.Enum):
 *     ADMINISTRATIVELY_PROHIBITED = c_ssh2.SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_DISCONNECT_ILLEGAL_USER_NAME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ILLEGAL_USER_NAME, __pyx_t_4) < 0) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":287
 *     SUCCESS = c_ssh2.SSH2_MSG_CHANNEL_SUCCESS
 *     FAILURE = c_ssh2.SSH2_MSG_CHANNEL_FAILURE
 * class SSH2_Disconnect(enum.Enum):             # <<<<<<<<<<<<<<
 *     HOST_NOT_ALLOWED_TO_CONNECT = c_ssh2.SSH2_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT
 *     PROTOCOL_ERROR = c_ssh2.SSH2_DISCONNECT_PROTOCOL_ERROR
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Disconnect, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Disconnect, __pyx_t_4) < 0) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":304
 *     NO_MORE_AUTH_METHODS_AVAILABLE = c_ssh2.SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE
 *     ILLEGAL_USER_NAME = c_ssh2.SSH2_DISCONNECT_ILLEGAL_USER_NAME
 * class SSH2_Open(enum.Enum):             # <<<<<<<<<<<<<<
 *     ADMINISTRATIVELY_PROHIBITED = c_ssh2.SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED
 *     CONNECT_FAILED = c_ssh2.SSH2_OPEN_CONNECT_FAILED
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_Open, __pyx_n_s_SSH2_Open, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":305
 *     ILLEGAL_USER_NAME = c_ssh2.SSH2_DISCONNECT_ILLEGAL_USER_NAME
 * class SSH2_Open(enum.Enum):
 *     ADMINISTRATIVELY_PROHIBITED = c_ssh2.SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED             # <<<<<<<<<<<<<<
 *     CONNECT_FAILED = c_ssh2.SSH2_OPEN_CONNECT_FAILED
 *     UNKNOWN_CHANNEL_TYPE = c_ssh2.SSH2_OPEN_UNKNOWN_CHANNEL_TYPE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ADMINISTRATIVELY_PROHIBITED, __pyx_t_4) < 0) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":306
 * class SSH2_Open(enum.Enum):
 *     ADMINISTRATIVELY_PROHIBITED = c_ssh2.SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED
 *     CONNECT_FAILED = c_ssh2.SSH2_OPEN_CONNECT_FAILED             # <<<<<<<<<<<<<<
 *     UNKNOWN_CHANNEL_TYPE = c_ssh2.SSH2_OPEN_UNKNOWN_CHANNEL_TYPE
 *     RESOURCE_SHORTAGE = c_ssh2.SSH2_OPEN_RESOURCE_SHORTAGE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_OPEN_CONNECT_FAILED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CONNECT_FAILED, __pyx_t_4) < 0) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":307
 *     ADMINISTRATIVELY_PROHIBITED = c_ssh2.SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED
 *     CONNECT_FAILED = c_ssh2.SSH2_OPEN_CONNECT_FAILED
 *     UNKNOWN_CHANNEL_TYPE = c_ssh2.SSH2_OPEN_UNKNOWN_CHANNEL_TYPE             # <<<<<<<<<<<<<<
 *     RESOURCE_SHORTAGE = c_ssh2.SSH2_OPEN_RESOURCE_SHORTAGE
 * class SSH2_ExtendedData(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_OPEN_UNKNOWN_CHANNEL_TYPE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN_CHANNEL_TYPE, __pyx_t_4) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":308
 *     CONNECT_FAILED = c_ssh2.SSH2_OPEN_CONNECT_FAILED
 *     UNKNOWN_CHANNEL_TYPE = c_ssh2.SSH2_OPEN_UNKNOWN_CHANNEL_TYPE
 *     RESOURCE_SHORTAGE = c_ssh2.SSH2_OPEN_RESOURCE_SHORTAGE             # <<<<<<<<<<<<<<
 * class SSH2_ExtendedData(enum.Enum):
 *     STDERR = c_ssh2.SSH2_EXTENDED_DATA_STDERR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_OPEN_RESOURCE_SHORTAGE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RESOURCE_SHORTAGE, __pyx_t_4) < 0) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":304
 *     NO_MORE_AUTH_METHODS_AVAILABLE = c_ssh2.SSH2_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE
 *     ILLEGAL_USER_NAME = c_ssh2.SSH2_DISCONNECT_ILLEGAL_USER_NAME
 * class SSH2_Open(enum.Enum):             # <<<<<<<<<<<<<<
 *     ADMINISTRATIVELY_PROHIBITED = c_ssh2.SSH2_OPEN_ADMINISTRATIVELY_PROHIBITED
 *     CONNECT_FAILED = c_ssh2.SSH2_OPEN_CONNECT_FAILED
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_Open, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_Open, __pyx_t_4) < 0) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":309
 *     UNKNOWN_CHANNEL_TYPE = c_ssh2.SSH2_OPEN_UNKNOWN_CHANNEL_TYPE
 *     RESOURCE_SHORTAGE = c_ssh2.SSH2_OPEN_RESOURCE_SHORTAGE
 * class SSH2_ExtendedData(enum.Enum):             # <<<<<<<<<<<<<<
 *     STDERR = c_ssh2.SSH2_EXTENDED_DATA_STDERR
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SSH2_ExtendedData, __pyx_n_s_SSH2_ExtendedData, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":310
 *     RESOURCE_SHORTAGE = c_ssh2.SSH2_OPEN_RESOURCE_SHORTAGE
 * class SSH2_ExtendedData(enum.Enum):
 *     STDERR = c_ssh2.SSH2_EXTENDED_DATA_STDERR             # <<<<<<<<<<<<<<
 * 
 * #sftp
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH2_EXTENDED_DATA_STDERR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STDERR, __pyx_t_4) < 0) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":309
 *     UNKNOWN_CHANNEL_TYPE = c_ssh2.SSH2_OPEN_UNKNOWN_CHANNEL_TYPE
 *     RESOURCE_SHORTAGE = c_ssh2.SSH2_OPEN_RESOURCE_SHORTAGE
 * class SSH2_ExtendedData(enum.Enum):             # <<<<<<<<<<<<<<
 *     STDERR = c_ssh2.SSH2_EXTENDED_DATA_STDERR
 * 
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SSH2_ExtendedData, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SSH2_ExtendedData, __pyx_t_4) < 0) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":313
 * 
 * #sftp
 * class SFTP(enum.Enum):             # <<<<<<<<<<<<<<
 *     OPEN = c_sftp.SFTP_OPEN
 *     CLOSE = c_sftp.SFTP_CLOSE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP, __pyx_n_s_SFTP, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":314
 * #sftp
 * class SFTP(enum.Enum):
 *     OPEN = c_sftp.SFTP_OPEN             # <<<<<<<<<<<<<<
 *     CLOSE = c_sftp.SFTP_CLOSE
 *     READ = c_sftp.SFTP_READ
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_OPEN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPEN, __pyx_t_4) < 0) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":315
 * class SFTP(enum.Enum):
 *     OPEN = c_sftp.SFTP_OPEN
 *     CLOSE = c_sftp.SFTP_CLOSE             # <<<<<<<<<<<<<<
 *     READ = c_sftp.SFTP_READ
 *     WRITE = c_sftp.SFTP_WRITE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_CLOSE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLOSE, __pyx_t_4) < 0) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":316
 *     OPEN = c_sftp.SFTP_OPEN
 *     CLOSE = c_sftp.SFTP_CLOSE
 *     READ = c_sftp.SFTP_READ             # <<<<<<<<<<<<<<
 *     WRITE = c_sftp.SFTP_WRITE
 *     LSTAT = c_sftp.SFTP_LSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_READ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READ, __pyx_t_4) < 0) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":317
 *     CLOSE = c_sftp.SFTP_CLOSE
 *     READ = c_sftp.SFTP_READ
 *     WRITE = c_sftp.SFTP_WRITE             # <<<<<<<<<<<<<<
 *     LSTAT = c_sftp.SFTP_LSTAT
 *     FSTAT = c_sftp.SFTP_FSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_WRITE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRITE, __pyx_t_4) < 0) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":318
 *     READ = c_sftp.SFTP_READ
 *     WRITE = c_sftp.SFTP_WRITE
 *     LSTAT = c_sftp.SFTP_LSTAT             # <<<<<<<<<<<<<<
 *     FSTAT = c_sftp.SFTP_FSTAT
 *     SETSTAT = c_sftp.SFTP_SETSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_LSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":319
 *     WRITE = c_sftp.SFTP_WRITE
 *     LSTAT = c_sftp.SFTP_LSTAT
 *     FSTAT = c_sftp.SFTP_FSTAT             # <<<<<<<<<<<<<<
 *     SETSTAT = c_sftp.SFTP_SETSTAT
 *     FSETSTAT = c_sftp.SFTP_FSETSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_FSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":320
 *     LSTAT = c_sftp.SFTP_LSTAT
 *     FSTAT = c_sftp.SFTP_FSTAT
 *     SETSTAT = c_sftp.SFTP_SETSTAT             # <<<<<<<<<<<<<<
 *     FSETSTAT = c_sftp.SFTP_FSETSTAT
 *     OPENDIR = c_sftp.SFTP_OPENDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_SETSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SETSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 320, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":321
 *     FSTAT = c_sftp.SFTP_FSTAT
 *     SETSTAT = c_sftp.SFTP_SETSTAT
 *     FSETSTAT = c_sftp.SFTP_FSETSTAT             # <<<<<<<<<<<<<<
 *     OPENDIR = c_sftp.SFTP_OPENDIR
 *     READDIR = c_sftp.SFTP_READDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_FSETSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FSETSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":322
 *     SETSTAT = c_sftp.SFTP_SETSTAT
 *     FSETSTAT = c_sftp.SFTP_FSETSTAT
 *     OPENDIR = c_sftp.SFTP_OPENDIR             # <<<<<<<<<<<<<<
 *     READDIR = c_sftp.SFTP_READDIR
 *     REMOVE = c_sftp.SFTP_REMOVE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_OPENDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPENDIR, __pyx_t_4) < 0) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":323
 *     FSETSTAT = c_sftp.SFTP_FSETSTAT
 *     OPENDIR = c_sftp.SFTP_OPENDIR
 *     READDIR = c_sftp.SFTP_READDIR             # <<<<<<<<<<<<<<
 *     REMOVE = c_sftp.SFTP_REMOVE
 *     MKDIR = c_sftp.SFTP_MKDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_READDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READDIR, __pyx_t_4) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":324
 *     OPENDIR = c_sftp.SFTP_OPENDIR
 *     READDIR = c_sftp.SFTP_READDIR
 *     REMOVE = c_sftp.SFTP_REMOVE             # <<<<<<<<<<<<<<
 *     MKDIR = c_sftp.SFTP_MKDIR
 *     RMDIR = c_sftp.SFTP_RMDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_REMOVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REMOVE, __pyx_t_4) < 0) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":325
 *     READDIR = c_sftp.SFTP_READDIR
 *     REMOVE = c_sftp.SFTP_REMOVE
 *     MKDIR = c_sftp.SFTP_MKDIR             # <<<<<<<<<<<<<<
 *     RMDIR = c_sftp.SFTP_RMDIR
 *     REALPATH = c_sftp.SFTP_REALPATH
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_MKDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MKDIR, __pyx_t_4) < 0) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":326
 *     REMOVE = c_sftp.SFTP_REMOVE
 *     MKDIR = c_sftp.SFTP_MKDIR
 *     RMDIR = c_sftp.SFTP_RMDIR             # <<<<<<<<<<<<<<
 *     REALPATH = c_sftp.SFTP_REALPATH
 *     STAT = c_sftp.SFTP_STAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_RMDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RMDIR, __pyx_t_4) < 0) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":327
 *     MKDIR = c_sftp.SFTP_MKDIR
 *     RMDIR = c_sftp.SFTP_RMDIR
 *     REALPATH = c_sftp.SFTP_REALPATH             # <<<<<<<<<<<<<<
 *     STAT = c_sftp.SFTP_STAT
 *     RENAME = c_sftp.SFTP_RENAME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_REALPATH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REALPATH, __pyx_t_4) < 0) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":328
 *     RMDIR = c_sftp.SFTP_RMDIR
 *     REALPATH = c_sftp.SFTP_REALPATH
 *     STAT = c_sftp.SFTP_STAT             # <<<<<<<<<<<<<<
 *     RENAME = c_sftp.SFTP_RENAME
 *     READLINK = c_sftp.SFTP_READLINK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_STAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STAT, __pyx_t_4) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":329
 *     REALPATH = c_sftp.SFTP_REALPATH
 *     STAT = c_sftp.SFTP_STAT
 *     RENAME = c_sftp.SFTP_RENAME             # <<<<<<<<<<<<<<
 *     READLINK = c_sftp.SFTP_READLINK
 *     SYMLINK = c_sftp.SFTP_SYMLINK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_RENAME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RENAME, __pyx_t_4) < 0) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":330
 *     STAT = c_sftp.SFTP_STAT
 *     RENAME = c_sftp.SFTP_RENAME
 *     READLINK = c_sftp.SFTP_READLINK             # <<<<<<<<<<<<<<
 *     SYMLINK = c_sftp.SFTP_SYMLINK
 * class SFTP_FXP(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_READLINK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READLINK, __pyx_t_4) < 0) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":331
 *     RENAME = c_sftp.SFTP_RENAME
 *     READLINK = c_sftp.SFTP_READLINK
 *     SYMLINK = c_sftp.SFTP_SYMLINK             # <<<<<<<<<<<<<<
 * class SFTP_FXP(enum.Enum):
 *     INIT = c_sftp.SSH_FXP_INIT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SFTP_SYMLINK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SYMLINK, __pyx_t_4) < 0) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":313
 * 
 * #sftp
 * class SFTP(enum.Enum):             # <<<<<<<<<<<<<<
 *     OPEN = c_sftp.SFTP_OPEN
 *     CLOSE = c_sftp.SFTP_CLOSE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP, __pyx_t_4) < 0) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":332
 *     READLINK = c_sftp.SFTP_READLINK
 *     SYMLINK = c_sftp.SFTP_SYMLINK
 * class SFTP_FXP(enum.Enum):             # <<<<<<<<<<<<<<
 *     INIT = c_sftp.SSH_FXP_INIT
 *     VERSION = c_sftp.SSH_FXP_VERSION
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FXP, __pyx_n_s_SFTP_FXP, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":333
 *     SYMLINK = c_sftp.SFTP_SYMLINK
 * class SFTP_FXP(enum.Enum):
 *     INIT = c_sftp.SSH_FXP_INIT             # <<<<<<<<<<<<<<
 *     VERSION = c_sftp.SSH_FXP_VERSION
 *     OPEN = c_sftp.SSH_FXP_OPEN
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_INIT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INIT, __pyx_t_4) < 0) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":334
 * class SFTP_FXP(enum.Enum):
 *     INIT = c_sftp.SSH_FXP_INIT
 *     VERSION = c_sftp.SSH_FXP_VERSION             # <<<<<<<<<<<<<<
 *     OPEN = c_sftp.SSH_FXP_OPEN
 *     CLOSE = c_sftp.SSH_FXP_CLOSE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_VERSION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_VERSION, __pyx_t_4) < 0) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":335
 *     INIT = c_sftp.SSH_FXP_INIT
 *     VERSION = c_sftp.SSH_FXP_VERSION
 *     OPEN = c_sftp.SSH_FXP_OPEN             # <<<<<<<<<<<<<<
 *     CLOSE = c_sftp.SSH_FXP_CLOSE
 *     READ = c_sftp.SSH_FXP_READ
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_OPEN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPEN, __pyx_t_4) < 0) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":336
 *     VERSION = c_sftp.SSH_FXP_VERSION
 *     OPEN = c_sftp.SSH_FXP_OPEN
 *     CLOSE = c_sftp.SSH_FXP_CLOSE             # <<<<<<<<<<<<<<
 *     READ = c_sftp.SSH_FXP_READ
 *     WRITE = c_sftp.SSH_FXP_WRITE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_CLOSE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLOSE, __pyx_t_4) < 0) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":337
 *     OPEN = c_sftp.SSH_FXP_OPEN
 *     CLOSE = c_sftp.SSH_FXP_CLOSE
 *     READ = c_sftp.SSH_FXP_READ             # <<<<<<<<<<<<<<
 *     WRITE = c_sftp.SSH_FXP_WRITE
 *     LSTAT = c_sftp.SSH_FXP_LSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_READ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READ, __pyx_t_4) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":338
 *     CLOSE = c_sftp.SSH_FXP_CLOSE
 *     READ = c_sftp.SSH_FXP_READ
 *     WRITE = c_sftp.SSH_FXP_WRITE             # <<<<<<<<<<<<<<
 *     LSTAT = c_sftp.SSH_FXP_LSTAT
 *     FSTAT = c_sftp.SSH_FXP_FSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_WRITE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 338, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRITE, __pyx_t_4) < 0) __PYX_ERR(0, 338, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":339
 *     READ = c_sftp.SSH_FXP_READ
 *     WRITE = c_sftp.SSH_FXP_WRITE
 *     LSTAT = c_sftp.SSH_FXP_LSTAT             # <<<<<<<<<<<<<<
 *     FSTAT = c_sftp.SSH_FXP_FSTAT
 *     SETSTAT = c_sftp.SSH_FXP_SETSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_LSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_LSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":340
 *     WRITE = c_sftp.SSH_FXP_WRITE
 *     LSTAT = c_sftp.SSH_FXP_LSTAT
 *     FSTAT = c_sftp.SSH_FXP_FSTAT             # <<<<<<<<<<<<<<
 *     SETSTAT = c_sftp.SSH_FXP_SETSTAT
 *     FSETSTAT = c_sftp.SSH_FXP_FSETSTAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_FSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":341
 *     LSTAT = c_sftp.SSH_FXP_LSTAT
 *     FSTAT = c_sftp.SSH_FXP_FSTAT
 *     SETSTAT = c_sftp.SSH_FXP_SETSTAT             # <<<<<<<<<<<<<<
 *     FSETSTAT = c_sftp.SSH_FXP_FSETSTAT
 *     OPENDIR = c_sftp.SSH_FXP_OPENDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_SETSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SETSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":342
 *     FSTAT = c_sftp.SSH_FXP_FSTAT
 *     SETSTAT = c_sftp.SSH_FXP_SETSTAT
 *     FSETSTAT = c_sftp.SSH_FXP_FSETSTAT             # <<<<<<<<<<<<<<
 *     OPENDIR = c_sftp.SSH_FXP_OPENDIR
 *     READDIR = c_sftp.SSH_FXP_READDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_FSETSTAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FSETSTAT, __pyx_t_4) < 0) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":343
 *     SETSTAT = c_sftp.SSH_FXP_SETSTAT
 *     FSETSTAT = c_sftp.SSH_FXP_FSETSTAT
 *     OPENDIR = c_sftp.SSH_FXP_OPENDIR             # <<<<<<<<<<<<<<
 *     READDIR = c_sftp.SSH_FXP_READDIR
 *     REMOVE = c_sftp.SSH_FXP_REMOVE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_OPENDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OPENDIR, __pyx_t_4) < 0) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":344
 *     FSETSTAT = c_sftp.SSH_FXP_FSETSTAT
 *     OPENDIR = c_sftp.SSH_FXP_OPENDIR
 *     READDIR = c_sftp.SSH_FXP_READDIR             # <<<<<<<<<<<<<<
 *     REMOVE = c_sftp.SSH_FXP_REMOVE
 *     MKDIR = c_sftp.SSH_FXP_MKDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_READDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 344, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READDIR, __pyx_t_4) < 0) __PYX_ERR(0, 344, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":345
 *     OPENDIR = c_sftp.SSH_FXP_OPENDIR
 *     READDIR = c_sftp.SSH_FXP_READDIR
 *     REMOVE = c_sftp.SSH_FXP_REMOVE             # <<<<<<<<<<<<<<
 *     MKDIR = c_sftp.SSH_FXP_MKDIR
 *     RMDIR = c_sftp.SSH_FXP_RMDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_REMOVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REMOVE, __pyx_t_4) < 0) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":346
 *     READDIR = c_sftp.SSH_FXP_READDIR
 *     REMOVE = c_sftp.SSH_FXP_REMOVE
 *     MKDIR = c_sftp.SSH_FXP_MKDIR             # <<<<<<<<<<<<<<
 *     RMDIR = c_sftp.SSH_FXP_RMDIR
 *     REALPATH = c_sftp.SSH_FXP_REALPATH
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_MKDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MKDIR, __pyx_t_4) < 0) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":347
 *     REMOVE = c_sftp.SSH_FXP_REMOVE
 *     MKDIR = c_sftp.SSH_FXP_MKDIR
 *     RMDIR = c_sftp.SSH_FXP_RMDIR             # <<<<<<<<<<<<<<
 *     REALPATH = c_sftp.SSH_FXP_REALPATH
 *     STAT = c_sftp.SSH_FXP_STAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_RMDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RMDIR, __pyx_t_4) < 0) __PYX_ERR(0, 347, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":348
 *     MKDIR = c_sftp.SSH_FXP_MKDIR
 *     RMDIR = c_sftp.SSH_FXP_RMDIR
 *     REALPATH = c_sftp.SSH_FXP_REALPATH             # <<<<<<<<<<<<<<
 *     STAT = c_sftp.SSH_FXP_STAT
 *     RENAME = c_sftp.SSH_FXP_RENAME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_REALPATH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REALPATH, __pyx_t_4) < 0) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":349
 *     RMDIR = c_sftp.SSH_FXP_RMDIR
 *     REALPATH = c_sftp.SSH_FXP_REALPATH
 *     STAT = c_sftp.SSH_FXP_STAT             # <<<<<<<<<<<<<<
 *     RENAME = c_sftp.SSH_FXP_RENAME
 *     READLINK = c_sftp.SSH_FXP_READLINK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_STAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STAT, __pyx_t_4) < 0) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":350
 *     REALPATH = c_sftp.SSH_FXP_REALPATH
 *     STAT = c_sftp.SSH_FXP_STAT
 *     RENAME = c_sftp.SSH_FXP_RENAME             # <<<<<<<<<<<<<<
 *     READLINK = c_sftp.SSH_FXP_READLINK
 *     SYMLINK = c_sftp.SSH_FXP_SYMLINK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_RENAME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_RENAME, __pyx_t_4) < 0) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":351
 *     STAT = c_sftp.SSH_FXP_STAT
 *     RENAME = c_sftp.SSH_FXP_RENAME
 *     READLINK = c_sftp.SSH_FXP_READLINK             # <<<<<<<<<<<<<<
 *     SYMLINK = c_sftp.SSH_FXP_SYMLINK
 *     STATUS = c_sftp.SSH_FXP_STATUS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_READLINK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READLINK, __pyx_t_4) < 0) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":352
 *     RENAME = c_sftp.SSH_FXP_RENAME
 *     READLINK = c_sftp.SSH_FXP_READLINK
 *     SYMLINK = c_sftp.SSH_FXP_SYMLINK             # <<<<<<<<<<<<<<
 *     STATUS = c_sftp.SSH_FXP_STATUS
 *     HANDLE = c_sftp.SSH_FXP_HANDLE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_SYMLINK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SYMLINK, __pyx_t_4) < 0) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":353
 *     READLINK = c_sftp.SSH_FXP_READLINK
 *     SYMLINK = c_sftp.SSH_FXP_SYMLINK
 *     STATUS = c_sftp.SSH_FXP_STATUS             # <<<<<<<<<<<<<<
 *     HANDLE = c_sftp.SSH_FXP_HANDLE
 *     DATA = c_sftp.SSH_FXP_DATA
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_STATUS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STATUS, __pyx_t_4) < 0) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":354
 *     SYMLINK = c_sftp.SSH_FXP_SYMLINK
 *     STATUS = c_sftp.SSH_FXP_STATUS
 *     HANDLE = c_sftp.SSH_FXP_HANDLE             # <<<<<<<<<<<<<<
 *     DATA = c_sftp.SSH_FXP_DATA
 *     NAME = c_sftp.SSH_FXP_NAME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_HANDLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_HANDLE, __pyx_t_4) < 0) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":355
 *     STATUS = c_sftp.SSH_FXP_STATUS
 *     HANDLE = c_sftp.SSH_FXP_HANDLE
 *     DATA = c_sftp.SSH_FXP_DATA             # <<<<<<<<<<<<<<
 *     NAME = c_sftp.SSH_FXP_NAME
 *     ATTRS = c_sftp.SSH_FXP_ATTRS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_DATA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DATA, __pyx_t_4) < 0) __PYX_ERR(0, 355, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":356
 *     HANDLE = c_sftp.SSH_FXP_HANDLE
 *     DATA = c_sftp.SSH_FXP_DATA
 *     NAME = c_sftp.SSH_FXP_NAME             # <<<<<<<<<<<<<<
 *     ATTRS = c_sftp.SSH_FXP_ATTRS
 *     EXTENDED = c_sftp.SSH_FXP_EXTENDED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_NAME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NAME, __pyx_t_4) < 0) __PYX_ERR(0, 356, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":357
 *     DATA = c_sftp.SSH_FXP_DATA
 *     NAME = c_sftp.SSH_FXP_NAME
 *     ATTRS = c_sftp.SSH_FXP_ATTRS             # <<<<<<<<<<<<<<
 *     EXTENDED = c_sftp.SSH_FXP_EXTENDED
 *     EXTENDED_REPLY = c_sftp.SSH_FXP_EXTENDED_REPLY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_ATTRS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ATTRS, __pyx_t_4) < 0) __PYX_ERR(0, 357, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":358
 *     NAME = c_sftp.SSH_FXP_NAME
 *     ATTRS = c_sftp.SSH_FXP_ATTRS
 *     EXTENDED = c_sftp.SSH_FXP_EXTENDED             # <<<<<<<<<<<<<<
 *     EXTENDED_REPLY = c_sftp.SSH_FXP_EXTENDED_REPLY
 * class SFTP_FileXfer_Attr(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_EXTENDED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXTENDED, __pyx_t_4) < 0) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":359
 *     ATTRS = c_sftp.SSH_FXP_ATTRS
 *     EXTENDED = c_sftp.SSH_FXP_EXTENDED
 *     EXTENDED_REPLY = c_sftp.SSH_FXP_EXTENDED_REPLY             # <<<<<<<<<<<<<<
 * class SFTP_FileXfer_Attr(enum.Enum):
 *     SIZE = c_sftp.SSH_FILEXFER_ATTR_SIZE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXP_EXTENDED_REPLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXTENDED_REPLY, __pyx_t_4) < 0) __PYX_ERR(0, 359, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":332
 *     READLINK = c_sftp.SFTP_READLINK
 *     SYMLINK = c_sftp.SFTP_SYMLINK
 * class SFTP_FXP(enum.Enum):             # <<<<<<<<<<<<<<
 *     INIT = c_sftp.SSH_FXP_INIT
 *     VERSION = c_sftp.SSH_FXP_VERSION
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FXP, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FXP, __pyx_t_4) < 0) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":360
 *     EXTENDED = c_sftp.SSH_FXP_EXTENDED
 *     EXTENDED_REPLY = c_sftp.SSH_FXP_EXTENDED_REPLY
 * class SFTP_FileXfer_Attr(enum.Enum):             # <<<<<<<<<<<<<<
 *     SIZE = c_sftp.SSH_FILEXFER_ATTR_SIZE
 *     PERMISSIONS = c_sftp.SSH_FILEXFER_ATTR_PERMISSIONS
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FileXfer_Attr, __pyx_n_s_SFTP_FileXfer_Attr, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":361
 *     EXTENDED_REPLY = c_sftp.SSH_FXP_EXTENDED_REPLY
 * class SFTP_FileXfer_Attr(enum.Enum):
 *     SIZE = c_sftp.SSH_FILEXFER_ATTR_SIZE             # <<<<<<<<<<<<<<
 *     PERMISSIONS = c_sftp.SSH_FILEXFER_ATTR_PERMISSIONS
 *     ACCESSTIME = c_sftp.SSH_FILEXFER_ATTR_ACCESSTIME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_SIZE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SIZE, __pyx_t_4) < 0) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":362
 * class SFTP_FileXfer_Attr(enum.Enum):
 *     SIZE = c_sftp.SSH_FILEXFER_ATTR_SIZE
 *     PERMISSIONS = c_sftp.SSH_FILEXFER_ATTR_PERMISSIONS             # <<<<<<<<<<<<<<
 *     ACCESSTIME = c_sftp.SSH_FILEXFER_ATTR_ACCESSTIME
 *     ACMODTIME = c_sftp.SSH_FILEXFER_ATTR_ACMODTIME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_PERMISSIONS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PERMISSIONS, __pyx_t_4) < 0) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":363
 *     SIZE = c_sftp.SSH_FILEXFER_ATTR_SIZE
 *     PERMISSIONS = c_sftp.SSH_FILEXFER_ATTR_PERMISSIONS
 *     ACCESSTIME = c_sftp.SSH_FILEXFER_ATTR_ACCESSTIME             # <<<<<<<<<<<<<<
 *     ACMODTIME = c_sftp.SSH_FILEXFER_ATTR_ACMODTIME
 *     CREATETIME = c_sftp.SSH_FILEXFER_ATTR_CREATETIME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_ACCESSTIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ACCESSTIME, __pyx_t_4) < 0) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":364
 *     PERMISSIONS = c_sftp.SSH_FILEXFER_ATTR_PERMISSIONS
 *     ACCESSTIME = c_sftp.SSH_FILEXFER_ATTR_ACCESSTIME
 *     ACMODTIME = c_sftp.SSH_FILEXFER_ATTR_ACMODTIME             # <<<<<<<<<<<<<<
 *     CREATETIME = c_sftp.SSH_FILEXFER_ATTR_CREATETIME
 *     MODIFYTIME = c_sftp.SSH_FILEXFER_ATTR_MODIFYTIME
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_ACMODTIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ACMODTIME, __pyx_t_4) < 0) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":365
 *     ACCESSTIME = c_sftp.SSH_FILEXFER_ATTR_ACCESSTIME
 *     ACMODTIME = c_sftp.SSH_FILEXFER_ATTR_ACMODTIME
 *     CREATETIME = c_sftp.SSH_FILEXFER_ATTR_CREATETIME             # <<<<<<<<<<<<<<
 *     MODIFYTIME = c_sftp.SSH_FILEXFER_ATTR_MODIFYTIME
 *     ACL = c_sftp.SSH_FILEXFER_ATTR_ACL
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_CREATETIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CREATETIME, __pyx_t_4) < 0) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":366
 *     ACMODTIME = c_sftp.SSH_FILEXFER_ATTR_ACMODTIME
 *     CREATETIME = c_sftp.SSH_FILEXFER_ATTR_CREATETIME
 *     MODIFYTIME = c_sftp.SSH_FILEXFER_ATTR_MODIFYTIME             # <<<<<<<<<<<<<<
 *     ACL = c_sftp.SSH_FILEXFER_ATTR_ACL
 *     OWNERGROUP = c_sftp.SSH_FILEXFER_ATTR_OWNERGROUP
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_MODIFYTIME); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_MODIFYTIME, __pyx_t_4) < 0) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":367
 *     CREATETIME = c_sftp.SSH_FILEXFER_ATTR_CREATETIME
 *     MODIFYTIME = c_sftp.SSH_FILEXFER_ATTR_MODIFYTIME
 *     ACL = c_sftp.SSH_FILEXFER_ATTR_ACL             # <<<<<<<<<<<<<<
 *     OWNERGROUP = c_sftp.SSH_FILEXFER_ATTR_OWNERGROUP
 *     SUBSECOND_TIMES = c_sftp.SSH_FILEXFER_ATTR_SUBSECOND_TIMES
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_ACL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ACL, __pyx_t_4) < 0) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":368
 *     MODIFYTIME = c_sftp.SSH_FILEXFER_ATTR_MODIFYTIME
 *     ACL = c_sftp.SSH_FILEXFER_ATTR_ACL
 *     OWNERGROUP = c_sftp.SSH_FILEXFER_ATTR_OWNERGROUP             # <<<<<<<<<<<<<<
 *     SUBSECOND_TIMES = c_sftp.SSH_FILEXFER_ATTR_SUBSECOND_TIMES
 *     EXTENDED = c_sftp.SSH_FILEXFER_ATTR_EXTENDED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_OWNERGROUP); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OWNERGROUP, __pyx_t_4) < 0) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":369
 *     ACL = c_sftp.SSH_FILEXFER_ATTR_ACL
 *     OWNERGROUP = c_sftp.SSH_FILEXFER_ATTR_OWNERGROUP
 *     SUBSECOND_TIMES = c_sftp.SSH_FILEXFER_ATTR_SUBSECOND_TIMES             # <<<<<<<<<<<<<<
 *     EXTENDED = c_sftp.SSH_FILEXFER_ATTR_EXTENDED
 *     UIDGID = c_sftp.SSH_FILEXFER_ATTR_UIDGID
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_SUBSECOND_TIMES); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SUBSECOND_TIMES, __pyx_t_4) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":370
 *     OWNERGROUP = c_sftp.SSH_FILEXFER_ATTR_OWNERGROUP
 *     SUBSECOND_TIMES = c_sftp.SSH_FILEXFER_ATTR_SUBSECOND_TIMES
 *     EXTENDED = c_sftp.SSH_FILEXFER_ATTR_EXTENDED             # <<<<<<<<<<<<<<
 *     UIDGID = c_sftp.SSH_FILEXFER_ATTR_UIDGID
 * # Types
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_EXTENDED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXTENDED, __pyx_t_4) < 0) __PYX_ERR(0, 370, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":371
 *     SUBSECOND_TIMES = c_sftp.SSH_FILEXFER_ATTR_SUBSECOND_TIMES
 *     EXTENDED = c_sftp.SSH_FILEXFER_ATTR_EXTENDED
 *     UIDGID = c_sftp.SSH_FILEXFER_ATTR_UIDGID             # <<<<<<<<<<<<<<
 * # Types
 * class SFTP_FileXfer_Type(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_ATTR_UIDGID); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UIDGID, __pyx_t_4) < 0) __PYX_ERR(0, 371, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":360
 *     EXTENDED = c_sftp.SSH_FXP_EXTENDED
 *     EXTENDED_REPLY = c_sftp.SSH_FXP_EXTENDED_REPLY
 * class SFTP_FileXfer_Attr(enum.Enum):             # <<<<<<<<<<<<<<
 *     SIZE = c_sftp.SSH_FILEXFER_ATTR_SIZE
 *     PERMISSIONS = c_sftp.SSH_FILEXFER_ATTR_PERMISSIONS
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FileXfer_Attr, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FileXfer_Attr, __pyx_t_4) < 0) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":373
 *     UIDGID = c_sftp.SSH_FILEXFER_ATTR_UIDGID
 * # Types
 * class SFTP_FileXfer_Type(enum.Enum):             # <<<<<<<<<<<<<<
 *     REGULAR = c_sftp.SSH_FILEXFER_TYPE_REGULAR
 *     DIRECTORY = c_sftp.SSH_FILEXFER_TYPE_DIRECTORY
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FileXfer_Type, __pyx_n_s_SFTP_FileXfer_Type, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":374
 * # Types
 * class SFTP_FileXfer_Type(enum.Enum):
 *     REGULAR = c_sftp.SSH_FILEXFER_TYPE_REGULAR             # <<<<<<<<<<<<<<
 *     DIRECTORY = c_sftp.SSH_FILEXFER_TYPE_DIRECTORY
 *     SYMLINK = c_sftp.SSH_FILEXFER_TYPE_SYMLINK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_TYPE_REGULAR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_REGULAR, __pyx_t_4) < 0) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":375
 * class SFTP_FileXfer_Type(enum.Enum):
 *     REGULAR = c_sftp.SSH_FILEXFER_TYPE_REGULAR
 *     DIRECTORY = c_sftp.SSH_FILEXFER_TYPE_DIRECTORY             # <<<<<<<<<<<<<<
 *     SYMLINK = c_sftp.SSH_FILEXFER_TYPE_SYMLINK
 *     SPECIAL = c_sftp.SSH_FILEXFER_TYPE_SPECIAL
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_TYPE_DIRECTORY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_DIRECTORY, __pyx_t_4) < 0) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":376
 *     REGULAR = c_sftp.SSH_FILEXFER_TYPE_REGULAR
 *     DIRECTORY = c_sftp.SSH_FILEXFER_TYPE_DIRECTORY
 *     SYMLINK = c_sftp.SSH_FILEXFER_TYPE_SYMLINK             # <<<<<<<<<<<<<<
 *     SPECIAL = c_sftp.SSH_FILEXFER_TYPE_SPECIAL
 *     UNKNOWN = c_sftp.SSH_FILEXFER_TYPE_UNKNOWN
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_TYPE_SYMLINK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SYMLINK, __pyx_t_4) < 0) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":377
 *     DIRECTORY = c_sftp.SSH_FILEXFER_TYPE_DIRECTORY
 *     SYMLINK = c_sftp.SSH_FILEXFER_TYPE_SYMLINK
 *     SPECIAL = c_sftp.SSH_FILEXFER_TYPE_SPECIAL             # <<<<<<<<<<<<<<
 *     UNKNOWN = c_sftp.SSH_FILEXFER_TYPE_UNKNOWN
 * class SFTP_FX(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_TYPE_SPECIAL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_SPECIAL, __pyx_t_4) < 0) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":378
 *     SYMLINK = c_sftp.SSH_FILEXFER_TYPE_SYMLINK
 *     SPECIAL = c_sftp.SSH_FILEXFER_TYPE_SPECIAL
 *     UNKNOWN = c_sftp.SSH_FILEXFER_TYPE_UNKNOWN             # <<<<<<<<<<<<<<
 * class SFTP_FX(enum.Enum):
 *     OK = c_sftp.SSH_FX_OK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FILEXFER_TYPE_UNKNOWN); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_UNKNOWN, __pyx_t_4) < 0) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":373
 *     UIDGID = c_sftp.SSH_FILEXFER_ATTR_UIDGID
 * # Types
 * class SFTP_FileXfer_Type(enum.Enum):             # <<<<<<<<<<<<<<
 *     REGULAR = c_sftp.SSH_FILEXFER_TYPE_REGULAR
 *     DIRECTORY = c_sftp.SSH_FILEXFER_TYPE_DIRECTORY
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FileXfer_Type, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FileXfer_Type, __pyx_t_4) < 0) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":379
 *     SPECIAL = c_sftp.SSH_FILEXFER_TYPE_SPECIAL
 *     UNKNOWN = c_sftp.SSH_FILEXFER_TYPE_UNKNOWN
 * class SFTP_FX(enum.Enum):             # <<<<<<<<<<<<<<
 *     OK = c_sftp.SSH_FX_OK
 *     EOF = c_sftp.SSH_FX_EOF
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FX, __pyx_n_s_SFTP_FX, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":380
 *     UNKNOWN = c_sftp.SSH_FILEXFER_TYPE_UNKNOWN
 * class SFTP_FX(enum.Enum):
 *     OK = c_sftp.SSH_FX_OK             # <<<<<<<<<<<<<<
 *     EOF = c_sftp.SSH_FX_EOF
 *     NO_SUCH_FILE = c_sftp.SSH_FX_NO_SUCH_FILE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_OK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OK, __pyx_t_4) < 0) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":381
 * class SFTP_FX(enum.Enum):
 *     OK = c_sftp.SSH_FX_OK
 *     EOF = c_sftp.SSH_FX_EOF             # <<<<<<<<<<<<<<
 *     NO_SUCH_FILE = c_sftp.SSH_FX_NO_SUCH_FILE
 *     PERMISSION_DENIED = c_sftp.SSH_FX_PERMISSION_DENIED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_EOF); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EOF, __pyx_t_4) < 0) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":382
 *     OK = c_sftp.SSH_FX_OK
 *     EOF = c_sftp.SSH_FX_EOF
 *     NO_SUCH_FILE = c_sftp.SSH_FX_NO_SUCH_FILE             # <<<<<<<<<<<<<<
 *     PERMISSION_DENIED = c_sftp.SSH_FX_PERMISSION_DENIED
 *     FAILURE = c_sftp.SSH_FX_FAILURE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_NO_SUCH_FILE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NO_SUCH_FILE, __pyx_t_4) < 0) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":383
 *     EOF = c_sftp.SSH_FX_EOF
 *     NO_SUCH_FILE = c_sftp.SSH_FX_NO_SUCH_FILE
 *     PERMISSION_DENIED = c_sftp.SSH_FX_PERMISSION_DENIED             # <<<<<<<<<<<<<<
 *     FAILURE = c_sftp.SSH_FX_FAILURE
 *     BAD_MESSAGE = c_sftp.SSH_FX_BAD_MESSAGE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_PERMISSION_DENIED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_PERMISSION_DENIED, __pyx_t_4) < 0) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":384
 *     NO_SUCH_FILE = c_sftp.SSH_FX_NO_SUCH_FILE
 *     PERMISSION_DENIED = c_sftp.SSH_FX_PERMISSION_DENIED
 *     FAILURE = c_sftp.SSH_FX_FAILURE             # <<<<<<<<<<<<<<
 *     BAD_MESSAGE = c_sftp.SSH_FX_BAD_MESSAGE
 *     NO_CONNECTION = c_sftp.SSH_FX_NO_CONNECTION
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_FAILURE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FAILURE, __pyx_t_4) < 0) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":385
 *     PERMISSION_DENIED = c_sftp.SSH_FX_PERMISSION_DENIED
 *     FAILURE = c_sftp.SSH_FX_FAILURE
 *     BAD_MESSAGE = c_sftp.SSH_FX_BAD_MESSAGE             # <<<<<<<<<<<<<<
 *     NO_CONNECTION = c_sftp.SSH_FX_NO_CONNECTION
 *     CONNECTION_LOST = c_sftp.SSH_FX_CONNECTION_LOST
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_BAD_MESSAGE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_BAD_MESSAGE, __pyx_t_4) < 0) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":386
 *     FAILURE = c_sftp.SSH_FX_FAILURE
 *     BAD_MESSAGE = c_sftp.SSH_FX_BAD_MESSAGE
 *     NO_CONNECTION = c_sftp.SSH_FX_NO_CONNECTION             # <<<<<<<<<<<<<<
 *     CONNECTION_LOST = c_sftp.SSH_FX_CONNECTION_LOST
 *     OP_UNSUPPORTED = c_sftp.SSH_FX_OP_UNSUPPORTED
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_NO_CONNECTION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NO_CONNECTION, __pyx_t_4) < 0) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":387
 *     BAD_MESSAGE = c_sftp.SSH_FX_BAD_MESSAGE
 *     NO_CONNECTION = c_sftp.SSH_FX_NO_CONNECTION
 *     CONNECTION_LOST = c_sftp.SSH_FX_CONNECTION_LOST             # <<<<<<<<<<<<<<
 *     OP_UNSUPPORTED = c_sftp.SSH_FX_OP_UNSUPPORTED
 *     INVALID_HANDLE = c_sftp.SSH_FX_INVALID_HANDLE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_CONNECTION_LOST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CONNECTION_LOST, __pyx_t_4) < 0) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":388
 *     NO_CONNECTION = c_sftp.SSH_FX_NO_CONNECTION
 *     CONNECTION_LOST = c_sftp.SSH_FX_CONNECTION_LOST
 *     OP_UNSUPPORTED = c_sftp.SSH_FX_OP_UNSUPPORTED             # <<<<<<<<<<<<<<
 *     INVALID_HANDLE = c_sftp.SSH_FX_INVALID_HANDLE
 *     NO_SUCH_PATH = c_sftp.SSH_FX_NO_SUCH_PATH
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_OP_UNSUPPORTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OP_UNSUPPORTED, __pyx_t_4) < 0) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":389
 *     CONNECTION_LOST = c_sftp.SSH_FX_CONNECTION_LOST
 *     OP_UNSUPPORTED = c_sftp.SSH_FX_OP_UNSUPPORTED
 *     INVALID_HANDLE = c_sftp.SSH_FX_INVALID_HANDLE             # <<<<<<<<<<<<<<
 *     NO_SUCH_PATH = c_sftp.SSH_FX_NO_SUCH_PATH
 *     FILE_ALREADY_EXISTS = c_sftp.SSH_FX_FILE_ALREADY_EXISTS
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_INVALID_HANDLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_INVALID_HANDLE, __pyx_t_4) < 0) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":390
 *     OP_UNSUPPORTED = c_sftp.SSH_FX_OP_UNSUPPORTED
 *     INVALID_HANDLE = c_sftp.SSH_FX_INVALID_HANDLE
 *     NO_SUCH_PATH = c_sftp.SSH_FX_NO_SUCH_PATH             # <<<<<<<<<<<<<<
 *     FILE_ALREADY_EXISTS = c_sftp.SSH_FX_FILE_ALREADY_EXISTS
 *     WRITE_PROTECT = c_sftp.SSH_FX_WRITE_PROTECT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_NO_SUCH_PATH); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NO_SUCH_PATH, __pyx_t_4) < 0) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":391
 *     INVALID_HANDLE = c_sftp.SSH_FX_INVALID_HANDLE
 *     NO_SUCH_PATH = c_sftp.SSH_FX_NO_SUCH_PATH
 *     FILE_ALREADY_EXISTS = c_sftp.SSH_FX_FILE_ALREADY_EXISTS             # <<<<<<<<<<<<<<
 *     WRITE_PROTECT = c_sftp.SSH_FX_WRITE_PROTECT
 *     NO_MEDIA = c_sftp.SSH_FX_NO_MEDIA
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_FILE_ALREADY_EXISTS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_FILE_ALREADY_EXISTS, __pyx_t_4) < 0) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":392
 *     NO_SUCH_PATH = c_sftp.SSH_FX_NO_SUCH_PATH
 *     FILE_ALREADY_EXISTS = c_sftp.SSH_FX_FILE_ALREADY_EXISTS
 *     WRITE_PROTECT = c_sftp.SSH_FX_WRITE_PROTECT             # <<<<<<<<<<<<<<
 *     NO_MEDIA = c_sftp.SSH_FX_NO_MEDIA
 * class SFTP_FXF(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_WRITE_PROTECT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRITE_PROTECT, __pyx_t_4) < 0) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":393
 *     FILE_ALREADY_EXISTS = c_sftp.SSH_FX_FILE_ALREADY_EXISTS
 *     WRITE_PROTECT = c_sftp.SSH_FX_WRITE_PROTECT
 *     NO_MEDIA = c_sftp.SSH_FX_NO_MEDIA             # <<<<<<<<<<<<<<
 * class SFTP_FXF(enum.Enum):
 *     READ = c_sftp.SSH_FXF_READ
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FX_NO_MEDIA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NO_MEDIA, __pyx_t_4) < 0) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":379
 *     SPECIAL = c_sftp.SSH_FILEXFER_TYPE_SPECIAL
 *     UNKNOWN = c_sftp.SSH_FILEXFER_TYPE_UNKNOWN
 * class SFTP_FX(enum.Enum):             # <<<<<<<<<<<<<<
 *     OK = c_sftp.SSH_FX_OK
 *     EOF = c_sftp.SSH_FX_EOF
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FX, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FX, __pyx_t_4) < 0) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":394
 *     WRITE_PROTECT = c_sftp.SSH_FX_WRITE_PROTECT
 *     NO_MEDIA = c_sftp.SSH_FX_NO_MEDIA
 * class SFTP_FXF(enum.Enum):             # <<<<<<<<<<<<<<
 *     READ = c_sftp.SSH_FXF_READ
 *     WRITE = c_sftp.SSH_FXF_WRITE
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FXF, __pyx_n_s_SFTP_FXF, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":395
 *     NO_MEDIA = c_sftp.SSH_FX_NO_MEDIA
 * class SFTP_FXF(enum.Enum):
 *     READ = c_sftp.SSH_FXF_READ             # <<<<<<<<<<<<<<
 *     WRITE = c_sftp.SSH_FXF_WRITE
 *     APPEND = c_sftp.SSH_FXF_APPEND
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_READ); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_READ, __pyx_t_4) < 0) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":396
 * class SFTP_FXF(enum.Enum):
 *     READ = c_sftp.SSH_FXF_READ
 *     WRITE = c_sftp.SSH_FXF_WRITE             # <<<<<<<<<<<<<<
 *     APPEND = c_sftp.SSH_FXF_APPEND
 *     CREAT = c_sftp.SSH_FXF_CREAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_WRITE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_WRITE, __pyx_t_4) < 0) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":397
 *     READ = c_sftp.SSH_FXF_READ
 *     WRITE = c_sftp.SSH_FXF_WRITE
 *     APPEND = c_sftp.SSH_FXF_APPEND             # <<<<<<<<<<<<<<
 *     CREAT = c_sftp.SSH_FXF_CREAT
 *     TRUNC = c_sftp.SSH_FXF_TRUNC
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_APPEND); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_APPEND, __pyx_t_4) < 0) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":398
 *     WRITE = c_sftp.SSH_FXF_WRITE
 *     APPEND = c_sftp.SSH_FXF_APPEND
 *     CREAT = c_sftp.SSH_FXF_CREAT             # <<<<<<<<<<<<<<
 *     TRUNC = c_sftp.SSH_FXF_TRUNC
 *     EXCL = c_sftp.SSH_FXF_EXCL
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_CREAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CREAT, __pyx_t_4) < 0) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":399
 *     APPEND = c_sftp.SSH_FXF_APPEND
 *     CREAT = c_sftp.SSH_FXF_CREAT
 *     TRUNC = c_sftp.SSH_FXF_TRUNC             # <<<<<<<<<<<<<<
 *     EXCL = c_sftp.SSH_FXF_EXCL
 *     TEXT = c_sftp.SSH_FXF_TEXT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_TRUNC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TRUNC, __pyx_t_4) < 0) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":400
 *     CREAT = c_sftp.SSH_FXF_CREAT
 *     TRUNC = c_sftp.SSH_FXF_TRUNC
 *     EXCL = c_sftp.SSH_FXF_EXCL             # <<<<<<<<<<<<<<
 *     TEXT = c_sftp.SSH_FXF_TEXT
 * class SFTP_FXF_Rename(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_EXCL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 400, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_EXCL, __pyx_t_4) < 0) __PYX_ERR(0, 400, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":401
 *     TRUNC = c_sftp.SSH_FXF_TRUNC
 *     EXCL = c_sftp.SSH_FXF_EXCL
 *     TEXT = c_sftp.SSH_FXF_TEXT             # <<<<<<<<<<<<<<
 * class SFTP_FXF_Rename(enum.Enum):
 *     OVERWRITE = c_sftp.SSH_FXF_RENAME_OVERWRITE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_TEXT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_TEXT, __pyx_t_4) < 0) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":394
 *     WRITE_PROTECT = c_sftp.SSH_FX_WRITE_PROTECT
 *     NO_MEDIA = c_sftp.SSH_FX_NO_MEDIA
 * class SFTP_FXF(enum.Enum):             # <<<<<<<<<<<<<<
 *     READ = c_sftp.SSH_FXF_READ
 *     WRITE = c_sftp.SSH_FXF_WRITE
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FXF, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FXF, __pyx_t_4) < 0) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":402
 *     EXCL = c_sftp.SSH_FXF_EXCL
 *     TEXT = c_sftp.SSH_FXF_TEXT
 * class SFTP_FXF_Rename(enum.Enum):             # <<<<<<<<<<<<<<
 *     OVERWRITE = c_sftp.SSH_FXF_RENAME_OVERWRITE
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FXF_Rename, __pyx_n_s_SFTP_FXF_Rename, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":403
 *     TEXT = c_sftp.SSH_FXF_TEXT
 * class SFTP_FXF_Rename(enum.Enum):
 *     OVERWRITE = c_sftp.SSH_FXF_RENAME_OVERWRITE             # <<<<<<<<<<<<<<
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC
 *     NATIVE = c_sftp.SSH_FXF_RENAME_NATIVE
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_RENAME_OVERWRITE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_OVERWRITE, __pyx_t_4) < 0) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":404
 * class SFTP_FXF_Rename(enum.Enum):
 *     OVERWRITE = c_sftp.SSH_FXF_RENAME_OVERWRITE
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC             # <<<<<<<<<<<<<<
 *     NATIVE = c_sftp.SSH_FXF_RENAME_NATIVE
 * class SFTP_S(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_RENAME_ATOMIC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ATOMIC, __pyx_t_4) < 0) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":405
 *     OVERWRITE = c_sftp.SSH_FXF_RENAME_OVERWRITE
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC
 *     NATIVE = c_sftp.SSH_FXF_RENAME_NATIVE             # <<<<<<<<<<<<<<
 * class SFTP_S(enum.Enum):
 *     IFMT = c_sftp.SSH_S_IFMT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXF_RENAME_NATIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_NATIVE, __pyx_t_4) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":402
 *     EXCL = c_sftp.SSH_FXF_EXCL
 *     TEXT = c_sftp.SSH_FXF_TEXT
 * class SFTP_FXF_Rename(enum.Enum):             # <<<<<<<<<<<<<<
 *     OVERWRITE = c_sftp.SSH_FXF_RENAME_OVERWRITE
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FXF_Rename, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FXF_Rename, __pyx_t_4) < 0) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":406
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC
 *     NATIVE = c_sftp.SSH_FXF_RENAME_NATIVE
 * class SFTP_S(enum.Enum):             # <<<<<<<<<<<<<<
 *     IFMT = c_sftp.SSH_S_IFMT
 *     IFSOCK = c_sftp.SSH_S_IFSOCK
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_S, __pyx_n_s_SFTP_S, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":407
 *     NATIVE = c_sftp.SSH_FXF_RENAME_NATIVE
 * class SFTP_S(enum.Enum):
 *     IFMT = c_sftp.SSH_S_IFMT             # <<<<<<<<<<<<<<
 *     IFSOCK = c_sftp.SSH_S_IFSOCK
 *     IFLNK = c_sftp.SSH_S_IFLNK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFMT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFMT, __pyx_t_4) < 0) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":408
 * class SFTP_S(enum.Enum):
 *     IFMT = c_sftp.SSH_S_IFMT
 *     IFSOCK = c_sftp.SSH_S_IFSOCK             # <<<<<<<<<<<<<<
 *     IFLNK = c_sftp.SSH_S_IFLNK
 *     IFREG = c_sftp.SSH_S_IFREG
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFSOCK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFSOCK, __pyx_t_4) < 0) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":409
 *     IFMT = c_sftp.SSH_S_IFMT
 *     IFSOCK = c_sftp.SSH_S_IFSOCK
 *     IFLNK = c_sftp.SSH_S_IFLNK             # <<<<<<<<<<<<<<
 *     IFREG = c_sftp.SSH_S_IFREG
 *     IFBLK = c_sftp.SSH_S_IFBLK
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFLNK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFLNK, __pyx_t_4) < 0) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":410
 *     IFSOCK = c_sftp.SSH_S_IFSOCK
 *     IFLNK = c_sftp.SSH_S_IFLNK
 *     IFREG = c_sftp.SSH_S_IFREG             # <<<<<<<<<<<<<<
 *     IFBLK = c_sftp.SSH_S_IFBLK
 *     IFDIR = c_sftp.SSH_S_IFDIR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFREG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFREG, __pyx_t_4) < 0) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":411
 *     IFLNK = c_sftp.SSH_S_IFLNK
 *     IFREG = c_sftp.SSH_S_IFREG
 *     IFBLK = c_sftp.SSH_S_IFBLK             # <<<<<<<<<<<<<<
 *     IFDIR = c_sftp.SSH_S_IFDIR
 *     IFCHR = c_sftp.SSH_S_IFCHR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFBLK); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 411, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFBLK, __pyx_t_4) < 0) __PYX_ERR(0, 411, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":412
 *     IFREG = c_sftp.SSH_S_IFREG
 *     IFBLK = c_sftp.SSH_S_IFBLK
 *     IFDIR = c_sftp.SSH_S_IFDIR             # <<<<<<<<<<<<<<
 *     IFCHR = c_sftp.SSH_S_IFCHR
 *     IFIFO = c_sftp.SSH_S_IFIFO
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFDIR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFDIR, __pyx_t_4) < 0) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":413
 *     IFBLK = c_sftp.SSH_S_IFBLK
 *     IFDIR = c_sftp.SSH_S_IFDIR
 *     IFCHR = c_sftp.SSH_S_IFCHR             # <<<<<<<<<<<<<<
 *     IFIFO = c_sftp.SSH_S_IFIFO
 * class SFTP_FXE(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFCHR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFCHR, __pyx_t_4) < 0) __PYX_ERR(0, 413, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":414
 *     IFDIR = c_sftp.SSH_S_IFDIR
 *     IFCHR = c_sftp.SSH_S_IFCHR
 *     IFIFO = c_sftp.SSH_S_IFIFO             # <<<<<<<<<<<<<<
 * class SFTP_FXE(enum.Enum):
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_S_IFIFO); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_IFIFO, __pyx_t_4) < 0) __PYX_ERR(0, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":406
 *     ATOMIC = c_sftp.SSH_FXF_RENAME_ATOMIC
 *     NATIVE = c_sftp.SSH_FXF_RENAME_NATIVE
 * class SFTP_S(enum.Enum):             # <<<<<<<<<<<<<<
 *     IFMT = c_sftp.SSH_S_IFMT
 *     IFSOCK = c_sftp.SSH_S_IFSOCK
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_S, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_S, __pyx_t_4) < 0) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":415
 *     IFCHR = c_sftp.SSH_S_IFCHR
 *     IFIFO = c_sftp.SSH_S_IFIFO
 * class SFTP_FXE(enum.Enum):             # <<<<<<<<<<<<<<
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_FXE, __pyx_n_s_SFTP_FXE, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":416
 *     IFIFO = c_sftp.SSH_S_IFIFO
 * class SFTP_FXE(enum.Enum):
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY             # <<<<<<<<<<<<<<
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID
 * class SFTP_AT(enum.Enum):
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXE_STATVFS_ST_RDONLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STATVFS_ST_RDONLY, __pyx_t_4) < 0) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":417
 * class SFTP_FXE(enum.Enum):
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID             # <<<<<<<<<<<<<<
 * class SFTP_AT(enum.Enum):
 *     O_RDONLY = c_sftp.O_RDONLY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(SSH_FXE_STATVFS_ST_NOSUID); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_STATVFS_ST_NOSUID, __pyx_t_4) < 0) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":415
 *     IFCHR = c_sftp.SSH_S_IFCHR
 *     IFIFO = c_sftp.SSH_S_IFIFO
 * class SFTP_FXE(enum.Enum):             # <<<<<<<<<<<<<<
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_FXE, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_FXE, __pyx_t_4) < 0) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":418
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID
 * class SFTP_AT(enum.Enum):             # <<<<<<<<<<<<<<
 *     O_RDONLY = c_sftp.O_RDONLY
 *     O_WRONLY = c_sftp.O_WRONLY
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SFTP_AT, __pyx_n_s_SFTP_AT, (PyObject *) NULL, __pyx_n_s_ssh_libssh_enums, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssh/libssh_enums.pyx":419
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID
 * class SFTP_AT(enum.Enum):
 *     O_RDONLY = c_sftp.O_RDONLY             # <<<<<<<<<<<<<<
 *     O_WRONLY = c_sftp.O_WRONLY
 *     O_RDWR = c_sftp.O_RDWR
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_RDONLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_RDONLY, __pyx_t_4) < 0) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":420
 * class SFTP_AT(enum.Enum):
 *     O_RDONLY = c_sftp.O_RDONLY
 *     O_WRONLY = c_sftp.O_WRONLY             # <<<<<<<<<<<<<<
 *     O_RDWR = c_sftp.O_RDWR
 *     O_CREAT = c_sftp.O_CREAT
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_WRONLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_WRONLY, __pyx_t_4) < 0) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":421
 *     O_RDONLY = c_sftp.O_RDONLY
 *     O_WRONLY = c_sftp.O_WRONLY
 *     O_RDWR = c_sftp.O_RDWR             # <<<<<<<<<<<<<<
 *     O_CREAT = c_sftp.O_CREAT
 *     O_EXCL = c_sftp.O_EXCL
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_RDWR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_RDWR, __pyx_t_4) < 0) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":422
 *     O_WRONLY = c_sftp.O_WRONLY
 *     O_RDWR = c_sftp.O_RDWR
 *     O_CREAT = c_sftp.O_CREAT             # <<<<<<<<<<<<<<
 *     O_EXCL = c_sftp.O_EXCL
 *     O_NOCTTY = c_sftp.O_NOCTTY
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_CREAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_CREAT, __pyx_t_4) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":423
 *     O_RDWR = c_sftp.O_RDWR
 *     O_CREAT = c_sftp.O_CREAT
 *     O_EXCL = c_sftp.O_EXCL             # <<<<<<<<<<<<<<
 *     O_NOCTTY = c_sftp.O_NOCTTY
 *     O_NOFOLLOW = c_sftp.O_NOFOLLOW
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_EXCL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_EXCL, __pyx_t_4) < 0) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":424
 *     O_CREAT = c_sftp.O_CREAT
 *     O_EXCL = c_sftp.O_EXCL
 *     O_NOCTTY = c_sftp.O_NOCTTY             # <<<<<<<<<<<<<<
 *     O_NOFOLLOW = c_sftp.O_NOFOLLOW
 *     O_TRUNC = c_sftp.O_TRUNC
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_NOCTTY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_NOCTTY, __pyx_t_4) < 0) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":425
 *     O_EXCL = c_sftp.O_EXCL
 *     O_NOCTTY = c_sftp.O_NOCTTY
 *     O_NOFOLLOW = c_sftp.O_NOFOLLOW             # <<<<<<<<<<<<<<
 *     O_TRUNC = c_sftp.O_TRUNC
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_NOFOLLOW); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_NOFOLLOW, __pyx_t_4) < 0) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":426
 *     O_NOCTTY = c_sftp.O_NOCTTY
 *     O_NOFOLLOW = c_sftp.O_NOFOLLOW
 *     O_TRUNC = c_sftp.O_TRUNC             # <<<<<<<<<<<<<<
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(O_TRUNC); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_O_TRUNC, __pyx_t_4) < 0) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ssh/libssh_enums.pyx":418
 *     STATVFS_ST_RDONLY = c_sftp.SSH_FXE_STATVFS_ST_RDONLY
 *     STATVFS_ST_NOSUID = c_sftp.SSH_FXE_STATVFS_ST_NOSUID
 * class SFTP_AT(enum.Enum):             # <<<<<<<<<<<<<<
 *     O_RDONLY = c_sftp.O_RDONLY
 *     O_WRONLY = c_sftp.O_WRONLY
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SFTP_AT, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SFTP_AT, __pyx_t_4) < 0) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssh/libssh_enums.pyx":1
 * # This file is part of redlibssh.             # <<<<<<<<<<<<<<
 * # Copyright (C) 2022 Red_M
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ssh.libssh_enums", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ssh.libssh_enums");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_kex_types_e(enum ssh_kex_types_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_kex_types_e neg_one = (enum ssh_kex_types_e) -1, const_zero = (enum ssh_kex_types_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_kex_types_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_kex_types_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_kex_types_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_kex_types_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_kex_types_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_kex_types_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_auth_e(enum ssh_auth_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_auth_e neg_one = (enum ssh_auth_e) -1, const_zero = (enum ssh_auth_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_auth_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_auth_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_auth_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_auth_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_auth_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_auth_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_requests_e(enum ssh_requests_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_requests_e neg_one = (enum ssh_requests_e) -1, const_zero = (enum ssh_requests_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_requests_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_requests_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_requests_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_requests_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_requests_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_requests_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_channel_type_e(enum ssh_channel_type_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_channel_type_e neg_one = (enum ssh_channel_type_e) -1, const_zero = (enum ssh_channel_type_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_channel_type_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_channel_type_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_channel_type_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_channel_type_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_channel_type_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_channel_type_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_channel_requests_e(enum ssh_channel_requests_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_channel_requests_e neg_one = (enum ssh_channel_requests_e) -1, const_zero = (enum ssh_channel_requests_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_channel_requests_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_channel_requests_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_channel_requests_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_channel_requests_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_channel_requests_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_channel_requests_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_global_requests_e(enum ssh_global_requests_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_global_requests_e neg_one = (enum ssh_global_requests_e) -1, const_zero = (enum ssh_global_requests_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_global_requests_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_global_requests_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_global_requests_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_global_requests_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_global_requests_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_global_requests_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_publickey_state_e(enum ssh_publickey_state_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_publickey_state_e neg_one = (enum ssh_publickey_state_e) -1, const_zero = (enum ssh_publickey_state_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_publickey_state_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_publickey_state_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_publickey_state_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_publickey_state_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_publickey_state_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_publickey_state_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_server_known_e(enum ssh_server_known_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_server_known_e neg_one = (enum ssh_server_known_e) -1, const_zero = (enum ssh_server_known_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_server_known_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_server_known_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_server_known_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_server_known_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_server_known_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_server_known_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_error_types_e(enum ssh_error_types_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_error_types_e neg_one = (enum ssh_error_types_e) -1, const_zero = (enum ssh_error_types_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_error_types_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_error_types_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_error_types_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_error_types_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_error_types_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_error_types_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_keytypes_e(enum ssh_keytypes_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_keytypes_e neg_one = (enum ssh_keytypes_e) -1, const_zero = (enum ssh_keytypes_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_keytypes_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_keytypes_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_keytypes_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_keytypes_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_keytypes_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_keytypes_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_keycmp_e(enum ssh_keycmp_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_keycmp_e neg_one = (enum ssh_keycmp_e) -1, const_zero = (enum ssh_keycmp_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_keycmp_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_keycmp_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_keycmp_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_keycmp_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_keycmp_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_keycmp_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_options_e(enum ssh_options_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_options_e neg_one = (enum ssh_options_e) -1, const_zero = (enum ssh_options_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_options_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_options_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_options_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_options_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_options_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_options_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_scp_request_types(enum ssh_scp_request_types value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_scp_request_types neg_one = (enum ssh_scp_request_types) -1, const_zero = (enum ssh_scp_request_types) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_scp_request_types) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_scp_request_types) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_scp_request_types) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_scp_request_types) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_scp_request_types) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_scp_request_types),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__ssh_connector_flags_e(enum ssh_connector_flags_e value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum ssh_connector_flags_e neg_one = (enum ssh_connector_flags_e) -1, const_zero = (enum ssh_connector_flags_e) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum ssh_connector_flags_e) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum ssh_connector_flags_e) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_connector_flags_e) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum ssh_connector_flags_e) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum ssh_connector_flags_e) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum ssh_connector_flags_e),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
